@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2018 - 2022 Michael Vetter <jubalh@idoru.org>
+ * Copyright (C) 2018 - 2023 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -85,9 +85,8 @@ void
 chat_log_msg_out(const char* const barejid, const char* const msg, const char* const resource)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        char* mybarejid = connection_get_barejid();
+        auto_char char* mybarejid = connection_get_barejid();
         _chat_log_chat(mybarejid, barejid, msg, PROF_OUT_LOG, NULL, resource);
-        free(mybarejid);
     }
 }
 
@@ -95,15 +94,13 @@ void
 chat_log_otr_msg_out(const char* const barejid, const char* const msg, const char* const resource)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        char* mybarejid = connection_get_barejid();
-        char* pref_otr_log = prefs_get_string(PREF_OTR_LOG);
+        auto_char char* mybarejid = connection_get_barejid();
+        auto_gchar gchar* pref_otr_log = prefs_get_string(PREF_OTR_LOG);
         if (strcmp(pref_otr_log, "on") == 0) {
             _chat_log_chat(mybarejid, barejid, msg, PROF_OUT_LOG, NULL, resource);
         } else if (strcmp(pref_otr_log, "redact") == 0) {
             _chat_log_chat(mybarejid, barejid, "[redacted]", PROF_OUT_LOG, NULL, resource);
         }
-        g_free(pref_otr_log);
-        free(mybarejid);
     }
 }
 
@@ -111,15 +108,13 @@ void
 chat_log_pgp_msg_out(const char* const barejid, const char* const msg, const char* const resource)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        char* mybarejid = connection_get_barejid();
-        char* pref_pgp_log = prefs_get_string(PREF_PGP_LOG);
+        auto_char char* mybarejid = connection_get_barejid();
+        auto_gchar gchar* pref_pgp_log = prefs_get_string(PREF_PGP_LOG);
         if (strcmp(pref_pgp_log, "on") == 0) {
             _chat_log_chat(mybarejid, barejid, msg, PROF_OUT_LOG, NULL, resource);
         } else if (strcmp(pref_pgp_log, "redact") == 0) {
             _chat_log_chat(mybarejid, barejid, "[redacted]", PROF_OUT_LOG, NULL, resource);
         }
-        g_free(pref_pgp_log);
-        free(mybarejid);
     }
 }
 
@@ -127,15 +122,13 @@ void
 chat_log_omemo_msg_out(const char* const barejid, const char* const msg, const char* const resource)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        char* mybarejid = connection_get_barejid();
-        char* pref_omemo_log = prefs_get_string(PREF_OMEMO_LOG);
+        auto_char char* mybarejid = connection_get_barejid();
+        auto_gchar gchar* pref_omemo_log = prefs_get_string(PREF_OMEMO_LOG);
         if (strcmp(pref_omemo_log, "on") == 0) {
             _chat_log_chat(mybarejid, barejid, msg, PROF_OUT_LOG, NULL, resource);
         } else if (strcmp(pref_omemo_log, "redact") == 0) {
             _chat_log_chat(mybarejid, barejid, "[redacted]", PROF_OUT_LOG, NULL, resource);
         }
-        g_free(pref_omemo_log);
-        free(mybarejid);
     }
 }
 
@@ -143,8 +136,8 @@ void
 chat_log_otr_msg_in(ProfMessage* message)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        char* mybarejid = connection_get_barejid();
-        char* pref_otr_log = prefs_get_string(PREF_OTR_LOG);
+        auto_char char* mybarejid = connection_get_barejid();
+        auto_gchar gchar* pref_otr_log = prefs_get_string(PREF_OTR_LOG);
         if (message->enc == PROF_MSG_ENC_NONE || (strcmp(pref_otr_log, "on") == 0)) {
             if (message->type == PROF_MSG_TYPE_MUCPM) {
                 _chat_log_chat(mybarejid, message->from_jid->barejid, message->plain, PROF_IN_LOG, message->timestamp, message->from_jid->resourcepart);
@@ -158,8 +151,6 @@ chat_log_otr_msg_in(ProfMessage* message)
                 _chat_log_chat(mybarejid, message->from_jid->barejid, "[redacted]", PROF_IN_LOG, message->timestamp, NULL);
             }
         }
-        g_free(pref_otr_log);
-        free(mybarejid);
     }
 }
 
@@ -167,8 +158,8 @@ void
 chat_log_pgp_msg_in(ProfMessage* message)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        char* mybarejid = connection_get_barejid();
-        char* pref_pgp_log = prefs_get_string(PREF_PGP_LOG);
+        auto_char char* mybarejid = connection_get_barejid();
+        auto_gchar gchar* pref_pgp_log = prefs_get_string(PREF_PGP_LOG);
         if (strcmp(pref_pgp_log, "on") == 0) {
             if (message->type == PROF_MSG_TYPE_MUCPM) {
                 _chat_log_chat(mybarejid, message->from_jid->barejid, message->plain, PROF_IN_LOG, message->timestamp, message->from_jid->resourcepart);
@@ -182,8 +173,6 @@ chat_log_pgp_msg_in(ProfMessage* message)
                 _chat_log_chat(mybarejid, message->from_jid->barejid, "[redacted]", PROF_IN_LOG, message->timestamp, NULL);
             }
         }
-        g_free(pref_pgp_log);
-        free(mybarejid);
     }
 }
 
@@ -191,8 +180,8 @@ void
 chat_log_omemo_msg_in(ProfMessage* message)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        char* mybarejid = connection_get_barejid();
-        char* pref_omemo_log = prefs_get_string(PREF_OMEMO_LOG);
+        auto_char char* mybarejid = connection_get_barejid();
+        auto_gchar gchar* pref_omemo_log = prefs_get_string(PREF_OMEMO_LOG);
         if (strcmp(pref_omemo_log, "on") == 0) {
             if (message->type == PROF_MSG_TYPE_MUCPM) {
                 _chat_log_chat(mybarejid, message->from_jid->barejid, message->plain, PROF_IN_LOG, message->timestamp, message->from_jid->resourcepart);
@@ -206,8 +195,6 @@ chat_log_omemo_msg_in(ProfMessage* message)
                 _chat_log_chat(mybarejid, message->from_jid->barejid, "[redacted]", PROF_IN_LOG, message->timestamp, message->from_jid->resourcepart);
             }
         }
-        g_free(pref_omemo_log);
-        free(mybarejid);
     }
 }
 
@@ -215,22 +202,24 @@ void
 chat_log_msg_in(ProfMessage* message)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        char* mybarejid = connection_get_barejid();
+        auto_char char* mybarejid = connection_get_barejid();
 
         if (message->type == PROF_MSG_TYPE_MUCPM) {
             _chat_log_chat(mybarejid, message->from_jid->barejid, message->plain, PROF_IN_LOG, message->timestamp, message->from_jid->resourcepart);
         } else {
             _chat_log_chat(mybarejid, message->from_jid->barejid, message->plain, PROF_IN_LOG, message->timestamp, NULL);
         }
-
-        free(mybarejid);
     }
 }
 
 static void
-_chat_log_chat(const char* const login, const char* const other, const char* const msg,
+_chat_log_chat(const char* const login, const char* const other, const char* msg,
                chat_log_direction_t direction, GDateTime* timestamp, const char* const resourcepart)
 {
+    auto_gchar gchar* pref_dblog = prefs_get_string(PREF_DBLOG);
+    if (g_strcmp0(pref_dblog, "redact") == 0) {
+        msg = "[REDACTED]";
+    }
     char* other_name;
     GString* other_str = NULL;
 
@@ -272,7 +261,7 @@ _chat_log_chat(const char* const login, const char* const other, const char* con
         g_date_time_ref(timestamp);
     }
 
-    gchar* date_fmt = g_date_time_format_iso8601(timestamp);
+    auto_gchar gchar* date_fmt = g_date_time_format_iso8601(timestamp);
     FILE* chatlogp = fopen(dated_log->filename, "a");
     g_chmod(dated_log->filename, S_IRUSR | S_IWUSR);
     if (chatlogp) {
@@ -304,7 +293,6 @@ _chat_log_chat(const char* const login, const char* const other, const char* con
         }
     }
 
-    g_free(date_fmt);
     g_date_time_unref(timestamp);
 }
 
@@ -320,10 +308,9 @@ void
 groupchat_log_msg_out(const gchar* const room, const gchar* const msg)
 {
     if (prefs_get_boolean(PREF_GRLOG)) {
-        char* mybarejid = connection_get_barejid();
+        auto_char char* mybarejid = connection_get_barejid();
         char* mynick = muc_nick(room);
         _groupchat_log_chat(mybarejid, room, mynick, msg);
-        free(mybarejid);
     }
 }
 
@@ -331,9 +318,8 @@ void
 groupchat_log_msg_in(const gchar* const room, const gchar* const nick, const gchar* const msg)
 {
     if (prefs_get_boolean(PREF_GRLOG)) {
-        char* mybarejid = connection_get_barejid();
+        auto_char char* mybarejid = connection_get_barejid();
         _groupchat_log_chat(mybarejid, room, nick, msg);
-        free(mybarejid);
     }
 }
 
@@ -341,8 +327,8 @@ void
 groupchat_log_omemo_msg_out(const gchar* const room, const gchar* const msg)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        char* mybarejid = connection_get_barejid();
-        char* pref_omemo_log = prefs_get_string(PREF_OMEMO_LOG);
+        auto_char char* mybarejid = connection_get_barejid();
+        auto_gchar gchar* pref_omemo_log = prefs_get_string(PREF_OMEMO_LOG);
         char* mynick = muc_nick(room);
 
         if (strcmp(pref_omemo_log, "on") == 0) {
@@ -350,9 +336,6 @@ groupchat_log_omemo_msg_out(const gchar* const room, const gchar* const msg)
         } else if (strcmp(pref_omemo_log, "redact") == 0) {
             _groupchat_log_chat(mybarejid, room, mynick, "[redacted]");
         }
-
-        g_free(pref_omemo_log);
-        free(mybarejid);
     }
 }
 
@@ -360,17 +343,14 @@ void
 groupchat_log_omemo_msg_in(const gchar* const room, const gchar* const nick, const gchar* const msg)
 {
     if (prefs_get_boolean(PREF_CHLOG)) {
-        char* mybarejid = connection_get_barejid();
-        char* pref_omemo_log = prefs_get_string(PREF_OMEMO_LOG);
+        auto_char char* mybarejid = connection_get_barejid();
+        auto_gchar gchar* pref_omemo_log = prefs_get_string(PREF_OMEMO_LOG);
 
         if (strcmp(pref_omemo_log, "on") == 0) {
             _groupchat_log_chat(mybarejid, room, nick, msg);
         } else if (strcmp(pref_omemo_log, "redact") == 0) {
             _groupchat_log_chat(mybarejid, room, nick, "[redacted]");
         }
-
-        g_free(pref_omemo_log);
-        free(mybarejid);
     }
 }
 
@@ -393,7 +373,7 @@ _groupchat_log_chat(const gchar* const login, const gchar* const room, const gch
 
     GDateTime* dt_tmp = g_date_time_new_now_local();
 
-    gchar* date_fmt = g_date_time_format_iso8601(dt_tmp);
+    auto_gchar gchar* date_fmt = g_date_time_format_iso8601(dt_tmp);
 
     FILE* grpchatlogp = fopen(dated_log->filename, "a");
     g_chmod(dated_log->filename, S_IRUSR | S_IWUSR);
@@ -411,7 +391,6 @@ _groupchat_log_chat(const gchar* const login, const gchar* const room, const gch
         }
     }
 
-    g_free(date_fmt);
     g_date_time_unref(dt_tmp);
 }
 
@@ -425,21 +404,16 @@ chat_log_close(void)
 static char*
 _get_log_filename(const char* const other, const char* const login, GDateTime* dt, gboolean is_room)
 {
-    gchar* chatlogs_dir = files_file_in_account_data_path(DIR_CHATLOGS, login, is_room ? "rooms" : NULL);
-    gchar* logfile_name = g_date_time_format(dt, "%Y_%m_%d.log");
-    gchar* other_ = str_replace(other, "@", "_at_");
-    gchar* logs_path = g_strdup_printf("%s/%s", chatlogs_dir, other_);
+    auto_gchar gchar* chatlogs_dir = files_file_in_account_data_path(DIR_CHATLOGS, login, is_room ? "rooms" : NULL);
+    auto_gchar gchar* logfile_name = g_date_time_format(dt, "%Y_%m_%d.log");
+    auto_char char* other_ = str_replace(other, "@", "_at_");
+    auto_gchar gchar* logs_path = g_strdup_printf("%s/%s", chatlogs_dir, other_);
     gchar* logfile_path = NULL;
 
     if (create_dir(logs_path)) {
         logfile_path = g_strdup_printf("%s/%s", logs_path, logfile_name);
     }
 
-    g_free(logs_path);
-    g_free(other_);
-    g_free(logfile_name);
-    g_free(chatlogs_dir);
-
     return logfile_path;
 }
 
@@ -447,14 +421,12 @@ static struct dated_chat_log*
 _create_chatlog(const char* const other, const char* const login)
 {
     GDateTime* now = g_date_time_new_now_local();
-    char* filename = _get_log_filename(other, login, now, FALSE);
+    auto_char char* filename = _get_log_filename(other, login, now, FALSE);
 
     struct dated_chat_log* new_log = malloc(sizeof(struct dated_chat_log));
     new_log->filename = strdup(filename);
     new_log->date = now;
 
-    free(filename);
-
     return new_log;
 }
 
@@ -462,14 +434,12 @@ static struct dated_chat_log*
 _create_groupchat_log(const char* const room, const char* const login)
 {
     GDateTime* now = g_date_time_new_now_local();
-    char* filename = _get_log_filename(room, login, now, TRUE);
+    auto_char char* filename = _get_log_filename(room, login, now, TRUE);
 
     struct dated_chat_log* new_log = malloc(sizeof(struct dated_chat_log));
     new_log->filename = strdup(filename);
     new_log->date = now;
 
-    free(filename);
-
     return new_log;
 }
 
