@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2022 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -62,6 +62,10 @@
 #include "log.h"
 #include "common.h"
 
+#ifdef HAVE_GIT_VERSION
+#include "gitversion.h"
+#endif
+
 struct curl_data_t
 {
     char* buffer;
@@ -70,6 +74,65 @@ struct curl_data_t
 
 static size_t _data_callback(void* ptr, size_t size, size_t nmemb, void* data);
 
+/**
+ * Frees the memory allocated for a gchar* string.
+ *
+ * @param str Pointer to the gchar* string to be freed. If NULL, no action is taken.
+ */
+void
+auto_free_gchar(gchar** str)
+{
+    if (str == NULL)
+        return;
+    g_free(*str);
+}
+
+/**
+ * Frees the memory allocated for a guchar* string.
+ *
+ * @param str Pointer to the guchar* string to be freed. If NULL, no action is taken.
+ */
+void
+auto_free_guchar(guchar** ptr)
+{
+    if (ptr == NULL) {
+        return;
+    }
+    g_free(*ptr);
+}
+
+/**
+ * Frees the memory allocated for a gchar** string array.
+ *
+ * @param args Pointer to the gchar** string array to be freed. If NULL, no action is taken.
+ */
+void
+auto_free_gcharv(gchar*** args)
+{
+    if (args == NULL)
+        return;
+    g_strfreev(*args);
+}
+
+/**
+ * Frees the memory allocated for a char* string.
+ *
+ * @param str Pointer to the char* string to be freed. If NULL, no action is taken.
+ */
+void
+auto_free_char(char** str)
+{
+    if (str == NULL)
+        return;
+    free(*str);
+}
+
+gboolean
+string_to_verbosity(const char* cmd, int* verbosity, gchar** err_msg)
+{
+    return strtoi_range(cmd, verbosity, 0, 3, err_msg);
+}
+
 gboolean
 create_dir(const char* name)
 {
@@ -95,6 +158,18 @@ copy_file(const char* const sourcepath, const char* const targetpath, const gboo
     return success;
 }
 
+/**
+ * @brief Replaces all occurrences of a substring with a replacement in a string.
+ *
+ * @param string      The input string to perform replacement on.
+ * @param substr      The substring to be replaced.
+ * @param replacement The replacement string.
+ * @return            The modified string with replacements, or NULL on failure.
+ *
+ * @note Remember to free returned value using `auto_char` or `free()` when it is no longer needed.
+ * @note If 'string' is NULL, the function returns NULL.
+ * @note If 'substr' or 'replacement' is NULL or an empty string, a duplicate of 'string' is returned.
+ */
 char*
 str_replace(const char* string, const char* substr,
             const char* replacement)
@@ -113,11 +188,10 @@ str_replace(const char* string, const char* substr,
     head = newstr;
 
     while ((tok = strstr(head, substr))) {
-        char* oldstr = newstr;
+        auto_char char* oldstr = newstr;
         newstr = malloc(strlen(oldstr) - strlen(substr) + strlen(replacement) + 1);
 
         if (newstr == NULL) {
-            free(oldstr);
             return NULL;
         }
 
@@ -129,31 +203,30 @@ str_replace(const char* string, const char* substr,
         memset(newstr + strlen(oldstr) - strlen(substr) + strlen(replacement), 0, 1);
 
         head = newstr + (tok - oldstr) + strlen(replacement);
-        free(oldstr);
     }
 
     return newstr;
 }
 
 gboolean
-strtoi_range(char* str, int* saveptr, int min, int max, char** err_msg)
+strtoi_range(const char* str, int* saveptr, int min, int max, gchar** err_msg)
 {
     char* ptr;
     int val;
-
+    if (str == NULL) {
+        if (err_msg)
+            *err_msg = g_strdup_printf("'str' input pointer can not be NULL");
+        return FALSE;
+    }
     errno = 0;
     val = (int)strtol(str, &ptr, 0);
     if (errno != 0 || *str == '\0' || *ptr != '\0') {
-        GString* err_str = g_string_new("");
-        g_string_printf(err_str, "Could not convert \"%s\" to a number.", str);
-        *err_msg = err_str->str;
-        g_string_free(err_str, FALSE);
+        if (err_msg)
+            *err_msg = g_strdup_printf("Could not convert \"%s\" to a number.", str);
         return FALSE;
     } else if (val < min || val > max) {
-        GString* err_str = g_string_new("");
-        g_string_printf(err_str, "Value %s out of range. Must be in %d..%d.", str, min, max);
-        *err_msg = err_str->str;
-        g_string_free(err_str, FALSE);
+        if (err_msg)
+            *err_msg = g_strdup_printf("Value %s out of range. Must be in %d..%d.", str, min, max);
         return FALSE;
     }
 
@@ -252,10 +325,10 @@ _data_callback(void* ptr, size_t size, size_t nmemb, void* data)
     return realsize;
 }
 
-char*
-get_file_or_linked(char* loc, char* basedir)
+gchar*
+get_file_or_linked(gchar* loc, gchar* basedir)
 {
-    char* true_loc = NULL;
+    gchar* true_loc = NULL;
 
     // check for symlink
     if (g_file_test(loc, G_FILE_TEST_IS_SYMLINK)) {
@@ -263,16 +336,13 @@ get_file_or_linked(char* loc, char* basedir)
 
         // if relative, add basedir
         if (!g_str_has_prefix(true_loc, "/") && !g_str_has_prefix(true_loc, "~")) {
-            GString* base_str = g_string_new(basedir);
-            g_string_append(base_str, "/");
-            g_string_append(base_str, true_loc);
-            free(true_loc);
-            true_loc = base_str->str;
-            g_string_free(base_str, FALSE);
+            gchar* tmp = g_strdup_printf("%s/%s", basedir, true_loc);
+            g_free(true_loc);
+            true_loc = tmp;
         }
         // use given location
     } else {
-        true_loc = strdup(loc);
+        true_loc = g_strdup(loc);
     }
 
     return true_loc;
@@ -325,33 +395,32 @@ prof_occurrences(const char* const needle, const char* const haystack, int offse
         return *result;
     }
 
-    gchar* haystack_curr = g_utf8_offset_to_pointer(haystack, offset);
-    if (g_str_has_prefix(haystack_curr, needle)) {
-        if (whole_word) {
-            gunichar before = 0;
-            gchar* haystack_before_ch = g_utf8_find_prev_char(haystack, haystack_curr);
-            if (haystack_before_ch) {
-                before = g_utf8_get_char(haystack_before_ch);
-            }
+    do {
+        gchar* haystack_curr = g_utf8_offset_to_pointer(haystack, offset);
+        if (g_str_has_prefix(haystack_curr, needle)) {
+            if (whole_word) {
+                gunichar before = 0;
+                gchar* haystack_before_ch = g_utf8_find_prev_char(haystack, haystack_curr);
+                if (haystack_before_ch) {
+                    before = g_utf8_get_char(haystack_before_ch);
+                }
 
-            gunichar after = 0;
-            gchar* haystack_after_ch = haystack_curr + strlen(needle);
-            if (haystack_after_ch[0] != '\0') {
-                after = g_utf8_get_char(haystack_after_ch);
-            }
+                gunichar after = 0;
+                gchar* haystack_after_ch = haystack_curr + strlen(needle);
+                if (haystack_after_ch[0] != '\0') {
+                    after = g_utf8_get_char(haystack_after_ch);
+                }
 
-            if (!g_unichar_isalnum(before) && !g_unichar_isalnum(after)) {
+                if (!g_unichar_isalnum(before) && !g_unichar_isalnum(after)) {
+                    *result = g_slist_append(*result, GINT_TO_POINTER(offset));
+                }
+            } else {
                 *result = g_slist_append(*result, GINT_TO_POINTER(offset));
             }
-        } else {
-            *result = g_slist_append(*result, GINT_TO_POINTER(offset));
         }
-    }
 
-    offset++;
-    if (g_strcmp0(g_utf8_offset_to_pointer(haystack, offset), "\0") != 0) {
-        *result = prof_occurrences(needle, haystack, offset, whole_word, result);
-    }
+        offset++;
+    } while (g_strcmp0(g_utf8_offset_to_pointer(haystack, offset), "\0") != 0);
 
     return *result;
 }
@@ -389,20 +458,20 @@ get_file_paths_recursive(const char* path, GSList** contents)
 
     GDir* directory = g_dir_open(path, 0, NULL);
     const gchar* entry = g_dir_read_name(directory);
+    gchar* full;
     while (entry) {
-        GString* full = g_string_new(path);
-        if (!g_str_has_suffix(full->str, "/")) {
-            g_string_append(full, "/");
+        if (g_str_has_suffix(path, "/")) {
+            full = g_strdup_printf("%s%s", path, entry);
+        } else {
+            full = g_strdup_printf("%s/%s", path, entry);
         }
-        g_string_append(full, entry);
 
-        if (is_dir(full->str)) {
-            get_file_paths_recursive(full->str, contents);
-        } else if (is_regular_file(full->str)) {
-            *contents = g_slist_append(*contents, full->str);
+        if (is_dir(full)) {
+            get_file_paths_recursive(full, contents);
+        } else if (is_regular_file(full)) {
+            *contents = g_slist_append(*contents, full);
         }
 
-        g_string_free(full, FALSE);
         entry = g_dir_read_name(directory);
     }
 }
@@ -431,14 +500,11 @@ GSList*
 get_mentions(gboolean whole_word, gboolean case_sensitive, const char* const message, const char* const nick)
 {
     GSList* mentions = NULL;
-    gchar* message_search = case_sensitive ? g_strdup(message) : g_utf8_strdown(message, -1);
-    gchar* mynick_search = case_sensitive ? g_strdup(nick) : g_utf8_strdown(nick, -1);
+    auto_gchar gchar* message_search = case_sensitive ? g_strdup(message) : g_utf8_strdown(message, -1);
+    auto_gchar gchar* mynick_search = case_sensitive ? g_strdup(nick) : g_utf8_strdown(nick, -1);
 
     mentions = prof_occurrences(mynick_search, message_search, 0, whole_word, &mentions);
 
-    g_free(message_search);
-    g_free(mynick_search);
-
     return mentions;
 }
 
@@ -457,16 +523,27 @@ call_external(gchar** argv)
                                   NULL, NULL, NULL,
                                   &spawn_error);
     if (!is_successful) {
-        gchar* cmd = g_strjoinv(" ", argv);
+        auto_gchar gchar* cmd = g_strjoinv(" ", argv);
         log_error("Spawning '%s' failed with error '%s'", cmd, spawn_error ? spawn_error->message : "Unknown, spawn_error is NULL");
 
         g_error_free(spawn_error);
-        g_free(cmd);
     }
 
     return is_successful;
 }
 
+/**
+ * @brief Formats an argument vector for calling an external command with placeholders.
+ *
+ * This function constructs an argument vector (argv) based on the provided template string, replacing placeholders ("%u" and "%p") with the provided URL and filename, respectively.
+ *
+ * @param template  The template string with placeholders.
+ * @param url       The URL to replace "%u" (or NULL to skip).
+ * @param filename  The filename to replace "%p" (or NULL to skip).
+ * @return          The constructed argument vector (argv) as a null-terminated array of strings.
+ *
+ * @note Remember to free the returned argument vector using `auto_gcharv` or `g_strfreev()`.
+ */
 gchar**
 format_call_external_argv(const char* template, const char* url, const char* filename)
 {
@@ -519,49 +596,64 @@ _has_directory_suffix(const char* path)
             || g_str_has_suffix(path, G_DIR_SEPARATOR_S));
 }
 
+/**
+ * @brief Extracts the basename from a given URL.
+ *
+ * @param url The URL to extract the basename from.
+ * @return The extracted basename or a default name ("index") if the basename has a directory suffix.
+ *
+ * @note The returned string must be freed by the caller using `auto_free` or `free()`.
+ */
 char*
-_basename_from_url(const char* url)
+basename_from_url(const char* url)
 {
     const char* default_name = "index";
 
     GFile* file = g_file_new_for_commandline_arg(url);
-    char* basename = g_file_get_basename(file);
-
-    if (_has_directory_suffix(basename)) {
-        g_free(basename);
-        basename = strdup(default_name);
-    }
-
+    auto_gchar gchar* basename = g_file_get_basename(file);
     g_object_unref(file);
 
-    return basename;
+    if (_has_directory_suffix(basename)) {
+        return strdup(default_name);
+    }
+
+    return strdup(basename);
 }
 
+/**
+ * Expands the provided path by resolving special characters like '~' and 'file://'.
+ *
+ * @param path The path to expand.
+ * @return The expanded path.
+ *
+ * @note Remember to free the returned value using `auto_gchar` or `g_free()`.
+ */
 gchar*
 get_expanded_path(const char* path)
 {
-    GString* exp_path = g_string_new("");
-    gchar* result;
-
     if (g_str_has_prefix(path, "file://")) {
         path += strlen("file://");
     }
     if (strlen(path) >= 2 && path[0] == '~' && path[1] == '/') {
-        g_string_printf(exp_path, "%s/%s", getenv("HOME"), path + 2);
+        return g_strdup_printf("%s/%s", getenv("HOME"), path + 2);
     } else {
-        g_string_printf(exp_path, "%s", path);
+        return g_strdup_printf("%s", path);
     }
-
-    result = exp_path->str;
-    g_string_free(exp_path, FALSE);
-
-    return result;
 }
 
+/**
+ * Generates a unique filename based on the provided URL and path.
+ *
+ * @param url The URL to derive the basename from.
+ * @param path The path to prepend to the generated filename. If NULL, the current directory is used.
+ * @return A unique filename combining the path and derived basename from the URL.
+ *
+ * @note Remember to free the returned value using `auto_gchar` or `g_free()`.
+ */
 gchar*
 unique_filename_from_url(const char* url, const char* path)
 {
-    gchar* realpath;
+    auto_gchar gchar* realpath = NULL;
 
     // Default to './' as path when none has been provided.
     if (path == NULL) {
@@ -572,14 +664,13 @@ unique_filename_from_url(const char* url, const char* path)
 
     // Resolves paths such as './../.' for path.
     GFile* target = g_file_new_for_commandline_arg(realpath);
-    gchar* filename = NULL;
+    auto_gchar gchar* filename = NULL;
 
     if (_has_directory_suffix(realpath) || g_file_test(realpath, G_FILE_TEST_IS_DIR)) {
         // The target should be used as a directory. Assume that the basename
         // should be derived from the URL.
-        char* basename = _basename_from_url(url);
+        auto_char char* basename = basename_from_url(url);
         filename = g_build_filename(g_file_peek_path(target), basename, NULL);
-        g_free(basename);
     } else {
         // Just use the target as filename.
         filename = g_build_filename(g_file_peek_path(target), NULL);
@@ -587,14 +678,34 @@ unique_filename_from_url(const char* url, const char* path)
 
     gchar* unique_filename = _unique_filename(filename);
     if (unique_filename == NULL) {
-        g_free(filename);
-        g_free(realpath);
         return NULL;
     }
 
     g_object_unref(target);
-    g_free(filename);
-    g_free(realpath);
 
     return unique_filename;
 }
+
+void
+glib_hash_table_free(GHashTable* hash_table)
+{
+    g_hash_table_remove_all(hash_table);
+    g_hash_table_unref(hash_table);
+}
+
+/* build profanity version string.
+ * example: 0.13.1dev.master.69d8c1f9
+ */
+gchar*
+prof_get_version(void)
+{
+    if (strcmp(PACKAGE_STATUS, "development") == 0) {
+#ifdef HAVE_GIT_VERSION
+        return g_strdup_printf("%sdev.%s.%s", PACKAGE_VERSION, PROF_GIT_BRANCH, PROF_GIT_REVISION);
+#else
+        return g_strdup_printf("%sdev", PACKAGE_VERSION);
+#endif
+    } else {
+        return g_strdup_printf("%s", PACKAGE_VERSION);
+    }
+}
