@@ -31,6 +31,9 @@
  * statement from your version. If you delete this exception statement from all
  * source files in the program, then also delete it here.
  *
+ * @file common.h
+ *
+ * @brief Common utilities for the project.
  */
 
 #ifndef COMMON_H
@@ -47,6 +50,69 @@
 
 #define ARRAY_SIZE(x) (sizeof(x) / sizeof(x[0]))
 
+#define PROF_STRINGIFY_(n) #n
+#define PROF_STRINGIFY(n)  PROF_STRINGIFY_(n)
+
+void auto_free_gchar(gchar** str);
+
+/**
+ * Macro for automatically freeing a gchar* string when it goes out of scope.
+ *
+ * Example usage:
+ * ```
+ * auto_gchar gchar* myString = g_strdup("Hello, world!");
+ * ```
+ */
+#define auto_gchar __attribute__((__cleanup__(auto_free_gchar)))
+
+void auto_free_gcharv(gchar*** args);
+
+/**
+ * Macro for automatically freeing a gchar** string array when it goes out of scope.
+ *
+ * Example usage:
+ * ```
+ * auto_gcharv gchar** stringArray = g_strsplit("Hello, world!", " ", -1);
+ * ```
+ */
+#define auto_gcharv __attribute__((__cleanup__(auto_free_gcharv)))
+
+void auto_free_char(char** str);
+
+/**
+ * Macro for automatically freeing a char* string when it goes out of scope.
+ *
+ * Example usage:
+ * ```
+ * auto_char char* myString = strdup("Hello, world!");
+ * ```
+ */
+#define auto_char __attribute__((__cleanup__(auto_free_char)))
+
+void auto_free_guchar(guchar** str);
+
+/**
+ * @brief Macro for automatically freeing a guchar* string when it goes out of scope.
+ *
+ * This macro is used to automatically free a guchar* string when it goes out of scope.
+ * It utilizes the `auto_free_guchar` function and should be placed before the variable declaration.
+ *
+ * Example usage:
+ * ```
+ * auto_guchar guchar* myString = g_base64_decode("SGVsbG8sIHdvcmxkIQ==", NULL);
+ * ```
+ */
+#define auto_guchar __attribute__((__cleanup__(auto_free_guchar)))
+
+#if defined(__OpenBSD__)
+#define STR_MAYBE_NULL(p) (p) ?: "(null)"
+#else
+#define STR_MAYBE_NULL(p) (p)
+#endif
+
+/* Our own define of MB_CUR_MAX but this time at compile time */
+#define PROF_MB_CUR_MAX 8
+
 // assume malloc stores at most 8 bytes for size of allocated memory
 // and page size is at least 4KB
 #define READ_BUF_SIZE 4088
@@ -80,16 +146,18 @@ typedef enum {
     RESOURCE_XA
 } resource_presence_t;
 
+gboolean string_to_verbosity(const char* cmd, int* verbosity, gchar** err_msg);
+
 gboolean create_dir(const char* name);
 gboolean copy_file(const char* const src, const char* const target, const gboolean overwrite_existing);
 char* str_replace(const char* string, const char* substr, const char* replacement);
-gboolean strtoi_range(char* str, int* saveptr, int min, int max, char** err_msg);
+gboolean strtoi_range(const char* str, int* saveptr, int min, int max, char** err_msg);
 int utf8_display_len(const char* const str);
 
 char* release_get_latest(void);
 gboolean release_is_new(char* found_version);
 
-char* get_file_or_linked(char* loc, char* basedir);
+gchar* get_file_or_linked(gchar* loc, gchar* basedir);
 char* strip_arg_quotes(const char* const input);
 gboolean is_notify_enabled(void);
 
@@ -109,4 +177,9 @@ gchar** format_call_external_argv(const char* template, const char* url, const c
 gchar* unique_filename_from_url(const char* url, const char* path);
 gchar* get_expanded_path(const char* path);
 
+void glib_hash_table_free(GHashTable* hash_table);
+char* basename_from_url(const char* url);
+
+gchar* prof_get_version(void);
+
 #endif
