@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2022 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -53,7 +53,7 @@
 #include "xmpp/xmpp.h"
 #include "xmpp/jid.h"
 
-static char* accounts_loc;
+static gchar* accounts_loc;
 static GKeyFile* accounts;
 
 static Autocomplete all_ac;
@@ -78,7 +78,7 @@ accounts_load(void)
 
     // create the logins searchable list for autocompletion
     gsize naccounts;
-    gchar** account_names = g_key_file_get_groups(accounts, &naccounts);
+    auto_gcharv gchar** account_names = g_key_file_get_groups(accounts, &naccounts);
 
     for (gsize i = 0; i < naccounts; i++) {
         autocomplete_add(all_ac, account_names[i]);
@@ -86,8 +86,6 @@ accounts_load(void)
             autocomplete_add(enabled_ac, account_names[i]);
         }
     }
-
-    g_strfreev(account_names);
 }
 
 void
@@ -127,8 +125,8 @@ accounts_add(const char* account_name, const char* altdomain, const int port, co
 {
     // set account name and resource
     const char* barejid = account_name;
-    char* resource = jid_random_resource();
-    Jid* jid = jid_create(account_name);
+    auto_gchar gchar* resource = jid_random_resource();
+    auto_jid Jid* jid = jid_create(account_name);
     if (jid) {
         barejid = jid->barejid;
         if (jid->resourcepart) {
@@ -137,8 +135,6 @@ accounts_add(const char* account_name, const char* altdomain, const int port, co
     }
 
     if (g_key_file_has_group(accounts, account_name)) {
-        g_free(resource);
-        jid_destroy(jid);
         return;
     }
 
@@ -158,7 +154,7 @@ accounts_add(const char* account_name, const char* altdomain, const int port, co
         g_key_file_set_string(accounts, account_name, "auth.policy", auth_policy);
     }
 
-    Jid* jidp = jid_create(barejid);
+    auto_jid Jid* jidp = jid_create(barejid);
 
     if (jidp->localpart == NULL) {
         g_key_file_set_string(accounts, account_name, "muc.nick", jidp->domainpart);
@@ -166,8 +162,6 @@ accounts_add(const char* account_name, const char* altdomain, const int port, co
         g_key_file_set_string(accounts, account_name, "muc.nick", jidp->localpart);
     }
 
-    jid_destroy(jidp);
-
     g_key_file_set_string(accounts, account_name, "presence.last", "online");
     g_key_file_set_string(accounts, account_name, "presence.login", "online");
     g_key_file_set_integer(accounts, account_name, "priority.online", 0);
@@ -179,9 +173,6 @@ accounts_add(const char* account_name, const char* altdomain, const int port, co
     _save_accounts();
     autocomplete_add(all_ac, account_name);
     autocomplete_add(enabled_ac, account_name);
-
-    jid_destroy(jid);
-    g_free(resource);
 }
 
 int
@@ -252,30 +243,27 @@ accounts_get_account(const char* const name)
 
         gsize length;
         GList* otr_manual = NULL;
-        gchar** manual = g_key_file_get_string_list(accounts, name, "otr.manual", &length, NULL);
+        auto_gcharv gchar** manual = g_key_file_get_string_list(accounts, name, "otr.manual", &length, NULL);
         if (manual) {
             for (int i = 0; i < length; i++) {
                 otr_manual = g_list_append(otr_manual, strdup(manual[i]));
             }
-            g_strfreev(manual);
         }
 
         GList* otr_opportunistic = NULL;
-        gchar** opportunistic = g_key_file_get_string_list(accounts, name, "otr.opportunistic", &length, NULL);
+        auto_gcharv gchar** opportunistic = g_key_file_get_string_list(accounts, name, "otr.opportunistic", &length, NULL);
         if (opportunistic) {
             for (int i = 0; i < length; i++) {
                 otr_opportunistic = g_list_append(otr_opportunistic, strdup(opportunistic[i]));
             }
-            g_strfreev(opportunistic);
         }
 
         GList* otr_always = NULL;
-        gchar** always = g_key_file_get_string_list(accounts, name, "otr.always", &length, NULL);
+        auto_gcharv gchar** always = g_key_file_get_string_list(accounts, name, "otr.always", &length, NULL);
         if (always) {
             for (int i = 0; i < length; i++) {
                 otr_always = g_list_append(otr_always, strdup(always[i]));
             }
-            g_strfreev(always);
         }
 
         gchar* omemo_policy = NULL;
@@ -284,39 +272,35 @@ accounts_get_account(const char* const name)
         }
 
         GList* omemo_enabled = NULL;
-        gchar** omemo_enabled_list = g_key_file_get_string_list(accounts, name, "omemo.enabled", &length, NULL);
+        auto_gcharv gchar** omemo_enabled_list = g_key_file_get_string_list(accounts, name, "omemo.enabled", &length, NULL);
         if (omemo_enabled_list) {
             for (int i = 0; i < length; i++) {
                 omemo_enabled = g_list_append(omemo_enabled, strdup(omemo_enabled_list[i]));
             }
-            g_strfreev(omemo_enabled_list);
         }
 
         GList* omemo_disabled = NULL;
-        gchar** omemo_disabled_list = g_key_file_get_string_list(accounts, name, "omemo.disabled", &length, NULL);
+        auto_gcharv gchar** omemo_disabled_list = g_key_file_get_string_list(accounts, name, "omemo.disabled", &length, NULL);
         if (omemo_disabled_list) {
             for (int i = 0; i < length; i++) {
                 omemo_disabled = g_list_append(omemo_disabled, strdup(omemo_disabled_list[i]));
             }
-            g_strfreev(omemo_disabled_list);
         }
 
         GList* ox_enabled = NULL;
-        gchar** ox_enabled_list = g_key_file_get_string_list(accounts, name, "ox.enabled", &length, NULL);
+        auto_gcharv gchar** ox_enabled_list = g_key_file_get_string_list(accounts, name, "ox.enabled", &length, NULL);
         if (ox_enabled_list) {
             for (int i = 0; i < length; i++) {
                 ox_enabled = g_list_append(ox_enabled, strdup(ox_enabled_list[i]));
             }
-            g_strfreev(ox_enabled_list);
         }
 
         GList* pgp_enabled = NULL;
-        gchar** pgp_enabled_list = g_key_file_get_string_list(accounts, name, "pgp.enabled", &length, NULL);
+        auto_gcharv gchar** pgp_enabled_list = g_key_file_get_string_list(accounts, name, "pgp.enabled", &length, NULL);
         if (pgp_enabled_list) {
             for (int i = 0; i < length; i++) {
                 pgp_enabled = g_list_append(pgp_enabled, strdup(pgp_enabled_list[i]));
             }
-            g_strfreev(pgp_enabled_list);
         }
 
         gchar* pgp_keyid = NULL;
@@ -329,6 +313,11 @@ accounts_get_account(const char* const name)
             startscript = g_key_file_get_string(accounts, name, "script.start", NULL);
         }
 
+        gchar* client = NULL;
+        if (g_key_file_has_key(accounts, name, "client.name", NULL)) {
+            client = g_key_file_get_string(accounts, name, "client.name", NULL);
+        }
+
         gchar* theme = NULL;
         if (g_key_file_has_key(accounts, name, "theme", NULL)) {
             theme = g_key_file_get_string(accounts, name, "theme", NULL);
@@ -342,30 +331,15 @@ accounts_get_account(const char* const name)
 
         gchar* auth_policy = g_key_file_get_string(accounts, name, "auth.policy", NULL);
 
-        ProfAccount* new_account = account_new(name, jid, password, eval_password, enabled,
+        int max_sessions = g_key_file_get_integer(accounts, name, "max.sessions", 0);
+
+        ProfAccount* new_account = account_new(g_strdup(name), jid, password, eval_password, enabled,
                                                server, port, resource, last_presence, login_presence,
                                                priority_online, priority_chat, priority_away, priority_xa,
                                                priority_dnd, muc_service, muc_nick, otr_policy, otr_manual,
                                                otr_opportunistic, otr_always, omemo_policy, omemo_enabled,
                                                omemo_disabled, ox_enabled, pgp_enabled, pgp_keyid,
-                                               startscript, theme, tls_policy, auth_policy);
-
-        g_free(jid);
-        g_free(password);
-        g_free(eval_password);
-        g_free(server);
-        g_free(resource);
-        g_free(last_presence);
-        g_free(login_presence);
-        g_free(muc_service);
-        g_free(muc_nick);
-        g_free(otr_policy);
-        g_free(omemo_policy);
-        g_free(pgp_keyid);
-        g_free(startscript);
-        g_free(theme);
-        g_free(tls_policy);
-        g_free(auth_policy);
+                                               startscript, theme, tls_policy, auth_policy, client, max_sessions);
 
         return new_account;
     }
@@ -443,10 +417,9 @@ accounts_rename(const char* const account_name, const char* const new_name)
     };
 
     for (int i = 0; i < ARRAY_SIZE(string_keys); i++) {
-        char* value = g_key_file_get_string(accounts, account_name, string_keys[i], NULL);
+        auto_gchar gchar* value = g_key_file_get_string(accounts, account_name, string_keys[i], NULL);
         if (value) {
             g_key_file_set_string(accounts, new_name, string_keys[i], value);
-            g_free(value);
         }
     }
 
@@ -472,7 +445,7 @@ accounts_account_exists(const char* const account_name)
 void
 accounts_set_jid(const char* const account_name, const char* const value)
 {
-    Jid* jid = jid_create(value);
+    auto_jid Jid* jid = jid_create(value);
     if (jid) {
         if (accounts_account_exists(account_name)) {
             g_key_file_set_string(accounts, account_name, "jid", jid->barejid);
@@ -488,8 +461,6 @@ accounts_set_jid(const char* const account_name, const char* const value)
 
             _save_accounts();
         }
-
-        jid_destroy(jid);
     }
 }
 
@@ -568,12 +539,24 @@ accounts_set_script_start(const char* const account_name, const char* const valu
     _accounts_set_string_option(account_name, "script.start", value);
 }
 
+void
+accounts_set_client(const char* const account_name, const char* const value)
+{
+    _accounts_set_string_option(account_name, "client.name", value);
+}
+
 void
 accounts_set_theme(const char* const account_name, const char* const value)
 {
     _accounts_set_string_option(account_name, "theme", value);
 }
 
+void
+accounts_set_max_sessions(const char* const account_name, const int value)
+{
+    _accounts_set_int_option(account_name, "max.sessions", value);
+}
+
 void
 accounts_clear_password(const char* const account_name)
 {
@@ -610,6 +593,12 @@ accounts_clear_script_start(const char* const account_name)
     _accounts_clear_string_option(account_name, "script.start");
 }
 
+void
+accounts_clear_client(const char* const account_name)
+{
+    _accounts_clear_string_option(account_name, "client.name");
+}
+
 void
 accounts_clear_theme(const char* const account_name)
 {
@@ -634,6 +623,12 @@ accounts_clear_otr(const char* const account_name)
     _accounts_clear_string_option(account_name, "otr.policy");
 }
 
+void
+accounts_clear_max_sessions(const char* const account_name)
+{
+    _accounts_clear_string_option(account_name, "max.sessions");
+}
+
 void
 accounts_add_otr_policy(const char* const account_name, const char* const contact_jid, const char* const policy)
 {
@@ -847,15 +842,14 @@ accounts_set_last_activity(const char* const account_name)
         g_date_time_unref(nowdt);
 
         if (res) {
-            char* timestr = g_time_val_to_iso8601(&nowtv);
+            auto_char char* timestr = g_time_val_to_iso8601(&nowtv);
             g_key_file_set_string(accounts, account_name, "last.activity", timestr);
-            free(timestr);
             _save_accounts();
         }
     }
 }
 
-char*
+gchar*
 accounts_get_last_activity(const char* const account_name)
 {
     if (accounts_account_exists(account_name)) {
@@ -865,6 +859,24 @@ accounts_get_last_activity(const char* const account_name)
     }
 }
 
+gchar*
+accounts_get_resource(const char* const account_name)
+{
+    if (!accounts_account_exists(account_name)) {
+        return NULL;
+    }
+    return g_key_file_get_string(accounts, account_name, "resource", NULL);
+}
+
+int
+accounts_get_max_sessions(const char* const account_name)
+{
+    if (!accounts_account_exists(account_name)) {
+        return 0;
+    }
+    return g_key_file_get_integer(accounts, account_name, "max.sessions", 0);
+}
+
 void
 accounts_set_login_presence(const char* const account_name, const char* const value)
 {
@@ -878,7 +890,7 @@ resource_presence_t
 accounts_get_last_presence(const char* const account_name)
 {
     resource_presence_t result;
-    gchar* setting = g_key_file_get_string(accounts, account_name, "presence.last", NULL);
+    auto_gchar gchar* setting = g_key_file_get_string(accounts, account_name, "presence.last", NULL);
 
     if (setting == NULL || (strcmp(setting, "online") == 0)) {
         result = RESOURCE_ONLINE;
@@ -896,7 +908,6 @@ accounts_get_last_presence(const char* const account_name)
         result = RESOURCE_ONLINE;
     }
 
-    g_free(setting);
     return result;
 }
 
@@ -910,7 +921,7 @@ resource_presence_t
 accounts_get_login_presence(const char* const account_name)
 {
     resource_presence_t result;
-    gchar* setting = g_key_file_get_string(accounts, account_name, "presence.login", NULL);
+    auto_gchar gchar* setting = g_key_file_get_string(accounts, account_name, "presence.login", NULL);
 
     if (setting == NULL || (strcmp(setting, "online") == 0)) {
         result = RESOURCE_ONLINE;
@@ -929,21 +940,19 @@ accounts_get_login_presence(const char* const account_name)
         result = RESOURCE_ONLINE;
     }
 
-    g_free(setting);
     return result;
 }
 
 char*
 accounts_get_login_status(const char* const account_name)
 {
-    gchar* setting = g_key_file_get_string(accounts, account_name, "presence.login", NULL);
+    auto_gchar gchar* setting = g_key_file_get_string(accounts, account_name, "presence.login", NULL);
     gchar* status = NULL;
 
     if (g_strcmp0(setting, "last") == 0) {
         status = accounts_get_last_status(account_name);
     }
 
-    g_free(setting);
     return status;
 }
 
@@ -951,14 +960,10 @@ static void
 _save_accounts(void)
 {
     gsize g_data_size;
-    gchar* g_accounts_data = g_key_file_to_data(accounts, &g_data_size, NULL);
+    auto_gchar gchar* g_accounts_data = g_key_file_to_data(accounts, &g_data_size, NULL);
 
-    gchar* base = g_path_get_dirname(accounts_loc);
-    gchar* true_loc = get_file_or_linked(accounts_loc, base);
+    auto_gchar gchar* base = g_path_get_dirname(accounts_loc);
+    auto_gchar gchar* true_loc = get_file_or_linked(accounts_loc, base);
     g_file_set_contents(true_loc, g_accounts_data, g_data_size, NULL);
     g_chmod(accounts_loc, S_IRUSR | S_IWUSR);
-
-    g_free(base);
-    free(true_loc);
-    g_free(g_accounts_data);
 }
