@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2019 Aurelien Aptel <aurelien.aptel@gmail.com>
- * Copyright (C) 2019 - 2022 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -35,6 +35,7 @@
  */
 
 #include "config.h"
+#include "common.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -59,7 +60,7 @@ static struct color_pair_cache
     struct
     {
         int16_t fg, bg;
-    } * pairs;
+    }* pairs;
     int size;
     int capacity;
 } cache = { 0 };
@@ -508,10 +509,9 @@ color_pair_cache_hash_str(const char* str, color_profile profile)
     int fg = color_hash(str, profile);
     int bg = -1;
 
-    char* bkgnd = theme_get_bkgnd();
+    auto_char char* bkgnd = theme_get_bkgnd();
     if (bkgnd) {
         bg = find_col(bkgnd, strlen(bkgnd));
-        free(bkgnd);
     }
 
     return _color_pair_cache_get(fg, bg);
