@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2020 - 2022 Michael Vetter <jubalh@idoru.org>
+ * Copyright (C) 2020 - 2023 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -53,8 +53,8 @@ static char* _files_get_xdg_data_home(void);
 void
 files_create_directories(void)
 {
-    gchar* xdg_config = _files_get_xdg_config_home();
-    gchar* xdg_data = _files_get_xdg_data_home();
+    auto_gchar gchar* xdg_config = _files_get_xdg_config_home();
+    auto_gchar gchar* xdg_data = _files_get_xdg_data_home();
 
     GString* themes_dir = g_string_new(xdg_config);
     g_string_append(themes_dir, "/profanity/themes");
@@ -88,17 +88,13 @@ files_create_directories(void)
     g_string_free(chatlogs_dir, TRUE);
     g_string_free(logs_dir, TRUE);
     g_string_free(plugins_dir, TRUE);
-
-    g_free(xdg_config);
-    g_free(xdg_data);
 }
 
 gchar*
 files_get_inputrc_file(void)
 {
-    gchar* xdg_config = _files_get_xdg_config_home();
+    auto_gchar gchar* xdg_config = _files_get_xdg_config_home();
     GString* inputrc_file = g_string_new(xdg_config);
-    g_free(xdg_config);
 
     g_string_append(inputrc_file, "/profanity/inputrc");
 
@@ -114,18 +110,17 @@ files_get_inputrc_file(void)
     return NULL;
 }
 
-char*
+gchar*
 files_get_log_file(const char* const log_file)
 {
-    gchar* xdg_data = _files_get_xdg_data_home();
+    auto_gchar gchar* xdg_data = _files_get_xdg_data_home();
     GString* logfile;
 
     if (log_file) {
-        gchar* log_path = g_path_get_dirname(log_file);
+        auto_gchar gchar* log_path = g_path_get_dirname(log_file);
         if (!create_dir(log_path)) {
             log_error("Error while creating directory %s", log_path);
         }
-        g_free(log_path);
 
         logfile = g_string_new(log_file);
     } else {
@@ -139,9 +134,8 @@ files_get_log_file(const char* const log_file)
         g_string_append(logfile, ".log");
     }
 
-    char* result = g_strdup(logfile->str);
+    gchar* result = g_strdup(logfile->str);
 
-    free(xdg_data);
     g_string_free(logfile, TRUE);
 
     return result;
@@ -150,33 +144,34 @@ files_get_log_file(const char* const log_file)
 gchar*
 files_get_config_path(const char* const config_base)
 {
-    gchar* xdg_config = _files_get_xdg_config_home();
+    auto_gchar gchar* xdg_config = _files_get_xdg_config_home();
     gchar* result = g_strdup_printf("%s/profanity/%s", xdg_config, config_base);
-    g_free(xdg_config);
     return result;
 }
 
+/**
+ * Get the full path by appending the given location to the project base path.
+ *
+ * @param location The location (directory or file) to append to the project base path.
+ * @return The full path obtained by appending the location to the project base path.
+ *
+ * @note Remember to free returned string using `auto_gchar` or `g_free()` when it is no longer needed to prevent memory leaks.
+ */
 gchar*
-files_get_data_path(const char* const data_base)
+files_get_data_path(const char* const location)
 {
-    gchar* xdg_data = _files_get_xdg_data_home();
-    gchar* result = g_strdup_printf("%s/profanity/%s", xdg_data, data_base);
-    g_free(xdg_data);
+    auto_gchar gchar* xdg_data = _files_get_xdg_data_home();
+    gchar* result = g_strdup_printf("%s/profanity/%s", xdg_data, location);
     return result;
 }
 
 gchar*
 files_get_account_data_path(const char* const specific_dir, const char* const jid)
 {
-    gchar* data_dir = files_get_data_path(specific_dir);
-    gchar* account_dir = str_replace(jid, "@", "_at_");
+    auto_gchar gchar* data_dir = files_get_data_path(specific_dir);
+    auto_char char* account_dir = str_replace(jid, "@", "_at_");
 
-    gchar* result = g_strdup_printf("%s/%s", data_dir, account_dir);
-
-    g_free(account_dir);
-    g_free(data_dir);
-
-    return result;
+    return g_strdup_printf("%s/%s", data_dir, account_dir);
 }
 
 gchar*
