@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2022 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -101,21 +101,15 @@ _prefs_load(void)
 
     // move pre 0.5.0 autoaway.message to autoaway.awaymessage
     if (g_key_file_has_key(prefs, PREF_GROUP_PRESENCE, "autoaway.message", NULL)) {
-        char* message = g_key_file_get_string(prefs, PREF_GROUP_PRESENCE, "autoaway.message", NULL);
+        auto_gchar gchar* message = g_key_file_get_string(prefs, PREF_GROUP_PRESENCE, "autoaway.message", NULL);
         g_key_file_set_string(prefs, PREF_GROUP_PRESENCE, "autoaway.awaymessage", message);
         g_key_file_remove_key(prefs, PREF_GROUP_PRESENCE, "autoaway.message", NULL);
-        g_free(message);
     }
 
     // migrate pre 0.5.0 time settings
     if (g_key_file_has_key(prefs, PREF_GROUP_UI, "time", NULL)) {
-        char* time = g_key_file_get_string(prefs, PREF_GROUP_UI, "time", NULL);
-        char* val = NULL;
-        if (time) {
-            val = time;
-        } else {
-            val = "off";
-        }
+        auto_gchar gchar* time = g_key_file_get_string(prefs, PREF_GROUP_UI, "time", NULL);
+        char* val = time ? time : "off";
         g_key_file_set_string(prefs, PREF_GROUP_UI, "time.console", val);
         g_key_file_set_string(prefs, PREF_GROUP_UI, "time.chat", val);
         g_key_file_set_string(prefs, PREF_GROUP_UI, "time.muc", val);
@@ -123,12 +117,11 @@ _prefs_load(void)
         g_key_file_set_string(prefs, PREF_GROUP_UI, "time.private", val);
         g_key_file_set_string(prefs, PREF_GROUP_UI, "time.xmlconsole", val);
         g_key_file_remove_key(prefs, PREF_GROUP_UI, "time", NULL);
-        g_free(time);
     }
 
     // move pre 0.5.0 notify settings
     if (g_key_file_has_key(prefs, PREF_GROUP_NOTIFICATIONS, "room", NULL)) {
-        char* value = g_key_file_get_string(prefs, PREF_GROUP_NOTIFICATIONS, "room", NULL);
+        auto_gchar gchar* value = g_key_file_get_string(prefs, PREF_GROUP_NOTIFICATIONS, "room", NULL);
         if (g_strcmp0(value, "on") == 0) {
             g_key_file_set_boolean(prefs, PREF_GROUP_NOTIFICATIONS, "room", TRUE);
         } else if (g_strcmp0(value, "off") == 0) {
@@ -137,7 +130,6 @@ _prefs_load(void)
             g_key_file_set_boolean(prefs, PREF_GROUP_NOTIFICATIONS, "room", FALSE);
             g_key_file_set_boolean(prefs, PREF_GROUP_NOTIFICATIONS, "room.mention", TRUE);
         }
-        g_free(value);
     }
 
     // move pre 0.6.0 titlebar settings to wintitle
@@ -154,7 +146,7 @@ _prefs_load(void)
 
     // after 0.8.1: titlebar use jid|name -> titlebar show|hide jid|name
     if (g_key_file_has_key(prefs, PREF_GROUP_UI, "titlebar.muc.title", NULL)) {
-        char* value = g_key_file_get_string(prefs, PREF_GROUP_UI, "titlebar.muc.title", NULL);
+        auto_gchar gchar* value = g_key_file_get_string(prefs, PREF_GROUP_UI, "titlebar.muc.title", NULL);
         if (g_strcmp0(value, "name") == 0) {
             g_key_file_set_boolean(prefs, PREF_GROUP_UI, "titlebar.muc.title.name", TRUE);
         } else if (g_strcmp0(value, "jid") == 0) {
@@ -164,7 +156,7 @@ _prefs_load(void)
 
     // 0.9.0 introduced /urlopen. It was saved under "logging" section. Now we have a new "executables" section.
     if (g_key_file_has_key(prefs, PREF_GROUP_LOGGING, "urlopen.cmd", NULL)) {
-        char* val = g_key_file_get_string(prefs, PREF_GROUP_LOGGING, "urlopen.cmd", NULL);
+        auto_gchar gchar* val = g_key_file_get_string(prefs, PREF_GROUP_LOGGING, "urlopen.cmd", NULL);
 
         GString* value = g_string_new("false;");
         value = g_string_append(value, val);
@@ -178,7 +170,7 @@ _prefs_load(void)
 
     // 0.9.0 introduced configurable /avatar. It was saved under "logging" section. Now we have a new "executables" section.
     if (g_key_file_has_key(prefs, PREF_GROUP_LOGGING, "avatar.cmd", NULL)) {
-        char* value = g_key_file_get_string(prefs, PREF_GROUP_LOGGING, "avatar.cmd", NULL);
+        auto_gchar gchar* value = g_key_file_get_string(prefs, PREF_GROUP_LOGGING, "avatar.cmd", NULL);
         g_key_file_set_string(prefs, PREF_GROUP_EXECUTABLES, "avatar.cmd", value);
         g_key_file_remove_key(prefs, PREF_GROUP_LOGGING, "avatar.cmd", NULL);
     }
@@ -192,7 +184,7 @@ _prefs_load(void)
     // file type or scheme matching. Move value saved under 'DEF' locale to a
     // simple key-value string not under any locale.
     {
-        char** values = g_key_file_get_locale_string_list(prefs, PREF_GROUP_EXECUTABLES, "url.open.cmd", "DEF", NULL, NULL);
+        auto_gcharv gchar** values = g_key_file_get_locale_string_list(prefs, PREF_GROUP_EXECUTABLES, "url.open.cmd", "DEF", NULL, NULL);
         if (values && !g_key_file_has_key(prefs, PREF_GROUP_EXECUTABLES, "url.open.cmd", NULL)) {
             // First value in array is `require_save` option -- we ignore that
             // one as there is no such option anymore.
@@ -201,16 +193,13 @@ _prefs_load(void)
             g_key_file_set_string(prefs, PREF_GROUP_EXECUTABLES, "url.open.cmd", executable);
             g_key_file_set_comment(prefs, PREF_GROUP_EXECUTABLES, "url.open.cmd", " Migrated from url.open.cmd[DEF]. `require_save` option has been removed in v0.10 and was discarded.", NULL);
             g_key_file_remove_key(prefs, PREF_GROUP_EXECUTABLES, "url.open.cmd[DEF]", NULL);
-
-            g_strfreev(values);
         }
 
-        char* value = g_key_file_get_locale_string(prefs, PREF_GROUP_EXECUTABLES, "url.save.cmd", "DEF", NULL);
+        auto_gchar gchar* value = g_key_file_get_locale_string(prefs, PREF_GROUP_EXECUTABLES, "url.save.cmd", "DEF", NULL);
         if (value && !g_key_file_has_key(prefs, PREF_GROUP_EXECUTABLES, "url.save.cmd", NULL)) {
             g_key_file_set_string(prefs, PREF_GROUP_EXECUTABLES, "url.save.cmd", value);
             g_key_file_set_comment(prefs, PREF_GROUP_EXECUTABLES, "url.save.cmd", " Migrated from url.save.cmd[DEF].", NULL);
             g_key_file_remove_key(prefs, PREF_GROUP_EXECUTABLES, "url.save.cmd[DEF]", NULL);
-            g_free(value);
         }
     }
 
@@ -227,12 +216,11 @@ _prefs_load(void)
 
     room_trigger_ac = autocomplete_new();
     gsize len = 0;
-    gchar** triggers = g_key_file_get_string_list(prefs, PREF_GROUP_NOTIFICATIONS, "room.trigger.list", &len, NULL);
+    auto_gcharv gchar** triggers = g_key_file_get_string_list(prefs, PREF_GROUP_NOTIFICATIONS, "room.trigger.list", &len, NULL);
 
     for (int i = 0; i < len; i++) {
         autocomplete_add(room_trigger_ac, triggers[i]);
     }
-    g_strfreev(triggers);
 }
 
 /* Clean up after _prefs_load() */
@@ -299,7 +287,7 @@ prefs_close(void)
     prefs_loc = NULL;
 }
 
-char*
+gchar*
 prefs_autocomplete_boolean_choice(const char* const prefix, gboolean previous, void* context)
 {
     return autocomplete_complete(boolean_choice_ac, prefix, TRUE, previous);
@@ -311,7 +299,7 @@ prefs_reset_boolean_choice(void)
     autocomplete_reset(boolean_choice_ac);
 }
 
-char*
+gchar*
 prefs_autocomplete_room_trigger(const char* const prefix, gboolean previous, void* context)
 {
     return autocomplete_complete(room_trigger_ac, prefix, TRUE, previous);
@@ -342,21 +330,17 @@ prefs_message_get_triggers(const char* const message)
 {
     GList* result = NULL;
 
-    char* message_lower = g_utf8_strdown(message, -1);
+    auto_gchar gchar* message_lower = g_utf8_strdown(message, -1);
     gsize len = 0;
-    gchar** triggers = g_key_file_get_string_list(prefs, PREF_GROUP_NOTIFICATIONS, "room.trigger.list", &len, NULL);
+    auto_gcharv gchar** triggers = g_key_file_get_string_list(prefs, PREF_GROUP_NOTIFICATIONS, "room.trigger.list", &len, NULL);
 
     for (int i = 0; i < len; i++) {
-        char* trigger_lower = g_utf8_strdown(triggers[i], -1);
+        auto_gchar gchar* trigger_lower = g_utf8_strdown(triggers[i], -1);
         if (g_strrstr(message_lower, trigger_lower)) {
             result = g_list_append(result, strdup(triggers[i]));
         }
-        g_free(trigger_lower);
     }
 
-    g_strfreev(triggers);
-    g_free(message_lower);
-
     return result;
 }
 
@@ -533,14 +517,22 @@ prefs_set_boolean(preference_t pref, gboolean value)
     g_key_file_set_boolean(prefs, group, key, value);
 }
 
-char*
+/**
+ * @brief Retrieves a string preference value.
+ *
+ * @param pref The preference identifier.
+ * @return The string preference value or `NULL` if not found.
+ *
+ * @note Remember to free the returned string using `auto_gchar` or `g_free()`.
+ */
+gchar*
 prefs_get_string(preference_t pref)
 {
     const char* group = _get_group(pref);
     const char* key = _get_key(pref);
     char* def = _get_default_string(pref);
 
-    char* result = g_key_file_get_string(prefs, group, key, NULL);
+    gchar* result = g_key_file_get_string(prefs, group, key, NULL);
 
     if (result == NULL) {
         if (def) {
@@ -553,14 +545,23 @@ prefs_get_string(preference_t pref)
     }
 }
 
-char*
-prefs_get_string_with_option(preference_t pref, gchar* option)
+/**
+ * @brief Retrieves a localized string preference value.
+ *
+ * @param pref   The preference to retrieve the value for.
+ * @param locale The option to consider.
+ * @return       Returns the value associated with pref translated in the given locale if available.
+ *
+ * @note Remember to free the returned string using `auto_gchar` or `g_free()`.
+ */
+gchar*
+prefs_get_string_with_locale(preference_t pref, gchar* locale)
 {
     const char* group = _get_group(pref);
     const char* key = _get_key(pref);
     char* def = _get_default_string(pref);
 
-    char* result = g_key_file_get_locale_string(prefs, group, key, option, NULL);
+    gchar* result = g_key_file_get_locale_string(prefs, group, key, locale, NULL);
 
     if (result == NULL) {
         // check for user set default
@@ -578,15 +579,21 @@ prefs_get_string_with_option(preference_t pref, gchar* option)
     return result;
 }
 
+/**
+ * @brief Sets or deletes a string value for the given preference in the preference file.
+ *
+ * @param pref      The preference to set the value for.
+ * @param new_value The new string value to set. Pass NULL to remove the key.
+ */
 void
-prefs_set_string(preference_t pref, char* value)
+prefs_set_string(preference_t pref, gchar* new_value)
 {
     const char* group = _get_group(pref);
     const char* key = _get_key(pref);
-    if (value == NULL) {
+    if (new_value == NULL) {
         g_key_file_remove_key(prefs, group, key, NULL);
     } else {
-        g_key_file_set_string(prefs, group, key, value);
+        g_key_file_set_string(prefs, group, key, new_value);
     }
 }
 
@@ -632,10 +639,9 @@ prefs_get_tls_certpath(void)
     const char* group = _get_group(PREF_TLS_CERTPATH);
     const char* key = _get_key(PREF_TLS_CERTPATH);
 
-    char* setting = g_key_file_get_string(prefs, group, key, NULL);
+    auto_gchar gchar* setting = g_key_file_get_string(prefs, group, key, NULL);
 
     if (g_strcmp0(setting, "none") == 0) {
-        g_free(setting);
         return NULL;
     }
 
@@ -660,7 +666,6 @@ prefs_get_tls_certpath(void)
     }
 
     char* result = strdup(setting);
-    g_free(setting);
 
     return result;
 }
@@ -877,12 +882,6 @@ prefs_remove_plugin(const char* const name)
     _save_prefs();
 }
 
-void
-prefs_free_plugins(gchar** plugins)
-{
-    g_strfreev(plugins);
-}
-
 void
 prefs_set_occupants_size(gint value)
 {
@@ -1331,14 +1330,12 @@ prefs_get_room_notify_triggers(void)
 {
     GList* result = NULL;
     gsize len = 0;
-    gchar** triggers = g_key_file_get_string_list(prefs, PREF_GROUP_NOTIFICATIONS, "room.trigger.list", &len, NULL);
+    auto_gcharv gchar** triggers = g_key_file_get_string_list(prefs, PREF_GROUP_NOTIFICATIONS, "room.trigger.list", &len, NULL);
 
     for (int i = 0; i < len; i++) {
         result = g_list_append(result, strdup(triggers[i]));
     }
 
-    g_strfreev(triggers);
-
     return result;
 }
 
@@ -1646,7 +1643,7 @@ prefs_add_alias(const char* const name, const char* const value)
     }
 }
 
-char*
+gchar*
 prefs_get_alias(const char* const name)
 {
     return g_key_file_get_string(prefs, PREF_GROUP_ALIAS, name, NULL);
@@ -1680,25 +1677,21 @@ prefs_get_aliases(void)
     } else {
         GList* result = NULL;
         gsize len;
-        gchar** keys = g_key_file_get_keys(prefs, PREF_GROUP_ALIAS, &len, NULL);
+        auto_gcharv gchar** keys = g_key_file_get_keys(prefs, PREF_GROUP_ALIAS, &len, NULL);
 
         for (int i = 0; i < len; i++) {
             char* name = keys[i];
-            char* value = g_key_file_get_string(prefs, PREF_GROUP_ALIAS, name, NULL);
+            auto_gchar gchar* value = g_key_file_get_string(prefs, PREF_GROUP_ALIAS, name, NULL);
 
             if (value) {
                 ProfAlias* alias = malloc(sizeof(struct prof_alias_t));
                 alias->name = strdup(name);
                 alias->value = strdup(value);
 
-                free(value);
-
                 result = g_list_insert_sorted(result, alias, (GCompareFunc)_alias_cmp);
             }
         }
 
-        g_strfreev(keys);
-
         return result;
     }
 }
@@ -1721,16 +1714,12 @@ static void
 _save_prefs(void)
 {
     gsize g_data_size;
-    gchar* g_prefs_data = g_key_file_to_data(prefs, &g_data_size, NULL);
-    gchar* base = g_path_get_dirname(prefs_loc);
-    gchar* true_loc = get_file_or_linked(prefs_loc, base);
+    auto_gchar gchar* g_prefs_data = g_key_file_to_data(prefs, &g_data_size, NULL);
+    auto_gchar gchar* base = g_path_get_dirname(prefs_loc);
+    auto_gchar gchar* true_loc = get_file_or_linked(prefs_loc, base);
 
     g_file_set_contents(true_loc, g_prefs_data, g_data_size, NULL);
     g_chmod(prefs_loc, S_IRUSR | S_IWUSR);
-
-    g_free(base);
-    free(true_loc);
-    g_free(g_prefs_data);
 }
 
 // get the preference group for a specific preference
@@ -1770,6 +1759,7 @@ _get_group(preference_t pref)
     case PREF_TIME_XMLCONSOLE:
     case PREF_TIME_STATUSBAR:
     case PREF_TIME_LASTACTIVITY:
+    case PREF_TIME_VCARD:
     case PREF_ROSTER:
     case PREF_ROSTER_OFFLINE:
     case PREF_ROSTER_RESOURCE:
@@ -1812,6 +1802,7 @@ _get_group(preference_t pref)
     case PREF_STATUSBAR_SELF:
     case PREF_STATUSBAR_CHAT:
     case PREF_STATUSBAR_ROOM:
+    case PREF_STATUSBAR_TABMODE:
     case PREF_TITLEBAR_MUC_TITLE_JID:
     case PREF_TITLEBAR_MUC_TITLE_NAME:
     case PREF_SLASH_GUARD:
@@ -1842,6 +1833,7 @@ _get_group(preference_t pref)
     case PREF_TRAY_READ:
     case PREF_ADV_NOTIFY_DISCO_OR_VERSION:
         return PREF_GROUP_NOTIFICATIONS;
+    case PREF_DBLOG:
     case PREF_CHLOG:
     case PREF_GRLOG:
     case PREF_LOG_ROTATE:
@@ -1850,6 +1842,7 @@ _get_group(preference_t pref)
     case PREF_AVATAR_CMD:
     case PREF_URL_OPEN_CMD:
     case PREF_URL_SAVE_CMD:
+    case PREF_VCARD_PHOTO_CMD:
         return PREF_GROUP_EXECUTABLES;
     case PREF_AUTOAWAY_CHECK:
     case PREF_AUTOAWAY_MODE:
@@ -1867,6 +1860,9 @@ _get_group(preference_t pref)
     case PREF_CORRECTION_ALLOW:
     case PREF_MAM:
     case PREF_SILENCE_NON_ROSTER:
+    case PREF_STROPHE_VERBOSITY:
+    case PREF_STROPHE_SM_ENABLED:
+    case PREF_STROPHE_SM_RESEND:
         return PREF_GROUP_CONNECTION;
     case PREF_OTR_LOG:
     case PREF_OTR_POLICY:
@@ -1874,6 +1870,7 @@ _get_group(preference_t pref)
         return PREF_GROUP_OTR;
     case PREF_PGP_LOG:
     case PREF_PGP_SENDFILE:
+    case PREF_PGP_PUBKEY_AUTOIMPORT:
         return PREF_GROUP_PGP;
     case PREF_BOOKMARK_INVITE:
     case PREF_ROOM_LIST_CACHE:
@@ -1985,6 +1982,8 @@ _get_key(preference_t pref)
         return "room.mention.wholeword";
     case PREF_CHLOG:
         return "chlog";
+    case PREF_DBLOG:
+        return "dblog";
     case PREF_GRLOG:
         return "grlog";
     case PREF_AUTOAWAY_CHECK:
@@ -2029,6 +2028,8 @@ _get_key(preference_t pref)
         return "time.statusbar";
     case PREF_TIME_LASTACTIVITY:
         return "time.lastactivity";
+    case PREF_TIME_VCARD:
+        return "time.vcard";
     case PREF_ROSTER:
         return "roster";
     case PREF_ROSTER_OFFLINE:
@@ -2093,6 +2094,8 @@ _get_key(preference_t pref)
         return "log";
     case PREF_PGP_SENDFILE:
         return "sendfile";
+    case PREF_PGP_PUBKEY_AUTOIMPORT:
+        return "pgp.pubkey.autoimport";
     case PREF_TLS_CERTPATH:
         return "tls.certpath";
     case PREF_TLS_SHOW:
@@ -2129,6 +2132,8 @@ _get_key(preference_t pref)
         return "statusbar.chat";
     case PREF_STATUSBAR_ROOM:
         return "statusbar.room";
+    case PREF_STATUSBAR_TABMODE:
+        return "statusbar.tabmode";
     case PREF_OMEMO_LOG:
         return "log";
     case PREF_OMEMO_POLICY:
@@ -2159,6 +2164,14 @@ _get_key(preference_t pref)
         return "log";
     case PREF_MOOD:
         return "mood";
+    case PREF_VCARD_PHOTO_CMD:
+        return "vcard.photo.cmd";
+    case PREF_STROPHE_VERBOSITY:
+        return "strophe.verbosity";
+    case PREF_STROPHE_SM_ENABLED:
+        return "strophe.sm.enabled";
+    case PREF_STROPHE_SM_RESEND:
+        return "strophe.sm.resend";
     default:
         return NULL;
     }
@@ -2211,7 +2224,10 @@ _get_default_boolean(preference_t pref)
     case PREF_INTYPE_CONSOLE:
     case PREF_NOTIFY_MENTION_WHOLE_WORD:
     case PREF_MOOD:
+    case PREF_STROPHE_SM_ENABLED:
+    case PREF_STROPHE_SM_RESEND:
         return TRUE;
+    case PREF_PGP_PUBKEY_AUTOIMPORT:
     default:
         return FALSE;
     }
@@ -2269,6 +2285,8 @@ _get_default_string(preference_t pref)
         return "%H:%M";
     case PREF_TIME_LASTACTIVITY:
         return "%d/%m/%y %H:%M:%S";
+    case PREF_TIME_VCARD:
+        return "%d/%m/%y";
     case PREF_PGP_LOG:
         return "on";
     case PREF_CONSOLE_MUC:
@@ -2281,6 +2299,8 @@ _get_default_string(preference_t pref)
         return "user";
     case PREF_STATUSBAR_ROOM:
         return "room";
+    case PREF_STATUSBAR_TABMODE:
+        return "default";
     case PREF_OMEMO_LOG:
         return "on";
     case PREF_OMEMO_POLICY:
@@ -2290,9 +2310,11 @@ _get_default_string(preference_t pref)
     case PREF_COLOR_NICK:
         return "false";
     case PREF_AVATAR_CMD:
-        return "xdg-open";
+        return "xdg-open %p";
     case PREF_URL_OPEN_CMD:
         return "xdg-open %u";
+    case PREF_VCARD_PHOTO_CMD:
+        return "xdg-open %p";
     case PREF_COMPOSE_EDITOR:
     {
         gchar* editor = getenv("EDITOR");
@@ -2306,6 +2328,10 @@ _get_default_string(preference_t pref)
         return NULL; // Default to built-in method.
     case PREF_OX_LOG:
         return "on";
+    case PREF_STROPHE_VERBOSITY:
+        return "0";
+    case PREF_DBLOG:
+        return "on";
     default:
         return NULL;
     }
