@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2022 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -105,6 +105,7 @@ typedef enum {
     PREF_TIME_XMLCONSOLE,
     PREF_TIME_STATUSBAR,
     PREF_TIME_LASTACTIVITY,
+    PREF_TIME_VCARD,
     PREF_STATUSES,
     PREF_STATUSES_CONSOLE,
     PREF_STATUSES_CHAT,
@@ -126,6 +127,7 @@ typedef enum {
     PREF_NOTIFY_MENTION_CASE_SENSITIVE,
     PREF_NOTIFY_MENTION_WHOLE_WORD,
     PREF_CHLOG,
+    PREF_DBLOG,
     PREF_GRLOG,
     PREF_AUTOAWAY_CHECK,
     PREF_AUTOAWAY_MODE,
@@ -146,6 +148,7 @@ typedef enum {
     PREF_TITLEBAR_MUC_TITLE_NAME,
     PREF_PGP_LOG,
     PREF_PGP_SENDFILE,
+    PREF_PGP_PUBKEY_AUTOIMPORT,
     PREF_TLS_CERTPATH,
     PREF_TLS_SHOW,
     PREF_LASTACTIVITY,
@@ -181,6 +184,11 @@ typedef enum {
     PREF_NOTIFY_ROOM_OFFLINE,
     PREF_OX_LOG,
     PREF_MOOD,
+    PREF_STROPHE_VERBOSITY,
+    PREF_STROPHE_SM_ENABLED,
+    PREF_STROPHE_SM_RESEND,
+    PREF_VCARD_PHOTO_CMD,
+    PREF_STATUSBAR_TABMODE,
 } preference_t;
 
 typedef struct prof_alias_t
@@ -205,10 +213,10 @@ void prefs_reload(void);
 char* prefs_find_login(char* prefix);
 void prefs_reset_login_search(void);
 
-char* prefs_autocomplete_boolean_choice(const char* const prefix, gboolean previous, void* context);
+gchar* prefs_autocomplete_boolean_choice(const char* const prefix, gboolean previous, void* context);
 void prefs_reset_boolean_choice(void);
 
-char* prefs_autocomplete_room_trigger(const char* const prefix, gboolean previous, void* context);
+gchar* prefs_autocomplete_room_trigger(const char* const prefix, gboolean previous, void* context);
 void prefs_reset_room_trigger_ac(void);
 
 gint prefs_get_gone(void);
@@ -245,7 +253,6 @@ gint prefs_get_autoxa_time(void);
 void prefs_set_autoxa_time(gint value);
 
 gchar** prefs_get_plugins(void);
-void prefs_free_plugins(gchar** plugins);
 void prefs_add_plugin(const char* const name);
 void prefs_remove_plugin(const char* const name);
 
@@ -303,7 +310,7 @@ gint prefs_get_tray_timer(void);
 
 gboolean prefs_add_alias(const char* const name, const char* const value);
 gboolean prefs_remove_alias(const char* const name);
-char* prefs_get_alias(const char* const name);
+gchar* prefs_get_alias(const char* const name);
 GList* prefs_get_aliases(void);
 void prefs_free_aliases(GList* aliases);
 
@@ -327,9 +334,9 @@ void prefs_save_win_placement(ProfWinPlacement* placement);
 
 gboolean prefs_get_boolean(preference_t pref);
 void prefs_set_boolean(preference_t pref, gboolean value);
-char* prefs_get_string(preference_t pref);
-char* prefs_get_string_with_option(preference_t pref, gchar* option);
-void prefs_set_string(preference_t pref, char* value);
+gchar* prefs_get_string(preference_t pref);
+gchar* prefs_get_string_with_locale(preference_t pref, gchar* locale);
+void prefs_set_string(preference_t pref, gchar* new_value);
 void prefs_set_string_with_option(preference_t pref, char* option, char* value);
 void prefs_set_string_list_with_option(preference_t pref, char* option, const gchar* const* values);
 
