@@ -71,12 +71,11 @@ tlscerts_init(void)
 
     certs_ac = autocomplete_new();
     gsize len = 0;
-    gchar** groups = g_key_file_get_groups(tlscerts, &len);
+    auto_gcharv gchar** groups = g_key_file_get_groups(tlscerts, &len);
 
     for (int i = 0; i < g_strv_length(groups); i++) {
         autocomplete_add(certs_ac, groups[i]);
     }
-    g_strfreev(groups);
 
     current_fp = NULL;
 }
@@ -116,38 +115,25 @@ tlscerts_list(void)
 {
     GList* res = NULL;
     gsize len = 0;
-    gchar** groups = g_key_file_get_groups(tlscerts, &len);
+    auto_gcharv gchar** groups = g_key_file_get_groups(tlscerts, &len);
 
     for (int i = 0; i < g_strv_length(groups); i++) {
         char* fingerprint = strdup(groups[i]);
         int version = g_key_file_get_integer(tlscerts, fingerprint, "version", NULL);
-        char* serialnumber = g_key_file_get_string(tlscerts, fingerprint, "serialnumber", NULL);
-        char* subjectname = g_key_file_get_string(tlscerts, fingerprint, "subjectname", NULL);
-        char* issuername = g_key_file_get_string(tlscerts, fingerprint, "issuername", NULL);
-        char* notbefore = g_key_file_get_string(tlscerts, fingerprint, "start", NULL);
-        char* notafter = g_key_file_get_string(tlscerts, fingerprint, "end", NULL);
-        char* keyalg = g_key_file_get_string(tlscerts, fingerprint, "keyalg", NULL);
-        char* signaturealg = g_key_file_get_string(tlscerts, fingerprint, "signaturealg", NULL);
+        auto_gchar gchar* serialnumber = g_key_file_get_string(tlscerts, fingerprint, "serialnumber", NULL);
+        auto_gchar gchar* subjectname = g_key_file_get_string(tlscerts, fingerprint, "subjectname", NULL);
+        auto_gchar gchar* issuername = g_key_file_get_string(tlscerts, fingerprint, "issuername", NULL);
+        auto_gchar gchar* notbefore = g_key_file_get_string(tlscerts, fingerprint, "start", NULL);
+        auto_gchar gchar* notafter = g_key_file_get_string(tlscerts, fingerprint, "end", NULL);
+        auto_gchar gchar* keyalg = g_key_file_get_string(tlscerts, fingerprint, "keyalg", NULL);
+        auto_gchar gchar* signaturealg = g_key_file_get_string(tlscerts, fingerprint, "signaturealg", NULL);
 
         TLSCertificate* cert = tlscerts_new(fingerprint, version, serialnumber, subjectname, issuername, notbefore,
                                             notafter, keyalg, signaturealg, NULL);
 
-        free(fingerprint);
-        free(serialnumber);
-        free(subjectname);
-        free(issuername);
-        free(notbefore);
-        free(notafter);
-        free(keyalg);
-        free(signaturealg);
-
         res = g_list_append(res, cert);
     }
 
-    if (groups) {
-        g_strfreev(groups);
-    }
-
     return res;
 }
 
@@ -187,9 +173,9 @@ tlscerts_new(const char* const fingerprint, int version, const char* const seria
         cert->pem = strdup(pem);
     }
 
-    gchar** fields = g_strsplit(subjectname, "/", 0);
+    auto_gcharv gchar** fields = g_strsplit(subjectname, "/", 0);
     for (int i = 0; i < g_strv_length(fields); i++) {
-        gchar** keyval = g_strsplit(fields[i], "=", 2);
+        auto_gcharv gchar** keyval = g_strsplit(fields[i], "=", 2);
         if (g_strv_length(keyval) == 2) {
             if ((g_strcmp0(keyval[0], "C") == 0) || (g_strcmp0(keyval[0], "countryName") == 0)) {
                 cert->subject_country = strdup(keyval[1]);
@@ -216,13 +202,11 @@ tlscerts_new(const char* const fingerprint, int version, const char* const seria
                 cert->subject_email = strdup(keyval[1]);
             }
         }
-        g_strfreev(keyval);
     }
-    g_strfreev(fields);
 
-    fields = g_strsplit(issuername, "/", 0);
-    for (int i = 0; i < g_strv_length(fields); i++) {
-        gchar** keyval = g_strsplit(fields[i], "=", 2);
+    auto_gcharv gchar** fields2 = g_strsplit(issuername, "/", 0);
+    for (int i = 0; i < g_strv_length(fields2); i++) {
+        auto_gcharv gchar** keyval = g_strsplit(fields2[i], "=", 2);
         if (g_strv_length(keyval) == 2) {
             if ((g_strcmp0(keyval[0], "C") == 0) || (g_strcmp0(keyval[0], "countryName") == 0)) {
                 cert->issuer_country = strdup(keyval[1]);
@@ -249,9 +233,7 @@ tlscerts_new(const char* const fingerprint, int version, const char* const seria
                 cert->issuer_email = strdup(keyval[1]);
             }
         }
-        g_strfreev(keyval);
     }
-    g_strfreev(fields);
 
     return cert;
 }
@@ -316,25 +298,16 @@ tlscerts_get_trusted(const char* const fingerprint)
     }
 
     int version = g_key_file_get_integer(tlscerts, fingerprint, "version", NULL);
-    char* serialnumber = g_key_file_get_string(tlscerts, fingerprint, "serialnumber", NULL);
-    char* subjectname = g_key_file_get_string(tlscerts, fingerprint, "subjectname", NULL);
-    char* issuername = g_key_file_get_string(tlscerts, fingerprint, "issuername", NULL);
-    char* notbefore = g_key_file_get_string(tlscerts, fingerprint, "start", NULL);
-    char* notafter = g_key_file_get_string(tlscerts, fingerprint, "end", NULL);
-    char* keyalg = g_key_file_get_string(tlscerts, fingerprint, "keyalg", NULL);
-    char* signaturealg = g_key_file_get_string(tlscerts, fingerprint, "signaturealg", NULL);
+    auto_gchar gchar* serialnumber = g_key_file_get_string(tlscerts, fingerprint, "serialnumber", NULL);
+    auto_gchar gchar* subjectname = g_key_file_get_string(tlscerts, fingerprint, "subjectname", NULL);
+    auto_gchar gchar* issuername = g_key_file_get_string(tlscerts, fingerprint, "issuername", NULL);
+    auto_gchar gchar* notbefore = g_key_file_get_string(tlscerts, fingerprint, "start", NULL);
+    auto_gchar gchar* notafter = g_key_file_get_string(tlscerts, fingerprint, "end", NULL);
+    auto_gchar gchar* keyalg = g_key_file_get_string(tlscerts, fingerprint, "keyalg", NULL);
+    auto_gchar gchar* signaturealg = g_key_file_get_string(tlscerts, fingerprint, "signaturealg", NULL);
 
     TLSCertificate* cert = tlscerts_new(fingerprint, version, serialnumber, subjectname, issuername, notbefore,
                                         notafter, keyalg, signaturealg, NULL);
-
-    free(serialnumber);
-    free(subjectname);
-    free(issuername);
-    free(notbefore);
-    free(notafter);
-    free(keyalg);
-    free(signaturealg);
-
     return cert;
 }
 
@@ -405,8 +378,7 @@ static void
 _save_tlscerts(void)
 {
     gsize g_data_size;
-    gchar* g_tlscerts_data = g_key_file_to_data(tlscerts, &g_data_size, NULL);
+    auto_gchar gchar* g_tlscerts_data = g_key_file_to_data(tlscerts, &g_data_size, NULL);
     g_file_set_contents(tlscerts_loc, g_tlscerts_data, g_data_size, NULL);
     g_chmod(tlscerts_loc, S_IRUSR | S_IWUSR);
-    g_free(g_tlscerts_data);
 }
