@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2022 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -55,12 +55,14 @@
 #include "event/common.h"
 #include "plugins/plugins.h"
 #include "ui/window_list.h"
+#include "ui/window.h"
 #include "tools/bookmark_ignore.h"
 #include "xmpp/xmpp.h"
 #include "xmpp/muc.h"
 #include "xmpp/chat_session.h"
 #include "xmpp/roster_list.h"
 #include "xmpp/avatar.h"
+#include "xmpp/vcard_funcs.h"
 
 #ifdef HAVE_LIBOTR
 #include "otr/otr.h"
@@ -101,7 +103,7 @@ sv_ev_login_account_success(char* account_name, gboolean secured)
 #endif
 
     log_database_init(account);
-
+    vcard_user_refresh();
     avatar_pep_subscribe();
 
     ui_handle_login_account_success(account, secured);
@@ -149,18 +151,16 @@ sv_ev_roster_received(void)
     // check pgp key valid if specified
     ProfAccount* account = accounts_get_account(account_name);
     if (account && account->pgp_keyid) {
-        char* err_str = NULL;
+        auto_char char* err_str = NULL;
         if (!p_gpg_valid_key(account->pgp_keyid, &err_str)) {
             cons_show_error("Invalid PGP key ID specified: %s, %s", account->pgp_keyid, err_str);
         }
-        free(err_str);
 
         // Redraw the screen after entry of the PGP secret key, but not init
         ProfWin* win = wins_get_current();
-        char* theme = prefs_get_string(PREF_THEME);
-        win_clear(win);
+        auto_gchar gchar* theme = prefs_get_string(PREF_THEME);
+        win_redraw(win);
         theme_init(theme);
-        g_free(theme);
         ui_resize();
         ui_show_roster();
     }
@@ -169,8 +169,8 @@ sv_ev_roster_received(void)
 
     // send initial presence
     resource_presence_t conn_presence = accounts_get_login_presence(account_name);
-    char* last_activity_str = accounts_get_last_activity(account_name);
-    char* status_message = accounts_get_login_status(account_name);
+    auto_gchar gchar* last_activity_str = accounts_get_last_activity(account_name);
+    auto_gchar gchar* status_message = accounts_get_login_status(account_name);
     int diff_secs = 0;
 
     if (prefs_get_boolean(PREF_LASTACTIVITY) && last_activity_str) {
@@ -190,9 +190,6 @@ sv_ev_roster_received(void)
     connection_set_presence_msg(status_message);
     cl_ev_presence_send(conn_presence, diff_secs);
 
-    g_free(status_message);
-    g_free(last_activity_str);
-
     const char* fulljid = connection_get_fulljid();
     plugins_on_connect(account_name, fulljid);
 }
@@ -278,7 +275,7 @@ sv_ev_room_history(ProfMessage* message)
         // check if this message was sent while we were offline.
         // if so, treat it as a new message rather than a history event.
         char* account_name = session_get_account_name();
-        char* last_activity = accounts_get_last_activity(account_name);
+        auto_gchar gchar* last_activity = accounts_get_last_activity(account_name);
         int msg_is_new = 0;
 
         if (last_activity) {
@@ -292,7 +289,6 @@ sv_ev_room_history(ProfMessage* message)
                 msg_is_new = (time_diff > 0);
                 g_date_time_unref(lastdt);
             }
-            g_free(last_activity);
 
             if (msg_is_new) {
                 sv_ev_room_message(message);
@@ -399,10 +395,9 @@ sv_ev_room_message(ProfMessage* message)
     mucwin->last_msg_timestamp = g_date_time_new_now_local();
 
     if (prefs_do_room_notify(is_current, mucwin->roomjid, mynick, message->from_jid->resourcepart, message->plain, mention, triggers != NULL)) {
-        Jid* jidp = jid_create(mucwin->roomjid);
+        auto_jid Jid* jidp = jid_create(mucwin->roomjid);
         if (jidp) {
             notify_room_message(message->from_jid->resourcepart, jidp->localpart, num, message->plain);
-            jid_destroy(jidp);
         }
     }
 
@@ -431,7 +426,7 @@ sv_ev_incoming_private_message(ProfMessage* message)
 
     _clean_incoming_message(message);
     privwin_incoming_msg(privatewin, message);
-    log_database_add_incoming(message);
+    // Intentionally skipping log to DB because we can't authenticate the sender
     chat_log_msg_in(message);
 
     plugins_post_priv_message_display(message->from_jid->fulljid, message->plain);
@@ -455,6 +450,7 @@ sv_ev_delayed_private_message(ProfMessage* message)
 
     _clean_incoming_message(message);
     privwin_incoming_msg(privatewin, message);
+    // Intentionally skipping log to DB because we can't authenticate the sender
     chat_log_msg_in(message);
 
     plugins_post_priv_message_display(message->from_jid->fulljid, message->plain);
@@ -631,22 +627,26 @@ sv_ev_incoming_message(ProfMessage* message)
     char* looking_for_jid = message->from_jid->barejid;
 
     if (message->is_mam) {
-        char* mybarejid = connection_get_barejid();
+        auto_char char* mybarejid = connection_get_barejid();
         if (g_strcmp0(mybarejid, message->from_jid->barejid) == 0) {
             if (message->to_jid) {
                 looking_for_jid = message->to_jid->barejid;
             }
         }
-        free(mybarejid);
     }
 
     chatwin = wins_get_chat(looking_for_jid);
 
     if (!chatwin) {
-        ProfWin* window = wins_new_chat(looking_for_jid);
-        chatwin = (ProfChatWin*)window;
+        chatwin = chatwin_new(looking_for_jid);
+        ProfWin* window = (ProfWin*)chatwin;
         new_win = TRUE;
 
+        if (prefs_get_boolean(PREF_MAM)) {
+            win_print_loading_history(window);
+            iq_mam_request(chatwin, g_date_time_add_seconds(message->timestamp, 0)); // copy timestamp
+        }
+
 #ifdef HAVE_OMEMO
         if (!message->is_mam) {
             if (omemo_automatic_start(message->from_jid->barejid)) {
@@ -682,8 +682,7 @@ sv_ev_incoming_carbon(ProfMessage* message)
     gboolean new_win = FALSE;
     ProfChatWin* chatwin = wins_get_chat(message->from_jid->barejid);
     if (!chatwin) {
-        ProfWin* window = wins_new_chat(message->from_jid->barejid);
-        chatwin = (ProfChatWin*)window;
+        chatwin = chatwin_new(message->from_jid->barejid);
         new_win = TRUE;
 
 #ifdef HAVE_OMEMO
@@ -896,16 +895,14 @@ sv_ev_room_occupant_offline(const char* const room, const char* const nick,
 {
     muc_roster_remove(room, nick);
 
-    char* muc_status_pref = prefs_get_string(PREF_STATUSES_MUC);
+    auto_gchar gchar* muc_status_pref = prefs_get_string(PREF_STATUSES_MUC);
     ProfMucWin* mucwin = wins_get_muc(room);
     if (mucwin && (g_strcmp0(muc_status_pref, "none") != 0)) {
         mucwin_occupant_offline(mucwin, nick);
     }
-    g_free(muc_status_pref);
 
-    Jid* jidp = jid_create_from_bare_and_resource(room, nick);
+    auto_jid Jid* jidp = jid_create_from_bare_and_resource(room, nick);
     ProfPrivateWin* privwin = wins_get_private(jidp->fulljid);
-    jid_destroy(jidp);
     if (privwin != NULL) {
         privwin_occupant_offline(privwin);
     }
@@ -924,9 +921,8 @@ sv_ev_room_occupent_kicked(const char* const room, const char* const nick, const
         mucwin_occupant_kicked(mucwin, nick, actor, reason);
     }
 
-    Jid* jidp = jid_create_from_bare_and_resource(room, nick);
+    auto_jid Jid* jidp = jid_create_from_bare_and_resource(room, nick);
     ProfPrivateWin* privwin = wins_get_private(jidp->fulljid);
-    jid_destroy(jidp);
     if (privwin != NULL) {
         privwin_occupant_kicked(privwin, actor, reason);
     }
@@ -945,13 +941,11 @@ sv_ev_room_occupent_banned(const char* const room, const char* const nick, const
         mucwin_occupant_banned(mucwin, nick, actor, reason);
     }
 
-    Jid* jidp = jid_create_from_bare_and_resource(room, nick);
+    auto_jid Jid* jidp = jid_create_from_bare_and_resource(room, nick);
 
     muc_members_remove(room, jidp->fulljid);
     ProfPrivateWin* privwin = wins_get_private(jidp->fulljid);
 
-    jid_destroy(jidp);
-
     if (privwin != NULL) {
         privwin_occupant_banned(privwin, actor, reason);
     }
@@ -1004,11 +998,10 @@ sv_ev_muc_self_online(const char* const room, const char* const nick, gboolean c
             ui_room_join(room, TRUE);
         }
 
-        Jid* jidp = jid_create(room);
+        auto_jid Jid* jidp = jid_create(room);
         if (jidp->domainpart) {
             muc_confserver_add(jidp->domainpart);
         }
-        jid_destroy(jidp);
 
         iq_room_info_request(room, FALSE);
 
@@ -1101,14 +1094,13 @@ sv_ev_muc_occupant_online(const char* const room, const char* const nick, const
     }
 
     // handle nickname change
-    char* old_nick = muc_roster_nick_change_complete(room, nick);
+    auto_char char* old_nick = muc_roster_nick_change_complete(room, nick);
     if (old_nick) {
         ProfMucWin* mucwin = wins_get_muc(room);
         if (mucwin) {
             mucwin_occupant_nick_change(mucwin, old_nick, nick);
             wins_private_nick_change(mucwin->roomjid, old_nick, nick);
         }
-        free(old_nick);
 
         occupantswin_occupants(room);
         rosterwin_roster();
@@ -1117,17 +1109,15 @@ sv_ev_muc_occupant_online(const char* const room, const char* const nick, const
 
     // joined room
     if (!occupant) {
-        char* muc_status_pref = prefs_get_string(PREF_STATUSES_MUC);
+        auto_gchar gchar* muc_status_pref = prefs_get_string(PREF_STATUSES_MUC);
         ProfMucWin* mucwin = wins_get_muc(room);
         if (mucwin && g_strcmp0(muc_status_pref, "none") != 0) {
             mucwin_occupant_online(mucwin, nick, role, affiliation, show, status);
         }
-        g_free(muc_status_pref);
 
         if (mucwin) {
-            Jid* jidp = jid_create_from_bare_and_resource(mucwin->roomjid, nick);
+            auto_jid Jid* jidp = jid_create_from_bare_and_resource(mucwin->roomjid, nick);
             ProfPrivateWin* privwin = wins_get_private(jidp->fulljid);
-            jid_destroy(jidp);
             if (privwin) {
                 privwin_occupant_online(privwin);
             }
@@ -1140,12 +1130,11 @@ sv_ev_muc_occupant_online(const char* const room, const char* const nick, const
 
     // presence updated
     if (updated) {
-        char* muc_status_pref = prefs_get_string(PREF_STATUSES_MUC);
+        auto_gchar gchar* muc_status_pref = prefs_get_string(PREF_STATUSES_MUC);
         ProfMucWin* mucwin = wins_get_muc(room);
         if (mucwin && (g_strcmp0(muc_status_pref, "all") == 0)) {
             mucwin_occupant_presence(mucwin, nick, show, status);
         }
-        g_free(muc_status_pref);
         occupantswin_occupants(room);
 
         // presence unchanged, check for role/affiliation change
@@ -1196,7 +1185,7 @@ sv_ev_certfail(const char* const errormsg, const TLSCertificate* cert)
     cons_show("");
     ui_update();
 
-    char* cmd = ui_get_line();
+    auto_char char* cmd = ui_get_line();
 
     while ((g_strcmp0(cmd, "/tls allow") != 0)
            && (g_strcmp0(cmd, "/tls always") != 0)
@@ -1214,7 +1203,6 @@ sv_ev_certfail(const char* const errormsg, const TLSCertificate* cert)
     if (g_strcmp0(cmd, "/tls allow") == 0) {
         cons_show("Continuing with connection.");
         tlscerts_set_current(cert->fingerprint);
-        free(cmd);
         return 1;
     } else if (g_strcmp0(cmd, "/tls always") == 0) {
         cons_show("Adding %s to trusted certificates.", cert->fingerprint);
@@ -1222,23 +1210,20 @@ sv_ev_certfail(const char* const errormsg, const TLSCertificate* cert)
             tlscerts_add(cert);
             cafile_add(cert);
         }
-        free(cmd);
         return 1;
     } else if (g_strcmp0(cmd, "/quit") == 0) {
         prof_set_quit();
-        free(cmd);
-        return 0;
-    } else {
-        cons_show("Aborting connection.");
-        free(cmd);
         return 0;
     }
+
+    cons_show("Aborting connection.");
+    return 0;
 }
 
 void
 sv_ev_lastactivity_response(const char* const from, const int seconds, const char* const msg)
 {
-    Jid* jidp = jid_create(from);
+    auto_jid Jid* jidp = jid_create(from);
 
     if (!jidp) {
         return;
@@ -1247,10 +1232,9 @@ sv_ev_lastactivity_response(const char* const from, const int seconds, const cha
     GDateTime* now = g_date_time_new_now_local();
     GDateTime* active = g_date_time_add_seconds(now, 0 - seconds);
 
-    gchar* date_fmt = NULL;
-    char* time_pref = prefs_get_string(PREF_TIME_LASTACTIVITY);
+    auto_gchar gchar* date_fmt = NULL;
+    auto_gchar gchar* time_pref = prefs_get_string(PREF_TIME_LASTACTIVITY);
     date_fmt = g_date_time_format(active, time_pref);
-    g_free(time_pref);
     assert(date_fmt != NULL);
 
     // full jid - last activity
@@ -1301,8 +1285,6 @@ sv_ev_lastactivity_response(const char* const from, const int seconds, const cha
 
     g_date_time_unref(now);
     g_date_time_unref(active);
-    g_free(date_fmt);
-    jid_destroy(jidp);
 }
 
 void
@@ -1312,7 +1294,7 @@ sv_ev_bookmark_autojoin(Bookmark* bookmark)
         return;
     }
 
-    char* nick = NULL;
+    auto_char char* nick = NULL;
 
     if (bookmark->nick) {
         nick = strdup(bookmark->nick);
@@ -1325,24 +1307,21 @@ sv_ev_bookmark_autojoin(Bookmark* bookmark)
 
     log_debug("Autojoin %s with nick=%s", bookmark->barejid, nick);
     if (!muc_active(bookmark->barejid)) {
-        presence_join_room(bookmark->barejid, nick, bookmark->password);
         muc_join(bookmark->barejid, nick, bookmark->password, TRUE);
+        presence_join_room(bookmark->barejid, nick, bookmark->password);
         iq_room_affiliation_list(bookmark->barejid, "member", false);
         iq_room_affiliation_list(bookmark->barejid, "admin", false);
         iq_room_affiliation_list(bookmark->barejid, "owner", false);
     }
-
-    free(nick);
 }
 
 static void
 _cut(ProfMessage* message, const char* cut)
 {
     if (strstr(message->plain, cut)) {
-        char** split = g_strsplit(message->plain, cut, -1);
+        auto_gcharv gchar** split = g_strsplit(message->plain, cut, -1);
         free(message->plain);
         message->plain = g_strjoinv("", split);
-        g_strfreev(split);
     }
 }
 
