@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2019 Paul Fariello <paul@fariello.eu>
- * Copyright (C) 2019 - 2022 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -83,7 +83,6 @@ static char* _omemo_fingerprint(ec_public_key* identity, gboolean formatted);
 static unsigned char* _omemo_fingerprint_decode(const char* const fingerprint, size_t* len);
 static char* _omemo_unformat_fingerprint(const char* const fingerprint_formatted);
 static void _cache_device_identity(const char* const jid, uint32_t device_id, ec_public_key* identity);
-static void _g_hash_table_free(GHashTable* hash_table);
 static void _acquire_sender_devices_list(void);
 
 typedef gboolean (*OmemoDeviceListHandler)(const char* const jid, GList* device_list);
@@ -221,7 +220,7 @@ omemo_on_connect(ProfAccount* account)
         .get_local_registration_id = get_local_registration_id,
         .save_identity = save_identity,
         .is_trusted_identity = is_trusted_identity,
-        .destroy_func = NULL,
+        .destroy_func = (void (*)(void*))identity_key_store_destroy,
         .user_data = &omemo_ctx.identity_key_store
     };
     signal_protocol_store_context_set_identity_key_store(omemo_ctx.store, &identity_key_store);
@@ -229,9 +228,9 @@ omemo_on_connect(ProfAccount* account)
     loaded = FALSE;
     omemo_ctx.device_list = g_hash_table_new_full(g_str_hash, g_str_equal, free, (GDestroyNotify)g_list_free);
     omemo_ctx.device_list_handler = g_hash_table_new_full(g_str_hash, g_str_equal, free, NULL);
-    omemo_ctx.known_devices = g_hash_table_new_full(g_str_hash, g_str_equal, free, (GDestroyNotify)_g_hash_table_free);
+    omemo_ctx.known_devices = g_hash_table_new_full(g_str_hash, g_str_equal, free, (GDestroyNotify)glib_hash_table_free);
 
-    gchar* omemo_dir = files_file_in_account_data_path(DIR_OMEMO, account->jid, NULL);
+    auto_gchar gchar* omemo_dir = files_file_in_account_data_path(DIR_OMEMO, account->jid, NULL);
     if (!omemo_dir) {
         log_error("[OMEMO] failed creating directory");
         return;
@@ -246,8 +245,6 @@ omemo_on_connect(ProfAccount* account)
     omemo_ctx.known_devices_filename = g_string_new(omemo_dir);
     g_string_append(omemo_ctx.known_devices_filename, "/known_devices.txt");
 
-    g_free(omemo_dir);
-
     omemo_devicelist_subscribe();
 
     omemo_ctx.identity_keyfile = g_key_file_new();
@@ -306,9 +303,11 @@ omemo_on_disconnect(void)
         return;
     }
 
-    _g_hash_table_free(omemo_ctx.signed_pre_key_store);
-    _g_hash_table_free(omemo_ctx.pre_key_store);
-    _g_hash_table_free(omemo_ctx.device_list_handler);
+    glib_hash_table_free(omemo_ctx.signed_pre_key_store);
+    glib_hash_table_free(omemo_ctx.pre_key_store);
+    glib_hash_table_free(omemo_ctx.known_devices);
+    glib_hash_table_free(omemo_ctx.device_list_handler);
+    glib_hash_table_free(omemo_ctx.device_list);
 
     g_string_free(omemo_ctx.identity_filename, TRUE);
     g_key_file_free(omemo_ctx.identity_keyfile);
@@ -316,9 +315,18 @@ omemo_on_disconnect(void)
     g_key_file_free(omemo_ctx.trust_keyfile);
     g_string_free(omemo_ctx.sessions_filename, TRUE);
     g_key_file_free(omemo_ctx.sessions_keyfile);
-    _g_hash_table_free(omemo_ctx.session_store);
+    glib_hash_table_free(omemo_ctx.session_store);
     g_string_free(omemo_ctx.known_devices_filename, TRUE);
     g_key_file_free(omemo_ctx.known_devices_keyfile);
+
+    signal_protocol_store_context_destroy(omemo_ctx.store);
+    ec_public_key* pub = ratchet_identity_key_pair_get_public(omemo_ctx.identity_key_pair);
+    ec_private_key* priv = ratchet_identity_key_pair_get_private(omemo_ctx.identity_key_pair);
+    ratchet_identity_key_pair_destroy((signal_type_base*)omemo_ctx.identity_key_pair);
+    ec_private_key_destroy((signal_type_base*)priv);
+    ec_public_key_destroy((signal_type_base*)pub);
+
+    signal_context_destroy(omemo_ctx.signal);
 }
 
 void
@@ -340,14 +348,12 @@ omemo_generate_crypto_materials(ProfAccount* account)
     signal_protocol_key_helper_generate_identity_key_pair(&omemo_ctx.identity_key_pair, omemo_ctx.signal);
 
     ec_public_key_serialize(&omemo_ctx.identity_key_store.public, ratchet_identity_key_pair_get_public(omemo_ctx.identity_key_pair));
-    char* identity_key_public = g_base64_encode(signal_buffer_data(omemo_ctx.identity_key_store.public), signal_buffer_len(omemo_ctx.identity_key_store.public));
+    auto_gchar gchar* identity_key_public = g_base64_encode(signal_buffer_data(omemo_ctx.identity_key_store.public), signal_buffer_len(omemo_ctx.identity_key_store.public));
     g_key_file_set_string(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_IDENTITY_KEY_PUBLIC, identity_key_public);
-    g_free(identity_key_public);
 
     ec_private_key_serialize(&omemo_ctx.identity_key_store.private, ratchet_identity_key_pair_get_private(omemo_ctx.identity_key_pair));
-    char* identity_key_private = g_base64_encode(signal_buffer_data(omemo_ctx.identity_key_store.private), signal_buffer_len(omemo_ctx.identity_key_store.private));
+    auto_gchar gchar* identity_key_private = g_base64_encode(signal_buffer_data(omemo_ctx.identity_key_store.private), signal_buffer_len(omemo_ctx.identity_key_store.private));
     g_key_file_set_string(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_IDENTITY_KEY_PRIVATE, identity_key_private);
-    g_free(identity_key_private);
 
     /* Registration ID */
     signal_protocol_key_helper_generate_registration_id(&omemo_ctx.registration_id, 0, omemo_ctx.signal);
@@ -384,12 +390,10 @@ omemo_publish_crypto_materials(void)
 static void
 _acquire_sender_devices_list(void)
 {
-    char* barejid = connection_get_barejid();
+    auto_char char* barejid = connection_get_barejid();
 
     g_hash_table_insert(omemo_ctx.device_list_handler, strdup(barejid), _handle_own_device_list);
     omemo_devicelist_request(barejid);
-
-    free(barejid);
 }
 
 void
@@ -422,11 +426,10 @@ omemo_start_session(const char* const barejid)
             log_debug("[OMEMO] missing device list for %s", barejid);
             // Own devices are handled by _handle_own_device_list
             // We won't add _handle_device_list_start_session for ourself
-            char* mybarejid = connection_get_barejid();
+            auto_char char* mybarejid = connection_get_barejid();
             if (g_strcmp0(mybarejid, barejid) != 0) {
                 g_hash_table_insert(omemo_ctx.device_list_handler, strdup(barejid), _handle_device_list_start_session);
             }
-            free(mybarejid);
             omemo_devicelist_request(barejid);
             return;
         }
@@ -444,9 +447,8 @@ omemo_start_muc_sessions(const char* const roomjid)
     GList* members = muc_members(roomjid);
     GList* iter;
     for (iter = members; iter != NULL; iter = iter->next) {
-        Jid* jid = jid_create(iter->data);
+        auto_jid Jid* jid = jid_create(iter->data);
         omemo_start_session(jid->barejid);
-        jid_destroy(jid);
     }
     g_list_free(members);
 }
@@ -543,8 +545,8 @@ omemo_prekeys(GList** prekeys, GList** ids, GList** lengths)
 void
 omemo_set_device_list(const char* const from, GList* device_list)
 {
-    log_debug("[OMEMO] Setting device list for %s", from);
-    Jid* jid;
+    log_debug("[OMEMO] Setting device list for %s", STR_MAYBE_NULL(from));
+    auto_jid Jid* jid;
     if (from) {
         jid = jid_create(from);
     } else {
@@ -560,11 +562,12 @@ omemo_set_device_list(const char* const from, GList* device_list)
             g_hash_table_remove(omemo_ctx.device_list_handler, jid->barejid);
         }
     } else {
-        log_debug("[OMEMO] No Device List Handler for %s", from);
+        log_debug("[OMEMO] No Device List Handler for %s", STR_MAYBE_NULL(from));
     }
 
     // OMEMO trustmode ToFu
-    if (g_strcmp0(prefs_get_string(PREF_OMEMO_TRUST_MODE), "firstusage") == 0) {
+    auto_gchar gchar* trust_mode = prefs_get_string(PREF_OMEMO_TRUST_MODE);
+    if (g_strcmp0(trust_mode, "firstusage") == 0) {
         log_debug("[OMEMO] Checking firstusage state for %s", jid->barejid);
         GHashTable* trusted = g_hash_table_lookup(omemo_ctx.identity_key_store.trusted, jid->barejid);
         if (trusted) {
@@ -591,7 +594,6 @@ omemo_set_device_list(const char* const from, GList* device_list)
             }
         }
     }
-    jid_destroy(jid);
 }
 
 GKeyFile*
@@ -660,24 +662,29 @@ omemo_start_device_session(const char* const jid, uint32_t device_id,
                            const unsigned char* const identity_key_raw, size_t identity_key_len)
 {
     log_debug("[OMEMO] Starting device session for %s with device %d", jid, device_id);
-    signal_protocol_address address = {
+    signal_protocol_address jid_address = {
         .name = jid,
         .name_len = strlen(jid),
         .device_id = device_id,
     };
 
+    signal_protocol_address* address = NULL;
+    session_pre_key_bundle* bundle = NULL;
+    ec_public_key* prekey_public = NULL;
+    ec_public_key* signed_prekey = NULL;
+    session_builder* builder = NULL;
     ec_public_key* identity_key;
     curve_decode_point(&identity_key, identity_key_raw, identity_key_len, omemo_ctx.signal);
     _cache_device_identity(jid, device_id, identity_key);
 
-    gboolean trusted = is_trusted_identity(&address, (uint8_t*)identity_key_raw, identity_key_len, &omemo_ctx.identity_key_store);
+    gboolean trusted = is_trusted_identity(&jid_address, (uint8_t*)identity_key_raw, identity_key_len, &omemo_ctx.identity_key_store);
     log_debug("[OMEMO] Trust %s (%d): %d", jid, device_id, trusted);
 
-    if ((g_strcmp0(prefs_get_string(PREF_OMEMO_TRUST_MODE), "blind") == 0) && !trusted) {
-        char* fp = _omemo_fingerprint(identity_key, TRUE);
+    auto_gchar gchar* trust_mode = prefs_get_string(PREF_OMEMO_TRUST_MODE);
+    if ((g_strcmp0(trust_mode, "blind") == 0) && !trusted) {
+        auto_char char* fp = _omemo_fingerprint(identity_key, TRUE);
         cons_show("Blind trust for %s device %d (%s)", jid, device_id, fp);
         omemo_trust(jid, fp);
-        free(fp);
         trusted = TRUE;
     }
 
@@ -686,18 +693,15 @@ omemo_start_device_session(const char* const jid, uint32_t device_id,
         goto out;
     }
 
-    if (!contains_session(&address, omemo_ctx.session_store)) {
-        log_debug("[OMEMO] There is no Session for %s ( %d) ,... building session.", address.name, address.device_id);
+    if (!contains_session(&jid_address, omemo_ctx.session_store)) {
+        log_debug("[OMEMO] There is no Session for %s ( %d) ,… building session.", jid_address.name, jid_address.device_id);
         int res;
-        session_pre_key_bundle* bundle;
-        signal_protocol_address* address;
 
         address = malloc(sizeof(signal_protocol_address));
         address->name = strdup(jid);
         address->name_len = strlen(jid);
         address->device_id = device_id;
 
-        session_builder* builder;
         res = session_builder_create(&builder, omemo_ctx.store, address, omemo_ctx.signal);
         if (res != 0) {
             log_error("[OMEMO] cannot create session builder for %s device %d", jid, device_id);
@@ -709,9 +713,7 @@ omemo_start_device_session(const char* const jid, uint32_t device_id,
         prekey_index %= g_list_length(prekeys);
         omemo_key_t* prekey = g_list_nth_data(prekeys, prekey_index);
 
-        ec_public_key* prekey_public;
         curve_decode_point(&prekey_public, prekey->data, prekey->length, omemo_ctx.signal);
-        ec_public_key* signed_prekey;
         curve_decode_point(&signed_prekey, signed_prekey_raw, signed_prekey_len, omemo_ctx.signal);
 
         res = session_pre_key_bundle_create(&bundle, 0, device_id, prekey->id, prekey_public, signed_prekey_id, signed_prekey, signature, signature_len, identity_key);
@@ -732,6 +734,16 @@ omemo_start_device_session(const char* const jid, uint32_t device_id,
     }
 
 out:
+    if (bundle)
+        session_pre_key_bundle_destroy((signal_type_base*)bundle);
+    if (signed_prekey)
+        ec_public_key_destroy((signal_type_base*)signed_prekey);
+    if (prekey_public)
+        ec_public_key_destroy((signal_type_base*)prekey_public);
+    session_builder_free(builder);
+    if (address)
+        free((void*)address->name);
+    free(address);
     SIGNAL_UNREF(identity_key);
 }
 
@@ -740,7 +752,7 @@ omemo_on_message_send(ProfWin* win, const char* const message, gboolean request_
 {
     char* id = NULL;
     int res;
-    Jid* jid = jid_create(connection_get_fulljid());
+    auto_jid Jid* jid = jid_create(connection_get_fulljid());
     GList* keys = NULL;
 
     unsigned char* key;
@@ -776,9 +788,8 @@ omemo_on_message_send(ProfWin* win, const char* const message, gboolean request_
         GList* members = muc_members(mucwin->roomjid);
         GList* iter;
         for (iter = members; iter != NULL; iter = iter->next) {
-            Jid* jid = jid_create(iter->data);
-            recipients = g_list_append(recipients, strdup(jid->barejid));
-            jid_destroy(jid);
+            auto_jid Jid* jidp = jid_create(iter->data);
+            recipients = g_list_append(recipients, strdup(jidp->barejid));
         }
         g_list_free(members);
     } else {
@@ -918,7 +929,6 @@ omemo_on_message_send(ProfWin* win, const char* const message, gboolean request_
     }
 
 out:
-    jid_destroy(jid);
     g_list_free_full(keys, (GDestroyNotify)omemo_key_free);
     free(ciphertext);
     gcry_free(key);
@@ -935,11 +945,11 @@ omemo_on_message_recv(const char* const from_jid, uint32_t sid,
                       const unsigned char* const payload, size_t payload_len, gboolean muc, gboolean* trusted)
 {
     unsigned char* plaintext = NULL;
-    Jid* sender = NULL;
-    Jid* from = jid_create(from_jid);
+    auto_jid Jid* sender = NULL;
+    auto_jid Jid* from = jid_create(from_jid);
     if (!from) {
         log_error("[OMEMO][RECV] Invalid jid %s", from_jid);
-        goto out;
+        return NULL;
     }
 
     int res;
@@ -954,7 +964,7 @@ omemo_on_message_recv(const char* const from_jid, uint32_t sid,
 
     if (!key) {
         log_warning("[OMEMO][RECV] received a message with no corresponding key");
-        goto out;
+        return NULL;
     }
 
     if (muc) {
@@ -970,7 +980,7 @@ omemo_on_message_recv(const char* const from_jid, uint32_t sid,
         g_list_free(roster);
         if (!sender) {
             log_warning("[OMEMO][RECV] cannot find MUC message sender fulljid");
-            goto out;
+            return NULL;
         }
     } else {
         sender = jid_create(from->barejid);
@@ -987,7 +997,7 @@ omemo_on_message_recv(const char* const from_jid, uint32_t sid,
     res = session_cipher_create(&cipher, omemo_ctx.store, &address, omemo_ctx.signal);
     if (res != 0) {
         log_error("[OMEMO][RECV] cannot create session cipher");
-        goto out;
+        return NULL;
     }
 
     if (key->prekey) {
@@ -1009,6 +1019,7 @@ omemo_on_message_recv(const char* const from_jid, uint32_t sid,
         ec_public_key_serialize(&identity_buffer, their_identity_key);
         *trusted = is_trusted_identity(&address, signal_buffer_data(identity_buffer),
                                        signal_buffer_len(identity_buffer), &omemo_ctx.identity_key_store);
+        signal_buffer_free(identity_buffer);
 
         /* Replace used pre_key in bundle */
         uint32_t pre_key_id = pre_key_signal_message_get_pre_key_id(message);
@@ -1045,13 +1056,13 @@ omemo_on_message_recv(const char* const from_jid, uint32_t sid,
     session_cipher_free(cipher);
     if (res != 0) {
         log_error("[OMEMO][RECV] cannot decrypt message key");
-        goto out;
+        return NULL;
     }
 
     if (signal_buffer_len(plaintext_key) != AES128_GCM_KEY_LENGTH + AES128_GCM_TAG_LENGTH) {
         log_error("[OMEMO][RECV] invalid key length");
         signal_buffer_free(plaintext_key);
-        goto out;
+        return NULL;
     }
 
     size_t plaintext_len = payload_len;
@@ -1063,15 +1074,11 @@ omemo_on_message_recv(const char* const from_jid, uint32_t sid,
     if (res != 0) {
         log_error("[OMEMO][RECV] cannot decrypt message: %s", gcry_strerror(res));
         free(plaintext);
-        plaintext = NULL;
-        goto out;
+        return NULL;
     }
 
     plaintext[plaintext_len] = '\0';
 
-out:
-    jid_destroy(from);
-    jid_destroy(sender);
     return (char*)plaintext;
 }
 
@@ -1299,19 +1306,19 @@ omemo_untrust(const char* const jid, const char* const fingerprint_formatted)
     }
     free(identity);
 
-    char* fingerprint = _omemo_unformat_fingerprint(fingerprint_formatted);
+    auto_char char* fingerprint = _omemo_unformat_fingerprint(fingerprint_formatted);
 
     /* Remove existing session */
     GHashTable* known_identities = g_hash_table_lookup(omemo_ctx.known_devices, jid);
     if (!known_identities) {
         log_error("[OMEMO] cannot find known device while untrusting a fingerprint");
-        goto out;
+        return;
     }
 
     uint32_t device_id = GPOINTER_TO_INT(g_hash_table_lookup(known_identities, fingerprint));
     if (!device_id) {
         log_error("[OMEMO] cannot find device id while untrusting a fingerprint");
-        goto out;
+        return;
     }
     signal_protocol_address address = {
         .name = jid,
@@ -1322,13 +1329,9 @@ omemo_untrust(const char* const jid, const char* const fingerprint_formatted)
     delete_session(&address, omemo_ctx.session_store);
 
     /* Remove from keyfile */
-    char* device_id_str = g_strdup_printf("%d", device_id);
+    auto_gchar gchar* device_id_str = g_strdup_printf("%d", device_id);
     g_key_file_remove_key(omemo_ctx.trust_keyfile, jid, device_id_str, NULL);
-    g_free(device_id_str);
     omemo_trust_keyfile_save();
-
-out:
-    free(fingerprint);
 }
 
 static void
@@ -1452,7 +1455,7 @@ omemo_automatic_start(const char* const recipient)
         }
     } else {
         // check global setting
-        char* pref_omemo_policy = prefs_get_string(PREF_OMEMO_POLICY);
+        auto_gchar gchar* pref_omemo_policy = prefs_get_string(PREF_OMEMO_POLICY);
 
         // pref defaults to manual
         policy = PROF_OMEMOPOLICY_AUTOMATIC;
@@ -1462,8 +1465,6 @@ omemo_automatic_start(const char* const recipient)
         } else if (strcmp(pref_omemo_policy, "always") == 0) {
             policy = PROF_OMEMOPOLICY_ALWAYS;
         }
-
-        g_free(pref_omemo_policy);
     }
 
     switch (policy) {
@@ -1517,27 +1518,25 @@ _load_identity(void)
 
     /* Identity key */
     error = NULL;
-    char* identity_key_public_b64 = g_key_file_get_string(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_IDENTITY_KEY_PUBLIC, &error);
+    auto_gchar gchar* identity_key_public_b64 = g_key_file_get_string(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_IDENTITY_KEY_PUBLIC, &error);
     if (!identity_key_public_b64) {
         log_error("[OMEMO] cannot load identity public key: %s", error->message);
         return FALSE;
     }
 
     size_t identity_key_public_len;
-    unsigned char* identity_key_public = g_base64_decode(identity_key_public_b64, &identity_key_public_len);
-    g_free(identity_key_public_b64);
+    auto_guchar guchar* identity_key_public = g_base64_decode(identity_key_public_b64, &identity_key_public_len);
     omemo_ctx.identity_key_store.public = signal_buffer_create(identity_key_public, identity_key_public_len);
 
     error = NULL;
-    char* identity_key_private_b64 = g_key_file_get_string(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_IDENTITY_KEY_PRIVATE, &error);
+    auto_gchar gchar* identity_key_private_b64 = g_key_file_get_string(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_IDENTITY, OMEMO_STORE_KEY_IDENTITY_KEY_PRIVATE, &error);
     if (!identity_key_private_b64) {
         log_error("[OMEMO] cannot load identity private key: %s", error->message);
         return FALSE;
     }
 
     size_t identity_key_private_len;
-    unsigned char* identity_key_private = g_base64_decode(identity_key_private_b64, &identity_key_private_len);
-    g_free(identity_key_private_b64);
+    auto_guchar guchar* identity_key_private = g_base64_decode(identity_key_private_b64, &identity_key_private_len);
     omemo_ctx.identity_key_store.private = signal_buffer_create(identity_key_private, identity_key_private_len);
 
     ec_public_key* public_key;
@@ -1546,9 +1545,6 @@ _load_identity(void)
     curve_decode_private_point(&private_key, identity_key_private, identity_key_private_len, omemo_ctx.signal);
     ratchet_identity_key_pair_create(&omemo_ctx.identity_key_pair, public_key, private_key);
 
-    g_free(identity_key_public);
-    g_free(identity_key_private);
-
     char** keys = NULL;
     int i;
     /* Pre keys */
@@ -1556,12 +1552,10 @@ _load_identity(void)
     keys = g_key_file_get_keys(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_PREKEYS, NULL, NULL);
     if (keys) {
         for (i = 0; keys[i] != NULL; i++) {
-            char* pre_key_b64 = g_key_file_get_string(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_PREKEYS, keys[i], NULL);
+            auto_gchar gchar* pre_key_b64 = g_key_file_get_string(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_PREKEYS, keys[i], NULL);
             size_t pre_key_len;
-            unsigned char* pre_key = g_base64_decode(pre_key_b64, &pre_key_len);
-            g_free(pre_key_b64);
+            auto_guchar guchar* pre_key = g_base64_decode(pre_key_b64, &pre_key_len);
             signal_buffer* buffer = signal_buffer_create(pre_key, pre_key_len);
-            g_free(pre_key);
             g_hash_table_insert(omemo_ctx.pre_key_store, GINT_TO_POINTER(strtoul(keys[i], NULL, 10)), buffer);
         }
 
@@ -1578,12 +1572,10 @@ _load_identity(void)
     keys = g_key_file_get_keys(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_SIGNED_PREKEYS, NULL, NULL);
     if (keys) {
         for (i = 0; keys[i] != NULL; i++) {
-            char* signed_pre_key_b64 = g_key_file_get_string(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_SIGNED_PREKEYS, keys[i], NULL);
+            auto_gchar gchar* signed_pre_key_b64 = g_key_file_get_string(omemo_ctx.identity_keyfile, OMEMO_STORE_GROUP_SIGNED_PREKEYS, keys[i], NULL);
             size_t signed_pre_key_len;
-            unsigned char* signed_pre_key = g_base64_decode(signed_pre_key_b64, &signed_pre_key_len);
-            g_free(signed_pre_key_b64);
+            auto_guchar guchar* signed_pre_key = g_base64_decode(signed_pre_key_b64, &signed_pre_key_len);
             signal_buffer* buffer = signal_buffer_create(signed_pre_key, signed_pre_key_len);
-            g_free(signed_pre_key);
             g_hash_table_insert(omemo_ctx.signed_pre_key_store, GINT_TO_POINTER(strtoul(keys[i], NULL, 10)), buffer);
             omemo_ctx.signed_pre_key_id = strtoul(keys[i], NULL, 10);
         }
@@ -1605,96 +1597,87 @@ _load_identity(void)
 static void
 _load_trust(void)
 {
-    char** keys = NULL;
-    char** groups = g_key_file_get_groups(omemo_ctx.trust_keyfile, NULL);
-    if (groups) {
-        int i;
-        for (i = 0; groups[i] != NULL; i++) {
-            GHashTable* trusted;
+    auto_gcharv gchar** groups = g_key_file_get_groups(omemo_ctx.trust_keyfile, NULL);
 
-            trusted = g_hash_table_lookup(omemo_ctx.identity_key_store.trusted, groups[i]);
-            if (!trusted) {
-                trusted = g_hash_table_new_full(g_direct_hash, g_direct_equal, NULL, (GDestroyNotify)signal_buffer_free);
-                g_hash_table_insert(omemo_ctx.identity_key_store.trusted, strdup(groups[i]), trusted);
-            }
+    if (!groups) {
+        return;
+    }
 
-            keys = g_key_file_get_keys(omemo_ctx.trust_keyfile, groups[i], NULL, NULL);
-            int j;
-            for (j = 0; keys[j] != NULL; j++) {
-                char* key_b64 = g_key_file_get_string(omemo_ctx.trust_keyfile, groups[i], keys[j], NULL);
-                size_t key_len;
-                unsigned char* key = g_base64_decode(key_b64, &key_len);
-                g_free(key_b64);
-                signal_buffer* buffer = signal_buffer_create(key, key_len);
-                g_free(key);
-                uint32_t device_id = strtoul(keys[j], NULL, 10);
-                g_hash_table_insert(trusted, GINT_TO_POINTER(device_id), buffer);
-            }
-            g_strfreev(keys);
+    for (int i = 0; groups[i] != NULL; i++) {
+        GHashTable* trusted;
+
+        trusted = g_hash_table_lookup(omemo_ctx.identity_key_store.trusted, groups[i]);
+        if (!trusted) {
+            trusted = g_hash_table_new_full(g_direct_hash, g_direct_equal, NULL, (GDestroyNotify)signal_buffer_free);
+            g_hash_table_insert(omemo_ctx.identity_key_store.trusted, strdup(groups[i]), trusted);
+        }
+
+        auto_gcharv gchar** keys = g_key_file_get_keys(omemo_ctx.trust_keyfile, groups[i], NULL, NULL);
+        for (int j = 0; keys[j] != NULL; j++) {
+            auto_gchar gchar* key_b64 = g_key_file_get_string(omemo_ctx.trust_keyfile, groups[i], keys[j], NULL);
+            size_t key_len;
+            auto_guchar guchar* key = g_base64_decode(key_b64, &key_len);
+            signal_buffer* buffer = signal_buffer_create(key, key_len);
+            uint32_t device_id = strtoul(keys[j], NULL, 10);
+            g_hash_table_insert(trusted, GINT_TO_POINTER(device_id), buffer);
         }
-        g_strfreev(groups);
     }
 }
 
 static void
 _load_sessions(void)
 {
-    int i;
-    char** groups = g_key_file_get_groups(omemo_ctx.sessions_keyfile, NULL);
-    if (groups) {
-        for (i = 0; groups[i] != NULL; i++) {
-            int j;
-            GHashTable* device_store = NULL;
+    auto_gcharv gchar** groups = g_key_file_get_groups(omemo_ctx.sessions_keyfile, NULL);
 
-            device_store = g_hash_table_lookup(omemo_ctx.session_store, groups[i]);
-            if (!device_store) {
-                device_store = g_hash_table_new_full(g_direct_hash, g_direct_equal, NULL, (GDestroyNotify)signal_buffer_free);
-                g_hash_table_insert(omemo_ctx.session_store, strdup(groups[i]), device_store);
-            }
+    if (!groups) {
+        return;
+    }
 
-            char** keys = g_key_file_get_keys(omemo_ctx.sessions_keyfile, groups[i], NULL, NULL);
-            for (j = 0; keys[j] != NULL; j++) {
-                uint32_t id = strtoul(keys[j], NULL, 10);
-                char* record_b64 = g_key_file_get_string(omemo_ctx.sessions_keyfile, groups[i], keys[j], NULL);
-                size_t record_len;
-                unsigned char* record = g_base64_decode(record_b64, &record_len);
-                g_free(record_b64);
-                signal_buffer* buffer = signal_buffer_create(record, record_len);
-                g_free(record);
-                g_hash_table_insert(device_store, GINT_TO_POINTER(id), buffer);
-            }
-            g_strfreev(keys);
+    for (int i = 0; groups[i] != NULL; i++) {
+        GHashTable* device_store = NULL;
+
+        device_store = g_hash_table_lookup(omemo_ctx.session_store, groups[i]);
+        if (!device_store) {
+            device_store = g_hash_table_new_full(g_direct_hash, g_direct_equal, NULL, (GDestroyNotify)signal_buffer_free);
+            g_hash_table_insert(omemo_ctx.session_store, strdup(groups[i]), device_store);
+        }
+
+        auto_gcharv gchar** keys = g_key_file_get_keys(omemo_ctx.sessions_keyfile, groups[i], NULL, NULL);
+        for (int j = 0; keys[j] != NULL; j++) {
+            uint32_t id = strtoul(keys[j], NULL, 10);
+            auto_gchar gchar* record_b64 = g_key_file_get_string(omemo_ctx.sessions_keyfile, groups[i], keys[j], NULL);
+            size_t record_len;
+            auto_guchar guchar* record = g_base64_decode(record_b64, &record_len);
+            signal_buffer* buffer = signal_buffer_create(record, record_len);
+            g_hash_table_insert(device_store, GINT_TO_POINTER(id), buffer);
         }
-        g_strfreev(groups);
     }
 }
 
 static void
 _load_known_devices(void)
 {
-    int i;
-    char** groups = g_key_file_get_groups(omemo_ctx.known_devices_keyfile, NULL);
-    if (groups) {
-        for (i = 0; groups[i] != NULL; i++) {
-            int j;
-            GHashTable* known_identities = NULL;
+    auto_gcharv gchar** groups = g_key_file_get_groups(omemo_ctx.known_devices_keyfile, NULL);
 
-            known_identities = g_hash_table_lookup(omemo_ctx.known_devices, groups[i]);
-            if (!known_identities) {
-                known_identities = g_hash_table_new_full(g_str_hash, g_str_equal, free, NULL);
-                g_hash_table_insert(omemo_ctx.known_devices, strdup(groups[i]), known_identities);
-            }
+    if (!groups) {
+        return;
+    }
 
-            char** keys = g_key_file_get_keys(omemo_ctx.known_devices_keyfile, groups[i], NULL, NULL);
-            for (j = 0; keys[j] != NULL; j++) {
-                uint32_t device_id = strtoul(keys[j], NULL, 10);
-                char* fingerprint = g_key_file_get_string(omemo_ctx.known_devices_keyfile, groups[i], keys[j], NULL);
-                g_hash_table_insert(known_identities, strdup(fingerprint), GINT_TO_POINTER(device_id));
-                g_free(fingerprint);
-            }
-            g_strfreev(keys);
+    for (int i = 0; groups[i] != NULL; i++) {
+        GHashTable* known_identities = NULL;
+
+        known_identities = g_hash_table_lookup(omemo_ctx.known_devices, groups[i]);
+        if (!known_identities) {
+            known_identities = g_hash_table_new_full(g_str_hash, g_str_equal, free, NULL);
+            g_hash_table_insert(omemo_ctx.known_devices, strdup(groups[i]), known_identities);
+        }
+
+        auto_gcharv gchar** keys = g_key_file_get_keys(omemo_ctx.known_devices_keyfile, groups[i], NULL, NULL);
+        for (int j = 0; keys[j] != NULL; j++) {
+            uint32_t device_id = strtoul(keys[j], NULL, 10);
+            auto_gchar gchar* fingerprint = g_key_file_get_string(omemo_ctx.known_devices_keyfile, groups[i], keys[j], NULL);
+            g_hash_table_insert(known_identities, strdup(fingerprint), GINT_TO_POINTER(device_id));
         }
-        g_strfreev(groups);
     }
 }
 
@@ -1711,9 +1694,8 @@ _cache_device_identity(const char* const jid, uint32_t device_id, ec_public_key*
     log_debug("[OMEMO] cache identity for %s:%d: %s", jid, device_id, fingerprint);
     g_hash_table_insert(known_identities, strdup(fingerprint), GINT_TO_POINTER(device_id));
 
-    char* device_id_str = g_strdup_printf("%d", device_id);
+    auto_gchar gchar* device_id_str = g_strdup_printf("%d", device_id);
     g_key_file_set_string(omemo_ctx.known_devices_keyfile, jid, device_id_str, fingerprint);
-    g_free(device_id_str);
     omemo_known_devices_keyfile_save();
 
     Autocomplete ac = g_hash_table_lookup(omemo_ctx.fingerprint_ac, jid);
@@ -1728,13 +1710,6 @@ _cache_device_identity(const char* const jid, uint32_t device_id, ec_public_key*
     free(fingerprint);
 }
 
-static void
-_g_hash_table_free(GHashTable* hash_table)
-{
-    g_hash_table_remove_all(hash_table);
-    g_hash_table_unref(hash_table);
-}
-
 static void
 _generate_pre_keys(int count)
 {
@@ -1904,7 +1879,7 @@ char*
 omemo_qrcode_str()
 {
     char* mybarejid = connection_get_barejid();
-    char* fingerprint = omemo_own_fingerprint(TRUE);
+    char* fingerprint = omemo_own_fingerprint(FALSE);
     uint32_t sid = omemo_device_id();
 
     char* qrstr = g_strdup_printf("xmpp:%s?omemo-sid-%d=%s", mybarejid, sid, fingerprint);
