@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2018 - 2022 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2018 - 2023 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -53,10 +53,12 @@
 #include "tools/autocomplete.h"
 #include "ui/ui.h"
 
-#define PGP_SIGNATURE_HEADER "-----BEGIN PGP SIGNATURE-----"
-#define PGP_SIGNATURE_FOOTER "-----END PGP SIGNATURE-----"
-#define PGP_MESSAGE_HEADER   "-----BEGIN PGP MESSAGE-----"
-#define PGP_MESSAGE_FOOTER   "-----END PGP MESSAGE-----"
+#define PGP_SIGNATURE_HEADER  "-----BEGIN PGP SIGNATURE-----"
+#define PGP_SIGNATURE_FOOTER  "-----END PGP SIGNATURE-----"
+#define PGP_MESSAGE_HEADER    "-----BEGIN PGP MESSAGE-----"
+#define PGP_MESSAGE_FOOTER    "-----END PGP MESSAGE-----"
+#define PGP_PUBLIC_KEY_HEADER "-----BEGIN PGP PUBLIC KEY BLOCK-----"
+#define PGP_PUBLIC_KEY_FOOTER "-----END PGP PUBLIC KEY BLOCK-----"
 
 static const char* libversion = NULL;
 static GHashTable* pubkeys;
@@ -71,7 +73,9 @@ static Autocomplete key_ac;
 
 static char* _remove_header_footer(char* str, const char* const footer);
 static char* _add_header_footer(const char* const str, const char* const header, const char* const footer);
+static char* _gpgme_data_to_char(gpgme_data_t data);
 static void _save_pubkeys(void);
+static ProfPGPKey* _gpgme_key_to_ProfPGPKey(gpgme_key_t key);
 
 void
 _p_gpg_free_pubkeyid(ProfPGPPubKeyId* pubkeyid)
@@ -90,18 +94,16 @@ _p_gpg_passphrase_cb(void* hook, const char* uid_hint, const char* passphrase_in
     } else {
         GString* pass_term = g_string_new("");
 
-        char* password = ui_ask_pgp_passphrase(uid_hint, prev_was_bad);
+        auto_char char* password = ui_ask_pgp_passphrase(uid_hint, prev_was_bad);
         if (password) {
             g_string_append(pass_term, password);
-            free(password);
         }
 
         g_string_append(pass_term, "\n");
         if (passphrase_attempt) {
             free(passphrase_attempt);
         }
-        passphrase_attempt = pass_term->str;
-        g_string_free(pass_term, FALSE);
+        passphrase_attempt = g_string_free(pass_term, FALSE);
 
         gpgme_io_write(fd, passphrase_attempt, strlen(passphrase_attempt));
     }
@@ -175,25 +177,23 @@ p_gpg_on_connect(const char* const barejid)
 
     // load each keyid
     gsize len = 0;
-    gchar** jids = g_key_file_get_groups(pubkeyfile, &len);
+    auto_gcharv gchar** jids = g_key_file_get_groups(pubkeyfile, &len);
 
     gpgme_ctx_t ctx;
     gpgme_error_t error = gpgme_new(&ctx);
 
     if (error) {
         log_error("GPG: Failed to create gpgme context. %s %s", gpgme_strsource(error), gpgme_strerror(error));
-        g_strfreev(jids);
         return;
     }
 
     for (int i = 0; i < len; i++) {
         GError* gerr = NULL;
         gchar* jid = jids[i];
-        gchar* keyid = g_key_file_get_string(pubkeyfile, jid, "keyid", &gerr);
+        auto_gchar gchar* keyid = g_key_file_get_string(pubkeyfile, jid, "keyid", &gerr);
         if (gerr) {
             log_error("Error loading PGP key id for %s", jid);
             g_error_free(gerr);
-            g_free(keyid);
         } else {
             gpgme_key_t key = NULL;
             error = gpgme_get_key(ctx, keyid, &key, 0);
@@ -206,13 +206,11 @@ p_gpg_on_connect(const char* const barejid)
             pubkeyid->id = strdup(keyid);
             pubkeyid->received = FALSE;
             g_hash_table_replace(pubkeys, strdup(jid), pubkeyid);
-            g_free(keyid);
             gpgme_key_unref(key);
         }
     }
 
     gpgme_release(ctx);
-    g_strfreev(jids);
 
     _save_pubkeys();
 }
@@ -304,6 +302,21 @@ p_gpg_free_key(ProfPGPKey* key)
     }
 }
 
+/**
+ * Retrieve a list of GPG keys and create a hash table of ProfPGPKey objects.
+ *
+ * This function utilizes the GPGME library to retrieve both public and secret keys.
+ * It iterates over the keys and their subkeys to populate a hash table with ProfPGPKey objects.
+ * The key name is used as the key in the hash table, and the ProfPGPKey object is the corresponding value.
+ *
+ * @return A newly created GHashTable* containing ProfPGPKey objects, with the key name as the key.
+ *         Returns NULL if an error occurs during key retrieval or if memory allocation fails.
+ *
+ * @note The returned hash table should be released using p_gpg_free_keys()
+ *       when they are no longer needed to avoid memory leaks.
+ *
+ * @note This function may perform additional operations, such as autocomplete, related to the retrieved keys.
+ */
 GHashTable*
 p_gpg_list_keys(void)
 {
@@ -312,48 +325,23 @@ p_gpg_list_keys(void)
 
     gpgme_ctx_t ctx;
     error = gpgme_new(&ctx);
-
     if (error) {
-        log_error("GPG: Could not list keys. %s %s", gpgme_strsource(error), gpgme_strerror(error));
+        log_error("GPG: Could not create GPGME context. %s %s", gpgme_strsource(error), gpgme_strerror(error));
+        g_hash_table_destroy(result);
         return NULL;
     }
 
     error = gpgme_op_keylist_start(ctx, NULL, 0);
     if (error == GPG_ERR_NO_ERROR) {
         gpgme_key_t key;
+
         error = gpgme_op_keylist_next(ctx, &key);
         while (!error) {
-            gpgme_subkey_t sub = key->subkeys;
-
-            ProfPGPKey* p_pgpkey = p_gpg_key_new();
-            p_pgpkey->id = strdup(sub->keyid);
-            p_pgpkey->name = strdup(key->uids->uid);
-            p_pgpkey->fp = strdup(sub->fpr);
-            if (sub->can_encrypt)
-                p_pgpkey->encrypt = TRUE;
-            if (sub->can_authenticate)
-                p_pgpkey->authenticate = TRUE;
-            if (sub->can_certify)
-                p_pgpkey->certify = TRUE;
-            if (sub->can_sign)
-                p_pgpkey->sign = TRUE;
-
-            sub = sub->next;
-            while (sub) {
-                if (sub->can_encrypt)
-                    p_pgpkey->encrypt = TRUE;
-                if (sub->can_authenticate)
-                    p_pgpkey->authenticate = TRUE;
-                if (sub->can_certify)
-                    p_pgpkey->certify = TRUE;
-                if (sub->can_sign)
-                    p_pgpkey->sign = TRUE;
-
-                sub = sub->next;
+            ProfPGPKey* p_pgpkey = _gpgme_key_to_ProfPGPKey(key);
+            if (p_pgpkey != NULL) {
+                g_hash_table_insert(result, strdup(p_pgpkey->name), p_pgpkey);
             }
 
-            g_hash_table_insert(result, strdup(p_pgpkey->name), p_pgpkey);
-
             gpgme_key_unref(key);
             error = gpgme_op_keylist_next(ctx, &key);
         }
@@ -382,6 +370,7 @@ p_gpg_list_keys(void)
 
     gpgme_release(ctx);
 
+    // TODO: move autocomplete in other place
     autocomplete_clear(key_ac);
     GList* ids = g_hash_table_get_keys(result);
     GList* curr = ids;
@@ -423,7 +412,9 @@ p_gpg_valid_key(const char* const keyid, char** err_str)
     gpgme_error_t error = gpgme_new(&ctx);
     if (error) {
         log_error("GPG: Failed to create gpgme context. %s %s", gpgme_strsource(error), gpgme_strerror(error));
-        *err_str = strdup(gpgme_strerror(error));
+        if (err_str) {
+            *err_str = strdup(gpgme_strerror(error));
+        }
         return FALSE;
     }
 
@@ -432,13 +423,9 @@ p_gpg_valid_key(const char* const keyid, char** err_str)
 
     if (error || key == NULL) {
         log_error("GPG: Failed to get key. %s %s", gpgme_strsource(error), gpgme_strerror(error));
-        *err_str = strdup(gpgme_strerror(error));
-        gpgme_release(ctx);
-        return FALSE;
-    }
-
-    if (key == NULL) {
-        *err_str = strdup("Unknown error");
+        if (err_str) {
+            *err_str = strdup(error ? gpgme_strerror(error) : "gpgme didn't return any error, but it didn't return a key");
+        }
         gpgme_release(ctx);
         return FALSE;
     }
@@ -470,10 +457,9 @@ p_gpg_verify(const char* const barejid, const char* const sign)
         return;
     }
 
-    char* sign_with_header_footer = _add_header_footer(sign, PGP_SIGNATURE_HEADER, PGP_SIGNATURE_FOOTER);
+    auto_char char* sign_with_header_footer = _add_header_footer(sign, PGP_SIGNATURE_HEADER, PGP_SIGNATURE_FOOTER);
     gpgme_data_t sign_data;
     gpgme_data_new_from_mem(&sign_data, sign_with_header_footer, strlen(sign_with_header_footer), 1);
-    free(sign_with_header_footer);
 
     gpgme_data_t plain_data;
     gpgme_data_new(&plain_data);
@@ -541,7 +527,7 @@ p_gpg_sign(const char* const str, const char* const fp)
         return NULL;
     }
 
-    char* str_or_empty = NULL;
+    auto_char char* str_or_empty = NULL;
     if (str) {
         str_or_empty = strdup(str);
     } else {
@@ -549,7 +535,6 @@ p_gpg_sign(const char* const str, const char* const fp)
     }
     gpgme_data_t str_data;
     gpgme_data_new_from_mem(&str_data, str_or_empty, strlen(str_or_empty), 1);
-    free(str_or_empty);
 
     gpgme_data_t signed_data;
     gpgme_data_new(&signed_data);
@@ -672,10 +657,9 @@ p_gpg_decrypt(const char* const cipher)
 
     gpgme_set_passphrase_cb(ctx, (gpgme_passphrase_cb_t)_p_gpg_passphrase_cb, NULL);
 
-    char* cipher_with_headers = _add_header_footer(cipher, PGP_MESSAGE_HEADER, PGP_MESSAGE_FOOTER);
+    auto_char char* cipher_with_headers = _add_header_footer(cipher, PGP_MESSAGE_HEADER, PGP_MESSAGE_FOOTER);
     gpgme_data_t cipher_data;
     gpgme_data_new_from_mem(&cipher_data, cipher_with_headers, strlen(cipher_with_headers), 1);
-    free(cipher_with_headers);
 
     gpgme_data_t plain_data;
     gpgme_data_new(&plain_data);
@@ -718,20 +702,11 @@ p_gpg_decrypt(const char* const cipher)
     }
     gpgme_release(ctx);
 
-    size_t len = 0;
-    char* plain_str = gpgme_data_release_and_get_mem(plain_data, &len);
-    char* result = NULL;
-    if (plain_str) {
-        plain_str[len] = 0;
-        result = g_strdup(plain_str);
-    }
-    gpgme_free(plain_str);
-
     if (passphrase_attempt) {
         passphrase = strdup(passphrase_attempt);
     }
 
-    return result;
+    return _gpgme_data_to_char(plain_data);
 }
 
 void
@@ -768,12 +743,203 @@ p_gpg_format_fp_str(char* fp)
         }
     }
 
-    char* result = format->str;
-    g_string_free(format, FALSE);
+    return g_string_free(format, FALSE);
+}
 
+/**
+ * Returns the public key data for the given key ID.
+ *
+ * @param keyid The key ID for which to retrieve the public key data.
+ *              If the key ID is empty or NULL, returns NULL.
+ * @return The public key data as a null-terminated char* string allocated using malloc.
+ *         The returned string should be freed by the caller.
+ *         Returns NULL on error, and errors are written to the error log.
+ */
+char*
+p_gpg_get_pubkey(const char* keyid)
+{
+    if (!keyid || *keyid == '\0') {
+        return NULL;
+    }
+
+    gpgme_ctx_t context = NULL;
+    gpgme_error_t error = GPG_ERR_NO_ERROR;
+    gpgme_data_t data = NULL;
+
+    error = gpgme_new(&context);
+    if (error != GPG_ERR_NO_ERROR) {
+        log_error("GPG: Failed to create gpgme context. %s", gpgme_strerror(error));
+        goto cleanup;
+    }
+
+    error = gpgme_data_new(&data);
+    if (error != GPG_ERR_NO_ERROR || data == NULL) {
+        log_error("GPG: Failed to create new gpgme data. %s", gpgme_strerror(error));
+        goto cleanup;
+    }
+
+    gpgme_set_armor(context, 1);
+
+    error = gpgme_op_export(context, keyid, GPGME_EXPORT_MODE_MINIMAL, data);
+    if (error != GPG_ERR_NO_ERROR) {
+        log_error("GPG: Failed to export public key. %s", gpgme_strerror(error));
+        goto cleanup;
+    }
+
+cleanup:
+    gpgme_release(context);
+    return _gpgme_data_to_char(data);
+}
+
+/**
+ * Validate that the provided buffer has the format of an armored public key.
+ *
+ * This function only briefly checks for presence of armored header and footer.
+ *
+ * @param buffer The buffer containing the key data.
+ * @return TRUE if the buffer has the expected header and footer of an armored public key, FALSE otherwise.
+ */
+gboolean
+p_gpg_is_public_key_format(const char* buffer)
+{
+    if (buffer == NULL || buffer[0] == '\0') {
+        return false;
+    }
+
+    const char* headerPos = strstr(buffer, PGP_PUBLIC_KEY_HEADER);
+    if (headerPos == NULL) {
+        return false;
+    }
+
+    const char* footerPos = strstr(buffer, PGP_PUBLIC_KEY_FOOTER);
+
+    return (footerPos != NULL && footerPos > headerPos);
+}
+
+/**
+ * Imports a PGP public key(s) from a buffer.
+ *
+ * @param buffer The buffer containing the PGP key data.
+ * @return A pointer to the first imported ProfPGPKey structure, or NULL if an error occurs.
+ *
+ * @note The caller is responsible for freeing the memory of the returned ProfPGPKey structure
+ *       by calling p_gpg_free_key() to avoid resource leaks.
+ */
+ProfPGPKey*
+p_gpg_import_pubkey(const char* buffer)
+{
+    gpgme_ctx_t ctx;
+    ProfPGPKey* result = NULL;
+    gpgme_error_t error = gpgme_new(&ctx);
+    if (error != GPG_ERR_NO_ERROR) {
+        log_error("GPG: Error creating GPGME context");
+        goto out;
+    }
+
+    gpgme_data_t key_data;
+    error = gpgme_data_new_from_mem(&key_data, buffer, strlen(buffer), 1);
+    if (error != GPG_ERR_NO_ERROR) {
+        log_error("GPG: Error creating GPGME data from buffer");
+        goto out;
+    }
+
+    error = gpgme_op_import(ctx, key_data);
+
+    gpgme_data_release(key_data);
+
+    if (error != GPG_ERR_NO_ERROR) {
+        log_error("GPG: Error importing key data");
+        goto out;
+    }
+
+    gpgme_import_result_t import_result = gpgme_op_import_result(ctx);
+    gpgme_import_status_t status = import_result->imports;
+    gboolean is_valid = (status && status->result == GPG_ERR_NO_ERROR);
+
+    if (!is_valid) {
+        log_error("GPG: Error importing PGP key (%s).", status ? gpgme_strerror(status->result) : "Invalid import status.");
+        goto out;
+    }
+
+    gpgme_key_t key = NULL;
+    error = gpgme_get_key(ctx, status->fpr, &key, 0);
+    if (error != GPG_ERR_NO_ERROR) {
+        log_error("GPG: Unable to find imported PGP key (%s).", gpgme_strerror(error));
+        goto out;
+    }
+
+    result = _gpgme_key_to_ProfPGPKey(key);
+    gpgme_key_release(key);
+
+out:
+    gpgme_release(ctx);
     return result;
 }
 
+/**
+ * Converts a GPGME key struct to a ProfPGPKey struct.
+ *
+ * @param key The GPGME key struct to convert.
+ * @return A newly allocated ProfPGPKey struct populated with the converted data,
+ *         or NULL if an error occurs.
+ *
+ * @note The caller is responsible for freeing the memory of the returned ProfPGPKey structure
+ *       by calling p_gpg_free_key() to avoid resource leaks.
+ */
+static ProfPGPKey*
+_gpgme_key_to_ProfPGPKey(gpgme_key_t key)
+{
+    if (key == NULL) {
+        return NULL;
+    }
+
+    ProfPGPKey* p_pgpkey = p_gpg_key_new();
+    gpgme_subkey_t sub = key->subkeys;
+    p_pgpkey->id = strdup(sub->keyid);
+    p_pgpkey->name = strdup(key->uids->uid);
+    p_pgpkey->fp = strdup(sub->fpr);
+
+    while (sub) {
+        if (sub->can_encrypt)
+            p_pgpkey->encrypt = TRUE;
+        if (sub->can_authenticate)
+            p_pgpkey->authenticate = TRUE;
+        if (sub->can_certify)
+            p_pgpkey->certify = TRUE;
+        if (sub->can_sign)
+            p_pgpkey->sign = TRUE;
+
+        sub = sub->next;
+    }
+    return p_pgpkey;
+}
+
+/**
+ * Convert a gpgme_data_t object to a null-terminated char* string.
+ *
+ * @param data The gpgme_data_t object to convert.
+ * @return The converted string allocated using malloc, which should be freed by the caller.
+ *         If an error occurs or the data is empty, NULL is returned and errors are written to the error log.
+ */
+static char*
+_gpgme_data_to_char(gpgme_data_t data)
+{
+    size_t buffer_size = 0;
+    char* gpgme_buffer = gpgme_data_release_and_get_mem(data, &buffer_size);
+
+    if (!gpgme_buffer) {
+        log_error("GPG: Unable to extract gpgmedata.");
+        return NULL;
+    }
+
+    char* buffer = malloc(buffer_size + 1);
+    memcpy(buffer, gpgme_buffer, buffer_size);
+    buffer[buffer_size] = '\0';
+    gpgme_free(gpgme_buffer);
+
+    return buffer;
+}
+
 static char*
 _remove_header_footer(char* str, const char* const footer)
 {
@@ -801,26 +967,14 @@ _remove_header_footer(char* str, const char* const footer)
 static char*
 _add_header_footer(const char* const str, const char* const header, const char* const footer)
 {
-    GString* result_str = g_string_new("");
-
-    g_string_append(result_str, header);
-    g_string_append(result_str, "\n\n");
-    g_string_append(result_str, str);
-    g_string_append(result_str, "\n");
-    g_string_append(result_str, footer);
-
-    char* result = result_str->str;
-    g_string_free(result_str, FALSE);
-
-    return result;
+    return g_strdup_printf("%s\n\n%s\n%s", header, str, footer);
 }
 
 static void
 _save_pubkeys(void)
 {
     gsize g_data_size;
-    gchar* g_pubkeys_data = g_key_file_to_data(pubkeyfile, &g_data_size, NULL);
+    auto_gchar gchar* g_pubkeys_data = g_key_file_to_data(pubkeyfile, &g_data_size, NULL);
     g_file_set_contents(pubsloc, g_pubkeys_data, g_data_size, NULL);
     g_chmod(pubsloc, S_IRUSR | S_IWUSR);
-    g_free(g_pubkeys_data);
 }
