@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2020 Stefan Kropp <stefan@debxwoody.de>
- * Copyright (C) 2020 - 2022 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2020 - 2023 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -419,10 +419,12 @@ p_ox_gpg_decrypt(char* base64)
 
     size_t len;
     char* plain_str = gpgme_data_release_and_get_mem(plain, &len);
-    char* result = malloc(len + 1);
-    memcpy(result, plain_str, len);
-    result[len] = '\0';
-    gpgme_free(plain_str);
+    char* result = NULL;
+    if (plain_str) {
+        result = strndup(plain_str, len);
+        gpgme_free(plain_str);
+    }
+
     return result;
 }
 
