@@ -158,7 +158,7 @@ python_api_register_command(PyObject* self, PyObject* args)
         c_synopsis[len] = NULL;
 
         Py_ssize_t args_len = PyList_Size(arguments);
-        char* c_arguments[args_len == 0 ? 0 : args_len + 1][2];
+        char* c_arguments[args_len + 1][2];
         for (i = 0; i < args_len; i++) {
             PyObject* item = PyList_GetItem(arguments, i);
             Py_ssize_t len2 = PyList_Size(item);
@@ -196,7 +196,7 @@ python_api_register_command(PyObject* self, PyObject* args)
             free(c_synopsis[i++]);
         }
         i = 0;
-        while (c_arguments[i] != NULL && c_arguments[i][0] != NULL) {
+        while (c_arguments[i][0] != NULL) {
             free(c_arguments[i][0]);
             free(c_arguments[i][1]);
             i++;
@@ -1575,7 +1575,7 @@ static PyMethodDef apiMethods[] = {
     { NULL, NULL, 0, NULL }
 };
 
-#if PY_MAJOR_VERSION >= 3
+#ifdef PY_IS_PYTHON3
 static struct PyModuleDef profModule = {
     PyModuleDef_HEAD_INIT,
     "prof",
@@ -1588,7 +1588,7 @@ static struct PyModuleDef profModule = {
 PyMODINIT_FUNC
 python_api_init(void)
 {
-#if PY_MAJOR_VERSION >= 3
+#ifdef PY_IS_PYTHON3
     PyObject* result = PyModule_Create(&profModule);
     if (!result) {
         log_debug("Failed to initialise prof module");
@@ -1604,7 +1604,7 @@ python_api_init(void)
 void
 python_init_prof(void)
 {
-#if PY_MAJOR_VERSION >= 3
+#ifdef PY_IS_PYTHON3
     PyImport_AppendInittab("prof", python_api_init);
     Py_Initialize();
     PyEval_InitThreads();
@@ -1619,8 +1619,16 @@ static char*
 _python_plugin_name(void)
 {
     PyThreadState* ts = PyThreadState_Get();
+#if PY_VERSION_HEX >= 0x030B0000
+    PyFrameObject* frame = PyThreadState_GetFrame(ts);
+    PyCodeObject* code = PyFrame_GetCode(frame);
+    char* filename = python_str_or_unicode_to_string(code->co_filename);
+    Py_DECREF(code);
+    Py_DECREF(frame);
+#else
     PyFrameObject* frame = ts->frame;
     char* filename = python_str_or_unicode_to_string(frame->f_code->co_filename);
+#endif
     gchar** split = g_strsplit(filename, "/", 0);
     free(filename);
     char* plugin_name = strdup(split[g_strv_length(split) - 1]);
@@ -1640,7 +1648,7 @@ python_str_or_unicode_to_string(void* obj)
         return NULL;
     }
 
-#if PY_MAJOR_VERSION >= 3
+#ifdef PY_IS_PYTHON3
     if (PyUnicode_Check(pyobj)) {
         PyObject* utf8_str = PyUnicode_AsUTF8String(pyobj);
         char* result = strdup(PyBytes_AS_STRING(utf8_str));
