@@ -4,6 +4,7 @@
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
  * Copyright (C) 2020 William Wennerström <william@wstrm.dev>
+ * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -57,6 +58,7 @@
 #include "common.h"
 
 GSList* download_processes = NULL;
+gboolean silent = FALSE;
 
 static int
 _xferinfo(void* userdata, curl_off_t dltotal, curl_off_t dlnow, curl_off_t ultotal, curl_off_t ulnow)
@@ -82,8 +84,9 @@ _xferinfo(void* userdata, curl_off_t dltotal, curl_off_t dlnow, curl_off_t ultot
         dlperc = (100 * dlnow) / dltotal;
     }
 
-    http_print_transfer_update(download->window, download->url,
-                               "Downloading '%s': %d%%", download->url, dlperc);
+    if (!silent)
+        http_print_transfer_update(download->window, download->url,
+                                   "Downloading '%s': %d%%", download->url, dlperc);
 
     pthread_mutex_unlock(&lock);
 
@@ -107,17 +110,20 @@ http_file_get(void* userdata)
 
     CURL* curl;
     CURLcode res;
+    silent = download->silent;
 
     download->cancel = 0;
     download->bytes_received = 0;
 
     pthread_mutex_lock(&lock);
-    http_print_transfer(download->window, download->url,
-                        "Downloading '%s': 0%%", download->url);
+    if (!silent) {
+        http_print_transfer(download->window, download->id,
+                            "Downloading '%s': 0%%", download->url);
+    }
 
     FILE* outfh = fopen(download->filename, "wb");
     if (outfh == NULL) {
-        http_print_transfer_update(download->window, download->url,
+        http_print_transfer_update(download->window, download->id,
                                    "Downloading '%s' failed: Unable to open "
                                    "output file at '%s' for writing (%s).",
                                    download->url, download->filename,
@@ -125,7 +131,7 @@ http_file_get(void* userdata)
         goto out;
     }
 
-    char* cert_path = prefs_get_string(PREF_TLS_CERTPATH);
+    gchar* cert_path = prefs_get_string(PREF_TLS_CERTPATH);
     gchar* cafile = cafile_get_name();
     ProfAccount* account = accounts_get_account(session_get_account_name());
     gboolean insecure = account->tls_policy && strcmp(account->tls_policy, "trust") == 0;
@@ -177,22 +183,22 @@ http_file_get(void* userdata)
     g_free(cert_path);
     if (err) {
         if (download->cancel) {
-            http_print_transfer_update(download->window, download->url,
+            http_print_transfer_update(download->window, download->id,
                                        "Downloading '%s' failed: "
                                        "Download was canceled",
                                        download->url);
         } else {
-            http_print_transfer_update(download->window, download->url,
+            http_print_transfer_update(download->window, download->id,
                                        "Downloading '%s' failed: %s",
                                        download->url, err);
         }
         free(err);
     } else {
-        if (!download->cancel) {
-            http_print_transfer_update(download->window, download->url,
+        if (!download->cancel && !silent) {
+            http_print_transfer_update(download->window, download->id,
                                        "Downloading '%s': done\nSaved to '%s'",
                                        download->url, download->filename);
-            win_mark_received(download->window, download->url);
+            win_mark_received(download->window, download->id);
         }
     }
 
@@ -203,7 +209,7 @@ http_file_get(void* userdata)
 
         // TODO: Log the error.
         if (!call_external(argv)) {
-            http_print_transfer_update(download->window, download->url,
+            http_print_transfer_update(download->window, download->id,
                                        "Downloading '%s' failed: Unable to call "
                                        "command '%s' with file at '%s' (%s).",
                                        download->url,
@@ -221,6 +227,7 @@ out:
     download_processes = g_slist_remove(download_processes, download);
     pthread_mutex_unlock(&lock);
 
+    free(download->id);
     free(download->url);
     free(download->filename);
     free(download);
