@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2022 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -53,6 +53,9 @@
 #ifdef HAVE_LIBOTR
 #include "otr/otr.h"
 #endif
+#ifdef HAVE_LIBGPGME
+#include "pgp/gpg.h"
+#endif
 #ifdef HAVE_OMEMO
 #include "omemo/omemo.h"
 #endif
@@ -96,7 +99,7 @@ chatwin_new(const char* const barejid)
     ProfWin* window = wins_new_chat(barejid);
     ProfChatWin* chatwin = (ProfChatWin*)window;
 
-    if (!prefs_get_boolean(PREF_MAM) && prefs_get_boolean(PREF_CHLOG) && prefs_get_boolean(PREF_HISTORY)) {
+    if (!prefs_get_boolean(PREF_MAM) && (prefs_get_boolean(PREF_CHLOG) && prefs_get_boolean(PREF_HISTORY))) {
         _chatwin_history(chatwin, barejid);
     }
 
@@ -146,7 +149,8 @@ chatwin_new(const char* const barejid)
     }
 
     if (prefs_get_boolean(PREF_MAM)) {
-        iq_mam_request(chatwin);
+        iq_mam_request(chatwin, NULL);
+        win_print_loading_history(window);
     }
 
     return chatwin;
@@ -245,10 +249,10 @@ chatwin_otr_smp_event(ProfChatWin* chatwin, prof_otr_smp_event_t event, void* da
         win_println((ProfWin*)chatwin, THEME_DEFAULT, "!", "%s failed to authenticate you.", chatwin->barejid);
         break;
     case PROF_OTR_SMP_AUTH:
-        win_println((ProfWin*)chatwin, THEME_DEFAULT, "!", "Authenticating %s...", chatwin->barejid);
+        win_println((ProfWin*)chatwin, THEME_DEFAULT, "!", "Authenticating %s…", chatwin->barejid);
         break;
     case PROF_OTR_SMP_AUTH_WAIT:
-        win_println((ProfWin*)chatwin, THEME_DEFAULT, "!", "Awaiting authentication from %s...", chatwin->barejid);
+        win_println((ProfWin*)chatwin, THEME_DEFAULT, "!", "Awaiting authentication from %s…", chatwin->barejid);
         break;
     default:
         break;
@@ -319,39 +323,59 @@ chatwin_incoming_msg(ProfChatWin* chatwin, ProfMessage* message, gboolean win_cr
     char* old_plain = message->plain;
 
     message->plain = plugins_pre_chat_message_display(message->from_jid->barejid, message->from_jid->resourcepart, message->plain);
+    gboolean show_message = true;
 
     ProfWin* window = (ProfWin*)chatwin;
     int num = wins_get_num(window);
 
-    char* display_name;
-    char* mybarejid = connection_get_barejid();
+    auto_gchar gchar* display_name;
+    auto_char char* mybarejid = connection_get_barejid();
     if (g_strcmp0(mybarejid, message->from_jid->barejid) == 0) {
         display_name = strdup("me");
     } else {
         display_name = roster_get_msg_display_name(message->from_jid->barejid, message->from_jid->resourcepart);
     }
-    free(mybarejid);
+
+#ifdef HAVE_LIBGPGME
+    if (prefs_get_boolean(PREF_PGP_PUBKEY_AUTOIMPORT)) {
+        if (p_gpg_is_public_key_format(message->plain)) {
+            ProfPGPKey* key = p_gpg_import_pubkey(message->plain);
+            if (key != NULL) {
+                show_message = false;
+                win_println(window, THEME_DEFAULT, "-", "Received and imported PGP key %s: \"%s\". To assign it to the correspondent using /pgp setkey %s %s", key->fp, key->name, display_name, key->id);
+                p_gpg_free_key(key);
+            } else {
+                win_println(window, THEME_DEFAULT, "-", "Received PGP key, but couldn't import PGP key above.");
+            }
+        }
+    }
+#endif
 
     gboolean is_current = wins_is_current(window);
-    gboolean notify = prefs_do_chat_notify(is_current);
+    gboolean notify = prefs_do_chat_notify(is_current) && !message->is_mam;
 
     // currently viewing chat window with sender
     if (wins_is_current(window)) {
-        win_print_incoming(window, display_name, message);
+        if (show_message) {
+            win_print_incoming(window, display_name, message);
+        }
         title_bar_set_typing(FALSE);
         status_bar_active(num, WIN_CHAT, chatwin->barejid);
 
         // not currently viewing chat window with sender
     } else {
         status_bar_new(num, WIN_CHAT, chatwin->barejid);
-        cons_show_incoming_message(display_name, num, chatwin->unread, window);
 
-        if (prefs_get_boolean(PREF_FLASH)) {
-            flash();
+        if (!message->is_mam) {
+            cons_show_incoming_message(display_name, num, chatwin->unread, window);
+
+            if (prefs_get_boolean(PREF_FLASH)) {
+                flash();
+            }
+
+            chatwin->unread++;
         }
 
-        chatwin->unread++;
-
         // TODO: so far we don't ask for MAM when incoming message occurs.
         // Need to figure out:
         // 1) only send IQ once
@@ -373,22 +397,24 @@ chatwin_incoming_msg(ProfChatWin* chatwin, ProfMessage* message, gboolean win_cr
         }
 
         win_insert_last_read_position_marker((ProfWin*)chatwin, chatwin->barejid);
-        win_print_incoming(window, display_name, message);
+        if (show_message) {
+            win_print_incoming(window, display_name, message);
+        }
     }
 
-    wins_add_urls_ac(window, message);
-    wins_add_quotes_ac(window, message->plain);
+    if (!message->is_mam) {
+        wins_add_urls_ac(window, message, FALSE);
+        wins_add_quotes_ac(window, message->plain, FALSE);
 
-    if (prefs_get_boolean(PREF_BEEP)) {
-        beep();
+        if (prefs_get_boolean(PREF_BEEP)) {
+            beep();
+        }
     }
 
     if (notify) {
         notify_message(display_name, num, message->plain);
     }
 
-    free(display_name);
-
     plugins_post_chat_message_display(message->from_jid->barejid, message->from_jid->resourcepart, message->plain);
 
     free(message->plain);
@@ -402,9 +428,9 @@ chatwin_outgoing_msg(ProfChatWin* chatwin, const char* const message, char* id,
     assert(chatwin != NULL);
 
     ProfWin* window = (ProfWin*)chatwin;
-    wins_add_quotes_ac(window, message);
+    wins_add_quotes_ac(window, message, FALSE);
 
-    char* enc_char;
+    auto_char char* enc_char;
     if (chatwin->outgoing_char) {
         enc_char = chatwin->outgoing_char;
     } else if (enc_mode == PROF_MSG_ENC_OTR) {
@@ -425,8 +451,6 @@ chatwin_outgoing_msg(ProfChatWin* chatwin, const char* const message, char* id,
         win_print_outgoing((ProfWin*)chatwin, enc_char, id, replace_id, message);
     }
 
-    free(enc_char);
-
     // save last id and message for LMC
     if (id) {
         _chatwin_set_last_message(chatwin, id, message);
@@ -438,7 +462,7 @@ chatwin_outgoing_carbon(ProfChatWin* chatwin, ProfMessage* message)
 {
     assert(chatwin != NULL);
 
-    char* enc_char;
+    auto_char char* enc_char;
     if (message->enc == PROF_MSG_ENC_PGP) {
         enc_char = prefs_get_pgp_char();
     } else if (message->enc == PROF_MSG_ENC_OMEMO) {
@@ -452,8 +476,6 @@ chatwin_outgoing_carbon(ProfChatWin* chatwin, ProfMessage* message)
     win_print_outgoing(window, enc_char, message->id, message->replace_id, message->plain);
     int num = wins_get_num(window);
     status_bar_active(num, WIN_CHAT, chatwin->barejid);
-
-    free(enc_char);
 }
 
 void
@@ -463,11 +485,9 @@ chatwin_contact_online(ProfChatWin* chatwin, Resource* resource, GDateTime* last
 
     const char* show = string_from_resource_presence(resource->presence);
     PContact contact = roster_get_contact(chatwin->barejid);
-    char* display_str = p_contact_create_display_string(contact, resource->name);
+    auto_char char* display_str = p_contact_create_display_string(contact, resource->name);
 
     win_show_status_string((ProfWin*)chatwin, display_str, show, resource->status, last_activity, "++", "online");
-
-    free(display_str);
 }
 
 void
@@ -476,11 +496,9 @@ chatwin_contact_offline(ProfChatWin* chatwin, char* resource, char* status)
     assert(chatwin != NULL);
 
     PContact contact = roster_get_contact(chatwin->barejid);
-    char* display_str = p_contact_create_display_string(contact, resource);
+    auto_char char* display_str = p_contact_create_display_string(contact, resource);
 
     win_show_status_string((ProfWin*)chatwin, display_str, "offline", status, NULL, "--", "offline");
-
-    free(display_str);
 }
 
 char*
@@ -508,10 +526,7 @@ chatwin_get_string(ProfChatWin* chatwin)
         g_string_append_printf(res, ", %d unread", chatwin->unread);
     }
 
-    char* resstr = res->str;
-    g_string_free(res, FALSE);
-
-    return resstr;
+    return g_string_free(res, FALSE);
 }
 
 void
@@ -572,7 +587,7 @@ static void
 _chatwin_history(ProfChatWin* chatwin, const char* const contact_barejid)
 {
     if (!chatwin->history_shown) {
-        GSList* history = log_database_get_previous_chat(contact_barejid);
+        GSList* history = log_database_get_previous_chat(contact_barejid, NULL, NULL, FALSE, FALSE);
         GSList* curr = history;
 
         while (curr) {
@@ -591,6 +606,41 @@ _chatwin_history(ProfChatWin* chatwin, const char* const contact_barejid)
     }
 }
 
+// Print history starting from start_time to end_time if end_time is null the
+// first entry's timestamp in the buffer is used. Flip true to prepend to buffer.
+// Timestamps should be in iso8601
+gboolean
+chatwin_db_history(ProfChatWin* chatwin, const char* start_time, char* end_time, gboolean flip)
+{
+    if (!end_time) {
+        end_time = buffer_size(((ProfWin*)chatwin)->layout->buffer) == 0 ? NULL : g_date_time_format_iso8601(buffer_get_entry(((ProfWin*)chatwin)->layout->buffer, 0)->time);
+    }
+
+    GSList* history = log_database_get_previous_chat(chatwin->barejid, start_time, end_time, !flip, flip);
+    gboolean has_items = g_slist_length(history) != 0;
+    GSList* curr = history;
+
+    while (curr) {
+        ProfMessage* msg = curr->data;
+        char* msg_plain = msg->plain;
+        msg->plain = plugins_pre_chat_message_display(msg->from_jid->barejid, msg->from_jid->resourcepart, msg->plain);
+        // This is dirty workaround for memory leak. We reassign msg->plain above so have to free previous object
+        // TODO: Make a better solution, for example, pass msg object to the function and it will replace msg->plain properly if needed.
+        free(msg_plain);
+        if (flip) {
+            win_print_old_history((ProfWin*)chatwin, msg);
+        } else {
+            win_print_history((ProfWin*)chatwin, msg);
+        }
+        curr = g_slist_next(curr);
+    }
+
+    g_slist_free_full(history, (GDestroyNotify)message_free);
+    win_redraw((ProfWin*)chatwin);
+
+    return has_items;
+}
+
 static void
 _chatwin_set_last_message(ProfChatWin* chatwin, const char* const id, const char* const message)
 {
