@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2022 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -66,10 +66,6 @@
 #include "xmpp/muc.h"
 #include "xmpp/roster_list.h"
 
-#ifdef HAVE_GIT_VERSION
-#include "gitversion.h"
-#endif
-
 static void _cons_splash_logo(void);
 static void _show_roster_contacts(GSList* list, gboolean show_groups);
 static GList* alert_list;
@@ -293,7 +289,7 @@ cons_show_typing(const char* const barejid)
         display_usr = barejid;
     }
 
-    win_println(console, THEME_TYPING, "-", "!! %s is typing a message...", display_usr);
+    win_println(console, THEME_TYPING, "-", "!! %s is typing a message…", display_usr);
     cons_alert(NULL);
 }
 
@@ -310,9 +306,7 @@ _room_triggers_to_string(GList* triggers)
         }
     }
 
-    char* result = triggers_str->str;
-    g_string_free(triggers_str, FALSE);
-    return result;
+    return g_string_free(triggers_str, FALSE);
 }
 
 void
@@ -326,7 +320,7 @@ cons_show_incoming_room_message(const char* const nick, const char* const room,
         ui_index = 0;
     }
 
-    char* muc_show = prefs_get_string(PREF_CONSOLE_MUC);
+    auto_gchar gchar* muc_show = prefs_get_string(PREF_CONSOLE_MUC);
 
     // 'mention'
     if (g_strcmp0(muc_show, "mention") == 0) {
@@ -339,9 +333,8 @@ cons_show_incoming_room_message(const char* const nick, const char* const room,
         if (mention) {
             win_println(console, THEME_MENTION, "-", "<< room mention: %s in %s (win %d)", nick, room, ui_index);
         } else if (triggers) {
-            char* triggers_str = _room_triggers_to_string(triggers);
+            auto_char char* triggers_str = _room_triggers_to_string(triggers);
             win_println(console, THEME_TRIGGER, "-", "<< room trigger %s: %s in %s (win %d)", triggers_str, nick, room, ui_index);
-            free(triggers_str);
         } else {
             // 'all' or 'first' if its the first message
             if ((g_strcmp0(muc_show, "all") == 0) || ((g_strcmp0(muc_show, "first") == 0) && (unread == 0))) {
@@ -350,8 +343,6 @@ cons_show_incoming_room_message(const char* const nick, const char* const room,
             cons_alert(window);
         }
     }
-
-    free(muc_show);
 }
 
 void
@@ -364,13 +355,11 @@ cons_show_incoming_message(const char* const short_from, const int win_index, in
         ui_index = 0;
     }
 
-    char* chat_show = prefs_get_string(PREF_CONSOLE_CHAT);
+    auto_gchar gchar* chat_show = prefs_get_string(PREF_CONSOLE_CHAT);
     if (g_strcmp0(chat_show, "all") == 0 || ((g_strcmp0(chat_show, "first") == 0) && unread == 0)) {
         win_println(console, THEME_INCOMING, "-", "<< chat message: %s (win %d)", short_from, ui_index);
         cons_alert(window);
     }
-
-    g_free(chat_show);
 }
 
 void
@@ -383,19 +372,17 @@ cons_show_incoming_private_message(const char* const nick, const char* const roo
         ui_index = 0;
     }
 
-    char* priv_show = prefs_get_string(PREF_CONSOLE_PRIVATE);
+    auto_gchar gchar* priv_show = prefs_get_string(PREF_CONSOLE_PRIVATE);
     if (g_strcmp0(priv_show, "all") == 0 || ((g_strcmp0(priv_show, "first") == 0) && (unread == 0))) {
         win_println(console, THEME_INCOMING, "-", "<< private message: %s in %s (win %d)", nick, room, ui_index);
         cons_alert(window);
     }
-
-    free(priv_show);
 }
 
 static void
 _cons_welcome_first_start(void)
 {
-    gchar* ident_loc = files_get_data_path(FILE_PROFANITY_IDENTIFIER);
+    auto_gchar gchar* ident_loc = files_get_data_path(FILE_PROFANITY_IDENTIFIER);
     if (!g_file_test(ident_loc, G_FILE_TEST_EXISTS)) {
         ProfWin* console = wins_get_console();
         win_println(console, THEME_DEFAULT, "-", "This seems to be your first time starting Profanity.");
@@ -410,7 +397,6 @@ _cons_welcome_first_start(void)
         win_println(console, THEME_DEFAULT, "-", "/register myjid myserver.org");
         win_println(console, THEME_DEFAULT, "-", "");
     }
-    g_free(ident_loc);
 }
 
 void
@@ -423,20 +409,12 @@ cons_about(void)
     if (prefs_get_boolean(PREF_SPLASH)) {
         _cons_splash_logo();
     } else {
-
-        if (strcmp(PACKAGE_STATUS, "development") == 0) {
-#ifdef HAVE_GIT_VERSION
-            win_println(console, THEME_DEFAULT, "-", "Welcome to Profanity, version %sdev.%s.%s", PACKAGE_VERSION, PROF_GIT_BRANCH, PROF_GIT_REVISION);
-#else
-            win_println(console, THEME_DEFAULT, "Welcome to Profanity, version %sdev", PACKAGE_VERSION);
-#endif
-        } else {
-            win_println(console, THEME_DEFAULT, "-", "Welcome to Profanity, version %s", PACKAGE_VERSION);
-        }
+        auto_gchar gchar* prof_version = prof_get_version();
+        win_println(console, THEME_DEFAULT, "-", "Welcome to Profanity, version %s", prof_version);
     }
 
     win_println(console, THEME_DEFAULT, "-", "Copyright (C) 2012 - 2019 James Booth <boothj5web@gmail.com>.");
-    win_println(console, THEME_DEFAULT, "-", "Copyright (C) 2019 - 2022 Michael Vetter <jubalh@iodoru.org>.");
+    win_println(console, THEME_DEFAULT, "-", "Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>.");
     win_println(console, THEME_DEFAULT, "-", "License GPLv3+: GNU GPL version 3 or later <https://www.gnu.org/licenses/gpl.html>");
     win_println(console, THEME_DEFAULT, "-", "");
     win_println(console, THEME_DEFAULT, "-", "This is free software; you are free to change and redistribute it.");
@@ -460,26 +438,27 @@ void
 cons_check_version(gboolean not_available_msg)
 {
     ProfWin* console = wins_get_console();
-    char* latest_release = release_get_latest();
+    auto_char char* latest_release = release_get_latest();
 
-    if (latest_release) {
-        gboolean relase_valid = g_regex_match_simple("^\\d+\\.\\d+\\.\\d+$", latest_release, 0, 0);
+    if (!latest_release) {
+        return;
+    }
 
-        if (relase_valid) {
-            if (release_is_new(latest_release)) {
-                win_println(console, THEME_DEFAULT, "-", "A new version of Profanity is available: %s", latest_release);
-                win_println(console, THEME_DEFAULT, "-", "Check <https://profanity-im.github.io> for details.");
+    gboolean relase_valid = g_regex_match_simple("^\\d+\\.\\d+\\.\\d+$", latest_release, 0, 0);
+
+    if (relase_valid) {
+        if (release_is_new(latest_release)) {
+            win_println(console, THEME_DEFAULT, "-", "A new version of Profanity is available: %s", latest_release);
+            win_println(console, THEME_DEFAULT, "-", "Check <https://profanity-im.github.io> for details.");
+            win_println(console, THEME_DEFAULT, "-", "");
+        } else {
+            if (not_available_msg) {
+                win_println(console, THEME_DEFAULT, "-", "No new version available.");
                 win_println(console, THEME_DEFAULT, "-", "");
-            } else {
-                if (not_available_msg) {
-                    win_println(console, THEME_DEFAULT, "-", "No new version available.");
-                    win_println(console, THEME_DEFAULT, "-", "");
-                }
             }
-
-            cons_alert(NULL);
         }
-        free(latest_release);
+
+        cons_alert(NULL);
     }
 }
 
@@ -812,7 +791,7 @@ cons_show_disco_info(const char* jid, GSList* identities, GSList* features)
             cons_show("  Identities");
         }
         while (identities) {
-            DiscoIdentity* identity = identities->data; // anme trpe, cat
+            DiscoIdentity* identity = identities->data; // name type, category
             GString* identity_str = g_string_new("    ");
             if (identity->name) {
                 identity_str = g_string_append(identity_str, identity->name);
@@ -902,12 +881,10 @@ cons_show_qrcode(const char* const text)
 
     ProfWin* console = wins_get_console();
 
-    char* buf = calloc((width * 4) + 1, 1);
-    char* pad = calloc((width * 4) + 5, 1);
+    auto_char char* buf = calloc((width * 4) + 1, 1);
+    auto_char char* pad = calloc((width * 4) + 5, 1);
 
     if (!buf || !pad) {
-        free(pad);
-        free(buf);
         return;
     }
 
@@ -915,10 +892,12 @@ cons_show_qrcode(const char* const text)
         strcat(pad, "\u2588\u2588");
     }
 
-    win_println(console, THEME_DEFAULT, "", pad);
+    win_println(console, THEME_DEFAULT, "", "");
+    win_println(console, THEME_DEFAULT, "", "");
+    win_println(console, THEME_DEFAULT, "", "%s", pad);
     for (size_t y = 0; y < width; y += ZOOM_SIZE) {
         for (size_t x = 0; x < width; x += ZOOM_SIZE) {
-            strcat(buf, !(*data & 1) ? "\u2588\u2588" : "\u2800\u2800");
+            strcat(buf, !(*data & 1) ? "\u2588\u2588" : "  ");
 
             data++;
         }
@@ -929,9 +908,8 @@ cons_show_qrcode(const char* const text)
         buf[0] = '\0';
     }
     win_println(console, THEME_DEFAULT, "", "%s", pad);
-
-    free(pad);
-    free(buf);
+    win_println(console, THEME_DEFAULT, "", "");
+    win_println(console, THEME_DEFAULT, "", "");
 
     QRcode_free(qrcode);
 #else
@@ -957,7 +935,7 @@ cons_show_status(const char* const barejid)
 void
 cons_show_room_invite(const char* const invitor, const char* const room, const char* const reason)
 {
-    char* display_from = NULL;
+    auto_char char* display_from = NULL;
     PContact contact = roster_get_contact(invitor);
     if (contact) {
         if (p_contact_name(contact)) {
@@ -985,8 +963,6 @@ cons_show_room_invite(const char* const invitor, const char* const room, const c
         notify_invite(display_from, room, reason);
     }
 
-    free(display_from);
-
     cons_alert(NULL);
 }
 
@@ -1062,6 +1038,12 @@ cons_show_account(ProfAccount* account)
     if (account->startscript) {
         cons_show("Start script      : %s", account->startscript);
     }
+    if (account->client) {
+        cons_show("Client name       : %s", account->client);
+    }
+    if (account->max_sessions > 0) {
+        cons_show("Max sessions alarm: %d", account->max_sessions);
+    }
     if (account->theme) {
         cons_show("Theme             : %s", account->theme);
     }
@@ -1118,11 +1100,12 @@ cons_show_account(ProfAccount* account)
 
     if ((connection_get_status() == JABBER_CONNECTED) && (g_strcmp0(session_get_account_name(), account->name) == 0)) {
         GList* resources = connection_get_available_resources();
+        int resources_count = connection_count_available_resources();
         GList* ordered_resources = NULL;
 
         GList* curr = resources;
         if (curr) {
-            win_println(console, THEME_DEFAULT, "-", "Resources:");
+            win_println(console, THEME_DEFAULT, "-", "Resources (%u):", resources_count);
 
             // sort in order of availability
             while (curr) {
@@ -1146,9 +1129,8 @@ cons_show_account(ProfAccount* account)
                 win_append(console, presence_colour, ", \"%s\"", resource->status);
             }
             win_appendln(console, THEME_DEFAULT, "");
-            Jid* jidp = jid_create_from_bare_and_resource(account->jid, resource->name);
+            auto_jid Jid* jidp = jid_create_from_bare_and_resource(account->jid, resource->name);
             EntityCapabilities* caps = caps_lookup(jidp->fulljid);
-            jid_destroy(jidp);
 
             if (caps) {
                 // show identity
@@ -1227,13 +1209,12 @@ cons_show_aliases(GList* aliases)
 void
 cons_theme_setting(void)
 {
-    char* theme = prefs_get_string(PREF_THEME);
+    auto_gchar gchar* theme = prefs_get_string(PREF_THEME);
     if (theme == NULL) {
         cons_show("Theme (/theme)                      : default");
     } else {
         cons_show("Theme (/theme)                      : %s", theme);
     }
-    g_free(theme);
 }
 
 void
@@ -1321,17 +1302,14 @@ cons_titlebar_setting(void)
 void
 cons_console_setting(void)
 {
-    char* chatsetting = prefs_get_string(PREF_CONSOLE_CHAT);
+    auto_gchar gchar* chatsetting = prefs_get_string(PREF_CONSOLE_CHAT);
     cons_show("Console chat messages (/console)    : %s", chatsetting);
-    g_free(chatsetting);
 
-    char* mucsetting = prefs_get_string(PREF_CONSOLE_MUC);
+    auto_gchar gchar* mucsetting = prefs_get_string(PREF_CONSOLE_MUC);
     cons_show("Console MUC messages (/console)     : %s", mucsetting);
-    g_free(mucsetting);
 
-    char* privsetting = prefs_get_string(PREF_CONSOLE_PRIVATE);
+    auto_gchar gchar* privsetting = prefs_get_string(PREF_CONSOLE_PRIVATE);
     cons_show("Console private messages (/console) : %s", privsetting);
-    g_free(privsetting);
 }
 
 void
@@ -1342,17 +1320,13 @@ cons_presence_setting(void)
     else
         cons_show("Titlebar presence (/presence)       : OFF");
 
-    char* console = prefs_get_string(PREF_STATUSES_CONSOLE);
-    char* chat = prefs_get_string(PREF_STATUSES_CHAT);
-    char* room = prefs_get_string(PREF_STATUSES_MUC);
+    auto_gchar gchar* console = prefs_get_string(PREF_STATUSES_CONSOLE);
+    auto_gchar gchar* chat = prefs_get_string(PREF_STATUSES_CHAT);
+    auto_gchar gchar* room = prefs_get_string(PREF_STATUSES_MUC);
 
     cons_show("Console presence (/presence)        : %s", console);
     cons_show("Chat presence (/presence)           : %s", chat);
     cons_show("Room presence (/presence)           : %s", room);
-
-    g_free(console);
-    g_free(chat);
-    g_free(room);
 }
 
 void
@@ -1417,10 +1391,9 @@ cons_occupants_setting(void)
     else
         cons_show("Occupants wrap (/occupants)         : OFF");
 
-    char* occupants_ch = prefs_get_occupants_char();
+    auto_char char* occupants_ch = prefs_get_occupants_char();
     if (occupants_ch) {
         cons_show("Occupants char (/occupants)         : %s", occupants_ch);
-        free(occupants_ch);
     } else {
         cons_show("Occupants char (/occupants)         : none");
     }
@@ -1431,10 +1404,9 @@ cons_occupants_setting(void)
     int size = prefs_get_occupants_size();
     cons_show("Occupants size (/occupants)         : %d", size);
 
-    char* header_ch = prefs_get_occupants_header_char();
+    auto_char char* header_ch = prefs_get_occupants_header_char();
     if (header_ch) {
         cons_show("Occupants header char (/occupants)  : %s", header_ch);
-        free(header_ch);
     } else {
         cons_show("Occupants header char (/occupants)  : none");
     }
@@ -1453,70 +1425,63 @@ cons_rooms_cache_setting(void)
 void
 cons_autoconnect_setting(void)
 {
-    char* pref_connect_account = prefs_get_string(PREF_CONNECT_ACCOUNT);
+    auto_gchar gchar* pref_connect_account = prefs_get_string(PREF_CONNECT_ACCOUNT);
     if (pref_connect_account)
         cons_show("Autoconnect (/autoconnect)      : %s", pref_connect_account);
     else
         cons_show("Autoconnect (/autoconnect)      : OFF");
-
-    g_free(pref_connect_account);
 }
 
 void
 cons_time_setting(void)
 {
-    char* pref_time_console = prefs_get_string(PREF_TIME_CONSOLE);
+    auto_gchar gchar* pref_time_console = prefs_get_string(PREF_TIME_CONSOLE);
     if (g_strcmp0(pref_time_console, "off") == 0)
         cons_show("Time console (/time)                : OFF");
     else
         cons_show("Time console (/time)                : %s", pref_time_console);
-    g_free(pref_time_console);
 
-    char* pref_time_chat = prefs_get_string(PREF_TIME_CHAT);
+    auto_gchar gchar* pref_time_chat = prefs_get_string(PREF_TIME_CHAT);
     if (g_strcmp0(pref_time_chat, "off") == 0)
         cons_show("Time chat (/time)                   : OFF");
     else
         cons_show("Time chat (/time)                   : %s", pref_time_chat);
-    g_free(pref_time_chat);
 
-    char* pref_time_muc = prefs_get_string(PREF_TIME_MUC);
+    auto_gchar gchar* pref_time_muc = prefs_get_string(PREF_TIME_MUC);
     if (g_strcmp0(pref_time_muc, "off") == 0)
         cons_show("Time MUC (/time)                    : OFF");
     else
         cons_show("Time MUC (/time)                    : %s", pref_time_muc);
-    g_free(pref_time_muc);
 
-    char* pref_time_conf = prefs_get_string(PREF_TIME_CONFIG);
+    auto_gchar gchar* pref_time_conf = prefs_get_string(PREF_TIME_CONFIG);
     if (g_strcmp0(pref_time_conf, "off") == 0)
         cons_show("Time config (/time)                 : OFF");
     else
         cons_show("Time config (/time)                 : %s", pref_time_conf);
-    g_free(pref_time_conf);
 
-    char* pref_time_private = prefs_get_string(PREF_TIME_PRIVATE);
+    auto_gchar gchar* pref_time_private = prefs_get_string(PREF_TIME_PRIVATE);
     if (g_strcmp0(pref_time_private, "off") == 0)
         cons_show("Time private (/time)                : OFF");
     else
         cons_show("Time private (/time)                : %s", pref_time_private);
-    g_free(pref_time_private);
 
-    char* pref_time_xml = prefs_get_string(PREF_TIME_XMLCONSOLE);
+    auto_gchar gchar* pref_time_xml = prefs_get_string(PREF_TIME_XMLCONSOLE);
     if (g_strcmp0(pref_time_xml, "off") == 0)
         cons_show("Time XML Console (/time)            : OFF");
     else
         cons_show("Time XML Console (/time)            : %s", pref_time_xml);
-    g_free(pref_time_xml);
 
-    char* pref_time_statusbar = prefs_get_string(PREF_TIME_STATUSBAR);
+    auto_gchar gchar* pref_time_statusbar = prefs_get_string(PREF_TIME_STATUSBAR);
     if (g_strcmp0(pref_time_statusbar, "off") == 0)
         cons_show("Time statusbar (/time)              : OFF");
     else
         cons_show("Time statusbar (/time)              : %s", pref_time_statusbar);
-    g_free(pref_time_statusbar);
 
-    char* pref_time_lastactivity = prefs_get_string(PREF_TIME_LASTACTIVITY);
+    auto_gchar gchar* pref_time_lastactivity = prefs_get_string(PREF_TIME_LASTACTIVITY);
     cons_show("Time last activity (/time)          : %s", pref_time_lastactivity);
-    g_free(pref_time_lastactivity);
+
+    auto_gchar gchar* pref_time_vcard = prefs_get_string(PREF_TIME_VCARD);
+    cons_show("Time vCard (/time)                  : %s", pref_time_vcard);
 }
 
 void
@@ -1556,50 +1521,44 @@ cons_roster_setting(void)
     else
         cons_show("Roster offline (/roster)            : hide");
 
-    char* header_ch = prefs_get_roster_header_char();
+    auto_char char* header_ch = prefs_get_roster_header_char();
     if (header_ch) {
         cons_show("Roster header char (/roster)        : %s", header_ch);
-        free(header_ch);
     } else {
         cons_show("Roster header char (/roster)        : none");
     }
 
-    char* contact_ch = prefs_get_roster_contact_char();
+    auto_char char* contact_ch = prefs_get_roster_contact_char();
     if (contact_ch) {
         cons_show("Roster contact char (/roster)       : %s", contact_ch);
-        free(contact_ch);
     } else {
         cons_show("Roster contact char (/roster)       : none");
     }
 
-    char* resource_ch = prefs_get_roster_resource_char();
+    auto_char char* resource_ch = prefs_get_roster_resource_char();
     if (resource_ch) {
         cons_show("Roster resource char (/roster)      : %s", resource_ch);
-        free(resource_ch);
     } else {
         cons_show("Roster resource char (/roster)      : none");
     }
 
-    char* room_ch = prefs_get_roster_room_char();
+    auto_char char* room_ch = prefs_get_roster_room_char();
     if (room_ch) {
         cons_show("Roster room char (/roster)          : %s", room_ch);
-        free(room_ch);
     } else {
         cons_show("Roster room char (/roster)          : none");
     }
 
-    char* room_priv_ch = prefs_get_roster_room_private_char();
+    auto_char char* room_priv_ch = prefs_get_roster_room_private_char();
     if (room_priv_ch) {
         cons_show("Roster room private char (/roster)  : %s", room_priv_ch);
-        free(room_priv_ch);
     } else {
         cons_show("Roster room private char (/roster)  : none");
     }
 
-    char* private_ch = prefs_get_roster_private_char();
+    auto_char char* private_ch = prefs_get_roster_private_char();
     if (private_ch) {
         cons_show("Roster private char (/roster)       : %s", private_ch);
-        free(private_ch);
     } else {
         cons_show("Roster private char (/roster)       : none");
     }
@@ -1653,28 +1612,25 @@ cons_roster_setting(void)
     else
         cons_show("Roster unsubscribed (/roster)       : hide");
 
-    char* count = prefs_get_string(PREF_ROSTER_COUNT);
+    auto_gchar gchar* count = prefs_get_string(PREF_ROSTER_COUNT);
     if (g_strcmp0(count, "off") == 0) {
         cons_show("Roster count (/roster)              : OFF");
     } else {
         cons_show("Roster count (/roster)              : %s", count);
     }
-    g_free(count);
 
     if (prefs_get_boolean(PREF_ROSTER_COUNT_ZERO))
         cons_show("Roster count zero (/roster)         : ON");
     else
         cons_show("Roster count zero (/roster)         : OFF");
 
-    char* by = prefs_get_string(PREF_ROSTER_BY);
+    auto_gchar gchar* by = prefs_get_string(PREF_ROSTER_BY);
     cons_show("Roster by (/roster)                 : %s", by);
-    g_free(by);
 
-    char* order = prefs_get_string(PREF_ROSTER_ORDER);
+    auto_gchar gchar* order = prefs_get_string(PREF_ROSTER_ORDER);
     cons_show("Roster order (/roster)              : %s", order);
-    g_free(order);
 
-    char* unread = prefs_get_string(PREF_ROSTER_UNREAD);
+    auto_gchar gchar* unread = prefs_get_string(PREF_ROSTER_UNREAD);
     if (g_strcmp0(unread, "before") == 0) {
         cons_show("Roster unread (/roster)             : before");
     } else if (g_strcmp0(unread, "after") == 0) {
@@ -1682,14 +1638,13 @@ cons_roster_setting(void)
     } else {
         cons_show("Roster unread (/roster)             : OFF");
     }
-    g_free(unread);
 
     if (prefs_get_boolean(PREF_ROSTER_ROOMS))
         cons_show("Roster rooms (/roster)              : show");
     else
         cons_show("Roster rooms (/roster)              : hide");
 
-    char* priv = prefs_get_string(PREF_ROSTER_PRIVATE);
+    auto_gchar gchar* priv = prefs_get_string(PREF_ROSTER_PRIVATE);
     if (g_strcmp0(priv, "room") == 0) {
         cons_show("Roster private (/roster)            : room");
     } else if (g_strcmp0(priv, "group") == 0) {
@@ -1697,25 +1652,20 @@ cons_roster_setting(void)
     } else {
         cons_show("Roster private (/roster)            : OFF");
     }
-    g_free(priv);
 
-    char* rooms_pos = prefs_get_string(PREF_ROSTER_ROOMS_POS);
+    auto_gchar gchar* rooms_pos = prefs_get_string(PREF_ROSTER_ROOMS_POS);
     cons_show("Roster rooms position (/roster)     : %s", rooms_pos);
-    g_free(rooms_pos);
 
-    char* rooms_by = prefs_get_string(PREF_ROSTER_ROOMS_BY);
+    auto_gchar gchar* rooms_by = prefs_get_string(PREF_ROSTER_ROOMS_BY);
     cons_show("Roster rooms by (/roster)           : %s", rooms_by);
-    g_free(rooms_by);
 
-    char* rooms_use = prefs_get_string(PREF_ROSTER_ROOMS_USE_AS_NAME);
+    auto_gchar gchar* rooms_use = prefs_get_string(PREF_ROSTER_ROOMS_USE_AS_NAME);
     cons_show("Roster rooms use (/roster)          : %s", rooms_use);
-    g_free(rooms_use);
 
-    char* rooms_order = prefs_get_string(PREF_ROSTER_ROOMS_ORDER);
+    auto_gchar gchar* rooms_order = prefs_get_string(PREF_ROSTER_ROOMS_ORDER);
     cons_show("Roster rooms order (/roster)        : %s", rooms_order);
-    g_free(rooms_order);
 
-    char* roomsunread = prefs_get_string(PREF_ROSTER_ROOMS_UNREAD);
+    auto_gchar gchar* roomsunread = prefs_get_string(PREF_ROSTER_ROOMS_UNREAD);
     if (g_strcmp0(roomsunread, "before") == 0) {
         cons_show("Roster rooms unread (/roster)       : before");
     } else if (g_strcmp0(roomsunread, "after") == 0) {
@@ -1723,7 +1673,6 @@ cons_roster_setting(void)
     } else {
         cons_show("Roster rooms unread (/roster)       : OFF");
     }
-    g_free(roomsunread);
 
     int size = prefs_get_roster_size();
     cons_show("Roster size (/roster)               : %d", size);
@@ -1999,21 +1948,21 @@ cons_statusbar_setting(void)
         cons_show("Max tab length (/statusbar)                 : %d", pref_len);
     }
 
-    char* pref_self = prefs_get_string(PREF_STATUSBAR_SELF);
+    auto_gchar gchar* pref_self = prefs_get_string(PREF_STATUSBAR_SELF);
     if (g_strcmp0(pref_self, "off") == 0) {
         cons_show("Self statusbar display (/statusbar)         : OFF");
     } else {
         cons_show("Self statusbar display (/statusbar)         : %s", pref_self);
     }
-    g_free(pref_self);
 
-    char* pref_chat = prefs_get_string(PREF_STATUSBAR_CHAT);
+    auto_gchar gchar* pref_chat = prefs_get_string(PREF_STATUSBAR_CHAT);
     cons_show("Chat tab display (/statusbar)               : %s", pref_chat);
-    g_free(pref_chat);
 
-    char* pref_room = prefs_get_string(PREF_STATUSBAR_ROOM);
+    auto_gchar gchar* pref_room = prefs_get_string(PREF_STATUSBAR_ROOM);
     cons_show("Room tab display (/statusbar)               : %s", pref_room);
-    g_free(pref_room);
+
+    auto_gchar gchar* pref_tabmode = prefs_get_string(PREF_STATUSBAR_TABMODE);
+    cons_show("Tab mode (/statusbar)                       : %s", pref_tabmode);
 }
 
 void
@@ -2076,13 +2025,12 @@ cons_show_log_prefs(void)
 void
 cons_autoaway_setting(void)
 {
-    char* pref_autoaway_mode = prefs_get_string(PREF_AUTOAWAY_MODE);
+    auto_gchar gchar* pref_autoaway_mode = prefs_get_string(PREF_AUTOAWAY_MODE);
     if (strcmp(pref_autoaway_mode, "off") == 0) {
         cons_show("Autoaway (/autoaway mode)                 : OFF");
     } else {
         cons_show("Autoaway (/autoaway mode)                 : %s", pref_autoaway_mode);
     }
-    g_free(pref_autoaway_mode);
 
     int away_time = prefs_get_autoaway_time();
     if (away_time == 1) {
@@ -2100,21 +2048,19 @@ cons_autoaway_setting(void)
         cons_show("Autoaway xa minutes (/autoaway time)      : %d minutes", xa_time);
     }
 
-    char* pref_autoaway_message = prefs_get_string(PREF_AUTOAWAY_MESSAGE);
+    auto_gchar gchar* pref_autoaway_message = prefs_get_string(PREF_AUTOAWAY_MESSAGE);
     if ((pref_autoaway_message == NULL) || (strcmp(pref_autoaway_message, "") == 0)) {
         cons_show("Autoaway away message (/autoaway message) : OFF");
     } else {
         cons_show("Autoaway away message (/autoaway message) : \"%s\"", pref_autoaway_message);
     }
-    g_free(pref_autoaway_message);
 
-    char* pref_autoxa_message = prefs_get_string(PREF_AUTOXA_MESSAGE);
+    auto_gchar gchar* pref_autoxa_message = prefs_get_string(PREF_AUTOXA_MESSAGE);
     if ((pref_autoxa_message == NULL) || (strcmp(pref_autoxa_message, "") == 0)) {
         cons_show("Autoaway xa message (/autoaway message)   : OFF");
     } else {
         cons_show("Autoaway xa message (/autoaway message)   : \"%s\"", pref_autoxa_message);
     }
-    g_free(pref_autoxa_message);
 
     if (prefs_get_boolean(PREF_AUTOAWAY_CHECK)) {
         cons_show("Autoaway check (/autoaway check)          : ON");
@@ -2177,7 +2123,7 @@ cons_autoping_setting(void)
 void
 cons_color_setting(void)
 {
-    char* color_pref = prefs_get_string(PREF_COLOR_NICK);
+    auto_gchar gchar* color_pref = prefs_get_string(PREF_COLOR_NICK);
 
     if (!color_pref) {
         cons_show("Consistent color generation for nicks (/color)                     : OFF");
@@ -2194,8 +2140,6 @@ cons_color_setting(void)
         cons_show("Consistent color generation for nicks (/color)                     : OFF");
     }
 
-    g_free(color_pref);
-
     if (prefs_get_boolean(PREF_COLOR_NICK_OWN)) {
         cons_show("Consistent color generation for own nick (/color own)              : ON");
     } else {
@@ -2215,16 +2159,6 @@ cons_color_setting(void)
     }
 }
 
-void
-cons_os_setting(void)
-{
-    if (prefs_get_boolean(PREF_REVEAL_OS)) {
-        cons_show("Reveal OS name when asked for software version (XEP-0092) (/os)    : ON");
-    } else {
-        cons_show("Reveal OS name when asked for software version (XEP-0092) (/os)    : OFF");
-    }
-}
-
 void
 cons_correction_setting(void)
 {
@@ -2234,34 +2168,32 @@ cons_correction_setting(void)
         cons_show("Last Message Correction (XEP-0308) (/correction)                   : OFF");
     }
 
-    char* cc = prefs_get_correction_char();
+    auto_char char* cc = prefs_get_correction_char();
     cons_show("LMC indication char (/correction char)                             : %s", cc);
-    free(cc);
 }
 
 void
 cons_executable_setting(void)
 {
-    gchar* avatar = prefs_get_string(PREF_AVATAR_CMD);
+    auto_gchar gchar* avatar = prefs_get_string(PREF_AVATAR_CMD);
     cons_show("Default '/avatar open' command (/executable avatar)                      : %s", avatar);
-    g_free(avatar);
 
     // TODO: there needs to be a way to get all the "locales"/schemes so we can
     // display the default openers for all filetypes
-    gchar* urlopen = prefs_get_string(PREF_URL_OPEN_CMD);
+    auto_gchar gchar* urlopen = prefs_get_string(PREF_URL_OPEN_CMD);
     cons_show("Default '/url open' command (/executable urlopen)                        : %s", urlopen);
-    g_free(urlopen);
 
-    gchar* urlsave = prefs_get_string(PREF_URL_SAVE_CMD);
+    auto_gchar gchar* urlsave = prefs_get_string(PREF_URL_SAVE_CMD);
     if (urlsave == NULL) {
         urlsave = g_strdup("(built-in)");
     }
     cons_show("Default '/url save' command (/executable urlsave)                        : %s", urlsave);
-    g_free(urlsave);
 
-    gchar* editor = prefs_get_string(PREF_COMPOSE_EDITOR);
+    auto_gchar gchar* editor = prefs_get_string(PREF_COMPOSE_EDITOR);
     cons_show("Default '/editor' command (/executable editor)                           : %s", editor);
-    g_free(editor);
+
+    auto_gchar gchar* vcard_cmd = prefs_get_string(PREF_VCARD_PHOTO_CMD);
+    cons_show("Default '/vcard photo open' command (/executable vcard_photo)            : %s", vcard_cmd);
 }
 
 void
@@ -2303,6 +2235,7 @@ cons_show_connection_prefs(void)
     cons_autoping_setting();
     cons_autoconnect_setting();
     cons_rooms_cache_setting();
+    cons_strophe_setting();
 
     cons_alert(NULL);
 }
@@ -2313,11 +2246,10 @@ cons_show_otr_prefs(void)
     cons_show("OTR preferences:");
     cons_show("");
 
-    char* policy_value = prefs_get_string(PREF_OTR_POLICY);
+    auto_gchar gchar* policy_value = prefs_get_string(PREF_OTR_POLICY);
     cons_show("OTR policy (/otr policy) : %s", policy_value);
-    g_free(policy_value);
 
-    char* log_value = prefs_get_string(PREF_OTR_LOG);
+    auto_gchar gchar* log_value = prefs_get_string(PREF_OTR_LOG);
     if (strcmp(log_value, "on") == 0) {
         cons_show("OTR logging (/otr log)   : ON");
     } else if (strcmp(log_value, "off") == 0) {
@@ -2325,11 +2257,9 @@ cons_show_otr_prefs(void)
     } else {
         cons_show("OTR logging (/otr log)   : Redacted");
     }
-    g_free(log_value);
 
-    char* ch = prefs_get_otr_char();
+    auto_char char* ch = prefs_get_otr_char();
     cons_show("OTR char (/otr char)     : %s", ch);
-    free(ch);
 
     if (prefs_get_boolean(PREF_OTR_SENDFILE)) {
         cons_show("Allow sending unencrypted files in an OTR session via /sendfile (/otr sendfile): ON");
@@ -2346,7 +2276,7 @@ cons_show_pgp_prefs(void)
     cons_show("PGP preferences:");
     cons_show("");
 
-    char* log_value = prefs_get_string(PREF_PGP_LOG);
+    auto_gchar gchar* log_value = prefs_get_string(PREF_PGP_LOG);
     if (strcmp(log_value, "on") == 0) {
         cons_show("PGP logging (/pgp log)   : ON");
     } else if (strcmp(log_value, "off") == 0) {
@@ -2354,11 +2284,9 @@ cons_show_pgp_prefs(void)
     } else {
         cons_show("PGP logging (/pgp log)   : Redacted");
     }
-    g_free(log_value);
 
-    char* ch = prefs_get_pgp_char();
+    auto_char char* ch = prefs_get_pgp_char();
     cons_show("PGP char (/pgp char)     : %s", ch);
-    free(ch);
 
     if (prefs_get_boolean(PREF_PGP_SENDFILE)) {
         cons_show("Allow sending unencrypted files via /sendfile while otherwise using PGP (/pgp sendfile): ON");
@@ -2375,11 +2303,10 @@ cons_show_omemo_prefs(void)
     cons_show("OMEMO preferences:");
     cons_show("");
 
-    char* policy_value = prefs_get_string(PREF_OMEMO_POLICY);
+    auto_gchar gchar* policy_value = prefs_get_string(PREF_OMEMO_POLICY);
     cons_show("OMEMO policy (/omemo policy) : %s", policy_value);
-    g_free(policy_value);
 
-    char* log_value = prefs_get_string(PREF_OMEMO_LOG);
+    auto_gchar gchar* log_value = prefs_get_string(PREF_OMEMO_LOG);
     if (strcmp(log_value, "on") == 0) {
         cons_show("OMEMO logging (/omemo log)   : ON");
     } else if (strcmp(log_value, "off") == 0) {
@@ -2387,11 +2314,9 @@ cons_show_omemo_prefs(void)
     } else {
         cons_show("OMEMO logging (/omemo log)   : Redacted");
     }
-    g_free(log_value);
 
-    char* ch = prefs_get_omemo_char();
+    auto_char char* ch = prefs_get_omemo_char();
     cons_show("OMEMO char (/omemo char)     : %s", ch);
-    free(ch);
 
     cons_alert(NULL);
 }
@@ -2402,7 +2327,7 @@ cons_show_ox_prefs(void)
     cons_show("OX preferences:");
     cons_show("");
 
-    char* log_value = prefs_get_string(PREF_OX_LOG);
+    auto_gchar gchar* log_value = prefs_get_string(PREF_OX_LOG);
     if (strcmp(log_value, "on") == 0) {
         cons_show("OX logging (/ox log)   : ON");
     } else if (strcmp(log_value, "off") == 0) {
@@ -2410,11 +2335,9 @@ cons_show_ox_prefs(void)
     } else {
         cons_show("OX logging (/ox log)   : Redacted");
     }
-    g_free(log_value);
 
-    char* ch = prefs_get_ox_char();
+    auto_char char* ch = prefs_get_ox_char();
     cons_show("OX char (/ox char)     : %s", ch);
-    free(ch);
 
     cons_alert(NULL);
 }
@@ -2578,24 +2501,21 @@ void
 cons_show_contact_online(PContact contact, Resource* resource, GDateTime* last_activity)
 {
     const char* show = string_from_resource_presence(resource->presence);
-    char* display_str = p_contact_create_display_string(contact, resource->name);
+    auto_char char* display_str = p_contact_create_display_string(contact, resource->name);
 
     ProfWin* console = wins_get_console();
     win_show_status_string(console, display_str, show, resource->status, last_activity,
                            "++", "online");
-
-    free(display_str);
 }
 
 void
 cons_show_contact_offline(PContact contact, char* resource, char* status)
 {
-    char* display_str = p_contact_create_display_string(contact, resource);
+    auto_char char* display_str = p_contact_create_display_string(contact, resource);
 
     ProfWin* console = wins_get_console();
     win_show_status_string(console, display_str, "offline", status, NULL, "--",
                            "offline");
-    free(display_str);
 }
 
 void
@@ -2621,7 +2541,7 @@ cons_alert(ProfWin* alert_origin_window)
     if (current->type != WIN_CONSOLE) {
         status_bar_new(1, WIN_CONSOLE, "console");
 
-        char* win_name;
+        auto_char char* win_name;
         if (alert_origin_window) {
             win_name = win_to_string(alert_origin_window);
         } else {
@@ -2632,8 +2552,6 @@ cons_alert(ProfWin* alert_origin_window)
         if (!item) {
             alert_list = g_list_append(alert_list, g_strdup(win_name));
         }
-
-        free(win_name);
     }
 }
 
@@ -2848,15 +2766,8 @@ _cons_splash_logo(void)
     win_println(console, THEME_SPLASH, "-", "|_|                                   (____/ ");
     win_println(console, THEME_SPLASH, "-", "");
 
-    if (strcmp(PACKAGE_STATUS, "development") == 0) {
-#ifdef HAVE_GIT_VERSION
-        win_println(console, THEME_DEFAULT, "-", "Version %sdev.%s.%s", PACKAGE_VERSION, PROF_GIT_BRANCH, PROF_GIT_REVISION);
-#else
-        win_println(console, THEME_DEFAULT, "Version %sdev", PACKAGE_VERSION);
-#endif
-    } else {
-        win_println(console, THEME_DEFAULT, "-", "Version %s", PACKAGE_VERSION);
-    }
+    auto_gchar gchar* prof_version = prof_get_version();
+    win_println(console, THEME_DEFAULT, "-", "Version %s", prof_version);
 }
 
 static void
@@ -2970,11 +2881,10 @@ cons_clear_alerts(void)
 void
 cons_remove_alert(ProfWin* window)
 {
-    char* win_name = win_to_string(window);
+    auto_char char* win_name = win_to_string(window);
     GList* item = g_list_find_custom(alert_list, win_name, (GCompareFunc)g_strcmp0);
     alert_list = g_list_remove_link(alert_list, item);
     g_list_free_full(item, g_free);
-    free(win_name);
 }
 
 void
@@ -2986,3 +2896,60 @@ cons_mood_setting(void)
         cons_show("Display user mood (/mood)                 : OFF");
     }
 }
+
+void
+cons_strophe_setting(void)
+{
+    const char* sm_setting = "OFF";
+    if (prefs_get_boolean(PREF_STROPHE_SM_ENABLED)) {
+        if (prefs_get_boolean(PREF_STROPHE_SM_RESEND)) {
+            sm_setting = "ON";
+        } else {
+            sm_setting = "NO-RESEND";
+        }
+    }
+    cons_show("XEP-0198 Stream-Management                : %s", sm_setting);
+    cons_show("libstrophe Verbosity                      : %s", prefs_get_string(PREF_STROPHE_VERBOSITY));
+}
+
+void
+cons_privacy_setting(void)
+{
+    cons_show("Database logging                                           : %s", prefs_get_string(PREF_DBLOG));
+
+    if (prefs_get_boolean(PREF_CHLOG)) {
+        cons_show("Chat logging (/logging chat)                               : ON");
+    } else {
+        cons_show("Chat logging (/logging chat)                               : OFF");
+    }
+    if (prefs_get_boolean(PREF_HISTORY)) {
+        cons_show("Chat history (/history)                                    : ON");
+    } else {
+        cons_show("Chat history (/history)                                    : OFF");
+    }
+
+    if (prefs_get_boolean(PREF_REVEAL_OS)) {
+        cons_show("Reveal OS name when asked for software");
+        cons_show("version (XEP-0092) (/privacy os)                           : ON");
+    } else {
+        cons_show("Reveal OS name when asked for software");
+        cons_show("version (XEP-0092) (/privacy os)                           : OFF");
+    }
+
+    if (connection_get_status() == JABBER_CONNECTED) {
+        char* account_name = session_get_account_name();
+        ProfAccount* account = accounts_get_account(account_name);
+
+        if (account->client) {
+            cons_show("Client name (/account set <account> clientid)              : %s", account->client);
+        } else {
+            auto_gchar gchar* prof_version = prof_get_version();
+            cons_show("Client name (/account set <account> clientid)              : Profanity %s", prof_version);
+        }
+        if (account->max_sessions > 0) {
+            cons_show("Max sessions alarm (/account set <account> session_alarm)  : %d", account->max_sessions);
+        } else {
+            cons_show("Max sessions alarm (/account set <account> session_alarm)  : not set");
+        }
+    }
+}
