@@ -35,10 +35,6 @@
 
 #include "config.h"
 
-#ifdef HAVE_GIT_VERSION
-#include "gitversion.h"
-#endif
-
 #include <stdlib.h>
 #include <string.h>
 #include <assert.h>
@@ -82,7 +78,6 @@
 #include "otr/otr.h"
 #endif
 
-static char* win_title;
 static int inp_size;
 static gboolean perform_resize = FALSE;
 static GTimer* ui_idle_time;
@@ -145,10 +140,8 @@ ui_update(void)
     doupdate();
 
     if (perform_resize) {
-        signal(SIGWINCH, SIG_IGN);
-        ui_resize();
         perform_resize = FALSE;
-        signal(SIGWINCH, ui_sigwinch_handler);
+        ui_resize();
     }
 }
 
@@ -231,14 +224,12 @@ ui_load_colours(void)
 void
 ui_contact_online(char* barejid, Resource* resource, GDateTime* last_activity)
 {
-    char* show_console = prefs_get_string(PREF_STATUSES_CONSOLE);
-    char* show_chat_win = prefs_get_string(PREF_STATUSES_CHAT);
+    auto_gchar gchar* show_console = prefs_get_string(PREF_STATUSES_CONSOLE);
+    auto_gchar gchar* show_chat_win = prefs_get_string(PREF_STATUSES_CHAT);
     PContact contact = roster_get_contact(barejid);
 
     // show nothing
     if (g_strcmp0(p_contact_subscription(contact), "none") == 0) {
-        free(show_console);
-        free(show_chat_win);
         return;
     }
 
@@ -265,9 +256,6 @@ ui_contact_online(char* barejid, Resource* resource, GDateTime* last_activity)
             chatwin_contact_online(chatwin, resource, last_activity);
         }
     }
-
-    free(show_console);
-    free(show_chat_win);
 }
 
 void
@@ -277,17 +265,21 @@ ui_contact_typing(const char* const barejid, const char* const resource)
     ProfWin* window = (ProfWin*)chatwin;
     ChatSession* session = chat_session_get(barejid);
 
-    if (prefs_get_boolean(PREF_INTYPE)) {
-        // no chat window for user
-        if (chatwin == NULL) {
+    // no chat window for user
+    if (chatwin == NULL) {
+        if (prefs_get_boolean(PREF_INTYPE_CONSOLE)) {
             cons_show_typing(barejid);
+        }
 
-            // have chat window but not currently in it
-        } else if (!wins_is_current(window)) {
+        // have chat window but not currently in it
+    } else if (!wins_is_current(window)) {
+        if (prefs_get_boolean(PREF_INTYPE_CONSOLE)) {
             cons_show_typing(barejid);
+        }
 
-            // in chat window with user, no session or session with resource
-        } else if (!session || (session && g_strcmp0(session->resource, resource) == 0)) {
+        // in chat window with user, no session or session with resource
+    } else if (!session || (session && g_strcmp0(session->resource, resource) == 0)) {
+        if (prefs_get_boolean(PREF_INTYPE)) {
             title_bar_set_typing(TRUE);
 
             int num = wins_get_num(window);
@@ -699,9 +691,8 @@ ui_focus_win(ProfWin* window)
     }
     status_bar_current(i);
 
-    char* identifier = win_get_tab_identifier(window);
+    auto_char char* identifier = win_get_tab_identifier(window);
     status_bar_active(i, window->type, identifier);
-    free(identifier);
 }
 
 void
@@ -767,20 +758,18 @@ ui_print_system_msg_from_recipient(const char* const barejid, const char* messag
         return;
 
     ProfChatWin* chatwin = wins_get_chat(barejid);
-    ProfWin* window = (ProfWin*)chatwin;
-    if (window == NULL) {
-        window = wins_new_chat(barejid);
-        if (window) {
-            chatwin = (ProfChatWin*)window;
-            int num = wins_get_num(window);
+    if (chatwin == NULL) {
+        chatwin = chatwin_new(barejid);
+        if (chatwin) {
+            int num = wins_get_num((ProfWin*)chatwin);
             status_bar_active(num, WIN_CHAT, chatwin->barejid);
         } else {
-            window = wins_get_console();
+            chatwin = (ProfChatWin*)wins_get_console();
             status_bar_active(1, WIN_CONSOLE, "console");
         }
     }
 
-    win_println(window, THEME_DEFAULT, "-", "*%s %s", barejid, message);
+    win_println((ProfWin*)chatwin, THEME_DEFAULT, "-", "*%s %s", barejid, message);
 }
 
 void
@@ -993,10 +982,27 @@ ui_win_unread(int index)
     }
 }
 
-char*
-ui_ask_password(void)
+gboolean
+ui_win_has_attention(int index)
 {
-    status_bar_set_prompt("Enter password:");
+    gboolean ret = FALSE;
+
+    ProfWin* window = wins_get_by_num(index);
+    if (window) {
+        ret = win_has_attention(window);
+    }
+
+    return ret;
+}
+
+char*
+ui_ask_password(gboolean confirm)
+{
+    if (!confirm) {
+        status_bar_set_prompt("Enter password:");
+    } else {
+        status_bar_set_prompt("Confirm password:");
+    }
     return inp_get_password();
 }
 
@@ -1033,9 +1039,8 @@ ui_ask_pgp_passphrase(const char* hint, int prev_fail)
 void
 ui_contact_offline(char* barejid, char* resource, char* status)
 {
-    char* show_console = prefs_get_string(PREF_STATUSES_CONSOLE);
-    char* show_chat_win = prefs_get_string(PREF_STATUSES_CHAT);
-    Jid* jid = jid_create_from_bare_and_resource(barejid, resource);
+    auto_gchar gchar* show_console = prefs_get_string(PREF_STATUSES_CONSOLE);
+    auto_gchar gchar* show_chat_win = prefs_get_string(PREF_STATUSES_CHAT);
     PContact contact = roster_get_contact(barejid);
     if (p_contact_subscription(contact)) {
         if (strcmp(p_contact_subscription(contact), "none") != 0) {
@@ -1070,30 +1075,25 @@ ui_contact_offline(char* barejid, char* resource, char* status)
     if (chatwin && chatwin->resource_override && (g_strcmp0(resource, chatwin->resource_override) == 0)) {
         FREE_SET_NULL(chatwin->resource_override);
     }
-
-    g_free(show_console);
-    g_free(show_chat_win);
-    jid_destroy(jid);
 }
 
 void
 ui_clear_win_title(void)
 {
-    printf("%c]0;%c", '\033', '\007');
+    fputs("\e]0;\a", stdout);
+    fflush(stdout);
 }
 
 void
 ui_goodbye_title(void)
 {
-    int result = system("/bin/echo -ne \"\033]0;Thanks for using Profanity\007\"");
-    if (result == -1)
-        log_error("Error printing title on shutdown");
+    fputs("\e]0;Thanks for using Profanity\a", stdout);
+    fflush(stdout);
 }
 
 static void
 _ui_draw_term_title(void)
 {
-    char new_win_title[100];
     jabber_conn_status_t status = connection_get_status();
 
     if (status == JABBER_CONNECTED) {
@@ -1101,29 +1101,14 @@ _ui_draw_term_title(void)
         gint unread = wins_get_total_unread();
 
         if (unread != 0) {
-            snprintf(new_win_title, sizeof(new_win_title),
-                     "/bin/echo -n \"%c]0;%s (%d) - %s%c\"", '\033', "Profanity",
-                     unread, jid, '\007');
+            fprintf(stdout, "\e]0;Profanity (%d) - %s\a", unread, jid);
         } else {
-            snprintf(new_win_title, sizeof(new_win_title),
-                     "/bin/echo -n \"%c]0;%s - %s%c\"", '\033', "Profanity", jid,
-                     '\007');
+            fprintf(stdout, "\e]0;Profanity - %s\a", jid);
         }
     } else {
-        snprintf(new_win_title, sizeof(new_win_title), "/bin/echo -n \"%c]0;%s%c\"", '\033',
-                 "Profanity", '\007');
-    }
-    if (g_strcmp0(win_title, new_win_title) != 0) {
-        // print to x-window title bar
-        int res = system(new_win_title);
-        if (res == -1) {
-            log_error("Error writing terminal window title.");
-        }
-        if (win_title) {
-            free(win_title);
-        }
-        win_title = strdup(new_win_title);
+        fputs("\e]0;Profanity\a", stdout);
     }
+    fflush(stdout);
 }
 
 void
@@ -1269,13 +1254,12 @@ void
 ui_show_software_version(const char* const jid, const char* const presence,
                          const char* const name, const char* const version, const char* const os)
 {
-    Jid* jidp = jid_create(jid);
+    auto_jid Jid* jidp = jid_create(jid);
     ProfWin* window = NULL;
     ProfWin* chatwin = (ProfWin*)wins_get_chat(jidp->barejid);
     ProfWin* mucwin = (ProfWin*)wins_get_muc(jidp->barejid);
     ProfWin* privwin = (ProfWin*)wins_get_private(jidp->fulljid);
     ProfWin* console = wins_get_console();
-    jid_destroy(jidp);
 
     if (chatwin) {
         if (wins_is_current(chatwin)) {
