@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2022 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -77,18 +77,16 @@ _occuptantswin_occupant(ProfLayoutSplit* layout, GList* item, gboolean showjid,
 
     GString* msg = g_string_new(spaces->str);
 
-    char* ch = prefs_get_occupants_char();
+    auto_char char* ch = prefs_get_occupants_char();
     if (ch) {
         g_string_append_printf(msg, "%s", ch);
-        free(ch);
     }
 
     gboolean wrap = prefs_get_boolean(PREF_OCCUPANTS_WRAP);
 
     if (isoffline) {
-        Jid* jid = jid_create(item->data);
+        auto_jid Jid* jid = jid_create(item->data);
         g_string_append(msg, jid->barejid);
-        jid_destroy(jid);
     } else {
         g_string_append(msg, occupant->nick);
     }
@@ -131,10 +129,9 @@ occupantswin_occupants(const char* const roomjid)
 
             GString* prefix = g_string_new(" ");
 
-            char* ch = prefs_get_occupants_header_char();
+            auto_char char* ch = prefs_get_occupants_header_char();
             if (ch) {
                 g_string_append_printf(prefix, "%s", ch);
-                free(ch);
             }
 
             if (prefs_get_boolean(PREF_MUC_PRIVILEGES)) {
@@ -210,7 +207,7 @@ occupantswin_occupants(const char* const roomjid)
                     g_string_free(role, TRUE);
                     roster_curr = members;
                     while (roster_curr) {
-                        Jid* jid = jid_create(roster_curr->data);
+                        auto_jid Jid* jid = jid_create(roster_curr->data);
                         gboolean found = false;
                         GList* iter = online_occupants;
                         for (; iter != NULL; iter = iter->next) {
@@ -225,7 +222,6 @@ occupantswin_occupants(const char* const roomjid)
                             offline_occupants = g_list_append(offline_occupants, jid->barejid);
                         }
 
-                        jid_destroy(jid);
                         roster_curr = g_list_next(roster_curr);
                     }
                     g_list_free(members);
