@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2022 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -98,11 +98,13 @@ rosterwin_roster(void)
     ProfLayoutSplit* layout = (ProfLayoutSplit*)console->layout;
     assert(layout->memcheck == LAYOUT_SPLIT_MEMCHECK);
 
-    if (layout->subwin != NULL) {
-        werase(layout->subwin);
+    if (layout->subwin == NULL) {
+        return;
     }
 
-    char* roomspos = prefs_get_string(PREF_ROSTER_ROOMS_POS);
+    werase(layout->subwin);
+
+    auto_gchar gchar* roomspos = prefs_get_string(PREF_ROSTER_ROOMS_POS);
     if (prefs_get_boolean(PREF_ROSTER_ROOMS) && (g_strcmp0(roomspos, "first") == 0)) {
         _rosterwin_print_rooms(layout);
 
@@ -111,24 +113,22 @@ rosterwin_roster(void)
         GList* curr = privchats;
         while (curr) {
             ProfPrivateWin* privwin = curr->data;
-            Jid* jidp = jid_create(privwin->fulljid);
+            auto_jid Jid* jidp = jid_create(privwin->fulljid);
             if (!muc_active(jidp->barejid)) {
                 orphaned_privchats = g_list_append(orphaned_privchats, privwin);
             }
-            jid_destroy(jidp);
             curr = g_list_next(curr);
         }
 
-        char* privpref = prefs_get_string(PREF_ROSTER_PRIVATE);
+        auto_gchar gchar* privpref = prefs_get_string(PREF_ROSTER_PRIVATE);
         if (g_strcmp0(privpref, "group") == 0 || orphaned_privchats) {
             _rosterwin_private_chats(layout, orphaned_privchats);
         }
-        g_free(privpref);
         g_list_free(orphaned_privchats);
     }
 
     if (prefs_get_boolean(PREF_ROSTER_CONTACTS)) {
-        char* by = prefs_get_string(PREF_ROSTER_BY);
+        auto_gchar gchar* by = prefs_get_string(PREF_ROSTER_BY);
         if (g_strcmp0(by, "presence") == 0) {
             _rosterwin_contacts_by_presence(layout, "chat", "Available for chat");
             _rosterwin_contacts_by_presence(layout, "online", "Online");
@@ -152,7 +152,6 @@ rosterwin_roster(void)
         if (prefs_get_boolean(PREF_ROSTER_UNSUBSCRIBED)) {
             _rosteriwin_unsubscribed(layout);
         }
-        g_free(by);
     }
 
     if (prefs_get_boolean(PREF_ROSTER_ROOMS) && (g_strcmp0(roomspos, "last") == 0)) {
@@ -163,24 +162,20 @@ rosterwin_roster(void)
         GList* curr = privchats;
         while (curr) {
             ProfPrivateWin* privwin = curr->data;
-            Jid* jidp = jid_create(privwin->fulljid);
+            auto_jid Jid* jidp = jid_create(privwin->fulljid);
             if (!muc_active(jidp->barejid)) {
                 orphaned_privchats = g_list_append(orphaned_privchats, privwin);
             }
-            jid_destroy(jidp);
             curr = g_list_next(curr);
         }
 
-        char* privpref = prefs_get_string(PREF_ROSTER_PRIVATE);
+        auto_gchar gchar* privpref = prefs_get_string(PREF_ROSTER_PRIVATE);
         if (g_strcmp0(privpref, "group") == 0 || orphaned_privchats) {
             _rosterwin_private_chats(layout, orphaned_privchats);
         }
-        g_free(privpref);
         g_list_free(privchats);
         g_list_free(orphaned_privchats);
     }
-
-    g_free(roomspos);
 }
 
 static void
@@ -188,13 +183,12 @@ _rosterwin_contacts_all(ProfLayoutSplit* layout)
 {
     GSList* contacts = NULL;
 
-    char* order = prefs_get_string(PREF_ROSTER_ORDER);
+    auto_gchar gchar* order = prefs_get_string(PREF_ROSTER_ORDER);
     if (g_strcmp0(order, "presence") == 0) {
         contacts = roster_get_contacts(ROSTER_ORD_PRESENCE);
     } else {
         contacts = roster_get_contacts(ROSTER_ORD_NAME);
     }
-    g_free(order);
 
     GSList* filtered_contacts = _filter_contacts(contacts);
     g_slist_free(contacts);
@@ -258,13 +252,12 @@ _rosterwin_contacts_by_group(ProfLayoutSplit* layout, char* group)
 {
     GSList* contacts = NULL;
 
-    char* order = prefs_get_string(PREF_ROSTER_ORDER);
+    auto_gchar gchar* order = prefs_get_string(PREF_ROSTER_ORDER);
     if (g_strcmp0(order, "presence") == 0) {
         contacts = roster_get_group(group, ROSTER_ORD_PRESENCE);
     } else {
         contacts = roster_get_group(group, ROSTER_ORD_NAME);
     }
-    g_free(order);
 
     GSList* filtered_contacts = _filter_contacts(contacts);
     g_slist_free(contacts);
@@ -314,13 +307,12 @@ _rosterwin_unsubscribed_item(ProfLayoutSplit* layout, ProfChatWin* chatwin)
             indent--;
         }
     }
-    char* ch = prefs_get_roster_contact_char();
+    auto_char char* ch = prefs_get_roster_contact_char();
     if (ch) {
         g_string_append_printf(msg, "%s", ch);
-        free(ch);
     }
 
-    char* unreadpos = prefs_get_string(PREF_ROSTER_UNREAD);
+    auto_gchar gchar* unreadpos = prefs_get_string(PREF_ROSTER_UNREAD);
     if ((g_strcmp0(unreadpos, "before") == 0) && unread > 0) {
         g_string_append_printf(msg, "(%d) ", unread);
         unread = 0;
@@ -329,7 +321,6 @@ _rosterwin_unsubscribed_item(ProfLayoutSplit* layout, ProfChatWin* chatwin)
     if ((g_strcmp0(unreadpos, "after") == 0) && unread > 0) {
         g_string_append_printf(msg, " (%d)", unread);
     }
-    g_free(unreadpos);
 
     win_sub_newline_lazy(layout->subwin);
     gboolean wrap = prefs_get_boolean(PREF_ROSTER_WRAP);
@@ -377,13 +368,12 @@ _rosterwin_contact(ProfLayoutSplit* layout, PContact contact)
             indent--;
         }
     }
-    char* ch = prefs_get_roster_contact_char();
+    auto_char char* ch = prefs_get_roster_contact_char();
     if (ch) {
         g_string_append_printf(msg, "%s", ch);
-        free(ch);
     }
 
-    char* unreadpos = prefs_get_string(PREF_ROSTER_UNREAD);
+    auto_gchar gchar* unreadpos = prefs_get_string(PREF_ROSTER_UNREAD);
     if ((g_strcmp0(unreadpos, "before") == 0) && unread > 0) {
         g_string_append_printf(msg, "(%d) ", unread);
         unread = 0;
@@ -397,7 +387,6 @@ _rosterwin_contact(ProfLayoutSplit* layout, PContact contact)
             unread = 0;
         }
     }
-    g_free(unreadpos);
 
     win_sub_newline_lazy(layout->subwin);
     gboolean wrap = prefs_get_boolean(PREF_ROSTER_WRAP);
@@ -437,9 +426,8 @@ _rosterwin_presence(ProfLayoutSplit* layout, const char* presence, const char* s
         return;
     }
 
-    char* by = prefs_get_string(PREF_ROSTER_BY);
+    auto_gchar gchar* by = prefs_get_string(PREF_ROSTER_BY);
     gboolean by_presence = g_strcmp0(by, "presence") == 0;
-    g_free(by);
 
     int presence_indent = prefs_get_roster_presence_indent();
     if (presence_indent > 0) {
@@ -529,10 +517,9 @@ _rosterwin_resources(ProfLayoutSplit* layout, PContact contact, int current_inde
 
             wattron(layout->subwin, theme_attrs(resource_presence_colour));
             GString* msg = g_string_new("");
-            char* ch = prefs_get_roster_resource_char();
+            auto_char char* ch = prefs_get_roster_resource_char();
             if (ch) {
                 g_string_append_printf(msg, "%s", ch);
-                free(ch);
             } else {
                 g_string_append(msg, " ");
             }
@@ -541,11 +528,10 @@ _rosterwin_resources(ProfLayoutSplit* layout, PContact contact, int current_inde
                 g_string_append_printf(msg, " %d", resource->priority);
             }
 
-            char* unreadpos = prefs_get_string(PREF_ROSTER_UNREAD);
+            auto_gchar gchar* unreadpos = prefs_get_string(PREF_ROSTER_UNREAD);
             if ((g_strcmp0(unreadpos, "after") == 0) && unread > 0) {
                 g_string_append_printf(msg, " (%d)", unread);
             }
-            g_free(unreadpos);
 
             gboolean wrap = prefs_get_boolean(PREF_ROSTER_WRAP);
             win_sub_print(layout->subwin, msg->str, FALSE, wrap, 0);
@@ -560,7 +546,7 @@ _rosterwin_resources(ProfLayoutSplit* layout, PContact contact, int current_inde
         } else {
             gboolean wrap = prefs_get_boolean(PREF_ROSTER_WRAP);
 
-            char* unreadpos = prefs_get_string(PREF_ROSTER_UNREAD);
+            auto_gchar gchar* unreadpos = prefs_get_string(PREF_ROSTER_UNREAD);
             if ((g_strcmp0(unreadpos, "after") == 0) && unread > 0) {
                 GString* unreadmsg = g_string_new("");
                 g_string_append_printf(unreadmsg, " (%d)", unread);
@@ -573,7 +559,6 @@ _rosterwin_resources(ProfLayoutSplit* layout, PContact contact, int current_inde
                 g_string_free(unreadmsg, TRUE);
                 wattroff(layout->subwin, theme_attrs(presence_colour));
             }
-            g_free(unreadpos);
 
             int resource_indent = prefs_get_roster_resource_indent();
             if (resource_indent > 0) {
@@ -593,10 +578,9 @@ _rosterwin_resources(ProfLayoutSplit* layout, PContact contact, int current_inde
                     g_string_append(msg, " ");
                     this_indent--;
                 }
-                char* ch = prefs_get_roster_resource_char();
+                auto_char char* ch = prefs_get_roster_resource_char();
                 if (ch) {
                     g_string_append_printf(msg, "%s", ch);
-                    free(ch);
                 }
                 g_string_append(msg, resource->name);
                 if (prefs_get_boolean(PREF_ROSTER_PRIORITY)) {
@@ -620,7 +604,7 @@ _rosterwin_resources(ProfLayoutSplit* layout, PContact contact, int current_inde
         theme_item_t presence_colour = _get_roster_theme(theme_type, presence);
         gboolean wrap = prefs_get_boolean(PREF_ROSTER_WRAP);
 
-        char* unreadpos = prefs_get_string(PREF_ROSTER_UNREAD);
+        auto_gchar gchar* unreadpos = prefs_get_string(PREF_ROSTER_UNREAD);
         if ((g_strcmp0(unreadpos, "after") == 0) && unread > 0) {
             GString* unreadmsg = g_string_new("");
             g_string_append_printf(unreadmsg, " (%d)", unread);
@@ -630,12 +614,11 @@ _rosterwin_resources(ProfLayoutSplit* layout, PContact contact, int current_inde
             g_string_free(unreadmsg, TRUE);
             wattroff(layout->subwin, theme_attrs(presence_colour));
         }
-        g_free(unreadpos);
         _rosterwin_presence(layout, presence, status, current_indent);
     } else {
         gboolean wrap = prefs_get_boolean(PREF_ROSTER_WRAP);
 
-        char* unreadpos = prefs_get_string(PREF_ROSTER_UNREAD);
+        auto_gchar gchar* unreadpos = prefs_get_string(PREF_ROSTER_UNREAD);
         if ((g_strcmp0(unreadpos, "after") == 0) && unread > 0) {
             GString* unreadmsg = g_string_new("");
             g_string_append_printf(unreadmsg, " (%d)", unread);
@@ -647,7 +630,6 @@ _rosterwin_resources(ProfLayoutSplit* layout, PContact contact, int current_inde
             g_string_free(unreadmsg, TRUE);
             wattroff(layout->subwin, theme_attrs(presence_colour));
         }
-        g_free(unreadpos);
     }
 
     g_list_free(resources);
@@ -661,13 +643,12 @@ _rosterwin_rooms(ProfLayoutSplit* layout, char* title, GList* rooms)
     while (curr_room) {
         ProfMucWin* mucwin = wins_get_muc(curr_room->data);
         if (mucwin) {
-            char* order = prefs_get_string(PREF_ROSTER_ROOMS_ORDER);
+            auto_gchar gchar* order = prefs_get_string(PREF_ROSTER_ROOMS_ORDER);
             if (g_strcmp0(order, "unread") == 0) {
                 rooms_sorted = g_list_insert_sorted(rooms_sorted, mucwin, (GCompareFunc)_compare_rooms_unread);
             } else {
                 rooms_sorted = g_list_insert_sorted(rooms_sorted, mucwin, (GCompareFunc)_compare_rooms_name);
             }
-            g_free(order);
         }
         curr_room = g_list_next(curr_room);
     }
@@ -694,13 +675,12 @@ _rosterwin_rooms_by_service(ProfLayoutSplit* layout)
     GList* services = NULL;
     while (curr) {
         char* roomjid = curr->data;
-        Jid* jidp = jid_create(roomjid);
+        auto_jid Jid* jidp = jid_create(roomjid);
 
         if (!g_list_find_custom(services, jidp->domainpart, (GCompareFunc)g_strcmp0)) {
             services = g_list_insert_sorted(services, strdup(jidp->domainpart), (GCompareFunc)g_strcmp0);
         }
 
-        jid_destroy(jidp);
         curr = g_list_next(curr);
     }
 
@@ -712,13 +692,12 @@ _rosterwin_rooms_by_service(ProfLayoutSplit* layout)
         curr = rooms;
         while (curr) {
             char* roomjid = curr->data;
-            Jid* jidp = jid_create(roomjid);
+            auto_jid Jid* jidp = jid_create(roomjid);
 
             if (g_strcmp0(curr_service->data, jidp->domainpart) == 0) {
                 filtered_rooms = g_list_append(filtered_rooms, strdup(jidp->barejid));
             }
 
-            jid_destroy(jidp);
             curr = g_list_next(curr);
         }
 
@@ -756,25 +735,23 @@ _rosterwin_room(ProfLayoutSplit* layout, ProfMucWin* mucwin)
             indent--;
         }
     }
-    char* ch = prefs_get_roster_room_char();
+    auto_char char* ch = prefs_get_roster_room_char();
     if (ch) {
         g_string_append_printf(msg, "%s", ch);
-        free(ch);
     }
 
-    char* unreadpos = prefs_get_string(PREF_ROSTER_ROOMS_UNREAD);
+    auto_gchar gchar* unreadpos = prefs_get_string(PREF_ROSTER_ROOMS_UNREAD);
     if ((g_strcmp0(unreadpos, "before") == 0) && mucwin->unread > 0) {
         g_string_append_printf(msg, "(%d) ", mucwin->unread);
     }
 
-    char* use_as_name = prefs_get_string(PREF_ROSTER_ROOMS_USE_AS_NAME);
-    char* roombypref = prefs_get_string(PREF_ROSTER_ROOMS_BY);
+    auto_gchar gchar* use_as_name = prefs_get_string(PREF_ROSTER_ROOMS_USE_AS_NAME);
+    auto_gchar gchar* roombypref = prefs_get_string(PREF_ROSTER_ROOMS_BY);
 
     if (g_strcmp0(roombypref, "service") == 0) {
         if (mucwin->room_name == NULL || (g_strcmp0(use_as_name, "jid") == 0)) {
-            Jid* jidp = jid_create(mucwin->roomjid);
+            auto_jid Jid* jidp = jid_create(mucwin->roomjid);
             g_string_append(msg, jidp->localpart);
-            jid_destroy(jidp);
         } else {
             g_string_append(msg, mucwin->room_name);
         }
@@ -788,25 +765,19 @@ _rosterwin_room(ProfLayoutSplit* layout, ProfMucWin* mucwin)
                 g_string_append(msg, mucwin->room_name);
             }
         } else {
-            Jid* jidp = jid_create(mucwin->roomjid);
+            auto_jid Jid* jidp = jid_create(mucwin->roomjid);
 
             if (mucwin->room_name == NULL || (g_strcmp0(use_as_name, "jid") == 0)) {
                 g_string_append(msg, jidp->localpart);
             } else {
                 g_string_append(msg, mucwin->room_name);
             }
-
-            jid_destroy(jidp);
         }
     }
 
-    g_free(use_as_name);
-    g_free(roombypref);
-
     if ((g_strcmp0(unreadpos, "after") == 0) && mucwin->unread > 0) {
         g_string_append_printf(msg, " (%d)", mucwin->unread);
     }
-    g_free(unreadpos);
 
     win_sub_newline_lazy(layout->subwin);
     gboolean wrap = prefs_get_boolean(PREF_ROSTER_WRAP);
@@ -823,7 +794,7 @@ _rosterwin_room(ProfLayoutSplit* layout, ProfMucWin* mucwin)
         wattroff(layout->subwin, theme_attrs(THEME_ROSTER_ROOM));
     }
 
-    char* privpref = prefs_get_string(PREF_ROSTER_PRIVATE);
+    auto_gchar gchar* privpref = prefs_get_string(PREF_ROSTER_PRIVATE);
     if (g_strcmp0(privpref, "room") == 0) {
         GList* privs = wins_get_private_chats(mucwin->roomjid);
         GList* curr = privs;
@@ -852,24 +823,22 @@ _rosterwin_room(ProfLayoutSplit* layout, ProfMucWin* mucwin)
                 }
             }
 
-            unreadpos = prefs_get_string(PREF_ROSTER_ROOMS_UNREAD);
-            if ((g_strcmp0(unreadpos, "before") == 0) && privwin->unread > 0) {
+            auto_gchar gchar* unreadpos2 = prefs_get_string(PREF_ROSTER_ROOMS_UNREAD);
+            if ((g_strcmp0(unreadpos2, "before") == 0) && privwin->unread > 0) {
                 g_string_append_printf(privmsg, "(%d) ", privwin->unread);
             }
 
-            ch = prefs_get_roster_room_private_char();
-            if (ch) {
-                g_string_append_printf(privmsg, "%s", ch);
-                free(ch);
+            auto_char char* ch2 = prefs_get_roster_room_private_char();
+            if (ch2) {
+                g_string_append_printf(privmsg, "%s", ch2);
             }
 
             char* nick = privwin->fulljid + strlen(mucwin->roomjid) + 1;
             g_string_append(privmsg, nick);
 
-            if ((g_strcmp0(unreadpos, "after") == 0) && privwin->unread > 0) {
+            if ((g_strcmp0(unreadpos2, "after") == 0) && privwin->unread > 0) {
                 g_string_append_printf(privmsg, " (%d)", privwin->unread);
             }
-            g_free(unreadpos);
 
             const char* presence = "offline";
 
@@ -895,14 +864,12 @@ _rosterwin_room(ProfLayoutSplit* layout, ProfMucWin* mucwin)
 
         g_list_free(privs);
     }
-
-    g_free(privpref);
 }
 
 static void
 _rosterwin_print_rooms(ProfLayoutSplit* layout)
 {
-    char* roomsbypref = prefs_get_string(PREF_ROSTER_ROOMS_BY);
+    auto_gchar gchar* roomsbypref = prefs_get_string(PREF_ROSTER_ROOMS_BY);
     if (g_strcmp0(roomsbypref, "service") == 0) {
         _rosterwin_rooms_by_service(layout);
     } else {
@@ -910,7 +877,6 @@ _rosterwin_print_rooms(ProfLayoutSplit* layout)
         _rosterwin_rooms(layout, "Rooms", rooms);
         g_list_free(rooms);
     }
-    g_free(roomsbypref);
 }
 
 static void
@@ -918,7 +884,7 @@ _rosterwin_private_chats(ProfLayoutSplit* layout, GList* orphaned_privchats)
 {
     GList* privs = NULL;
 
-    char* privpref = prefs_get_string(PREF_ROSTER_PRIVATE);
+    auto_gchar gchar* privpref = prefs_get_string(PREF_ROSTER_PRIVATE);
     if (g_strcmp0(privpref, "group") == 0) {
         privs = wins_get_private_chats(NULL);
     } else {
@@ -948,15 +914,14 @@ _rosterwin_private_chats(ProfLayoutSplit* layout, GList* orphaned_privchats)
                 }
             }
 
-            char* unreadpos = prefs_get_string(PREF_ROSTER_ROOMS_UNREAD);
+            auto_gchar gchar* unreadpos = prefs_get_string(PREF_ROSTER_ROOMS_UNREAD);
             if ((g_strcmp0(unreadpos, "before") == 0) && privwin->unread > 0) {
                 g_string_append_printf(privmsg, "(%d) ", privwin->unread);
             }
 
-            char* ch = prefs_get_roster_private_char();
+            auto_char char* ch = prefs_get_roster_private_char();
             if (ch) {
                 g_string_append_printf(privmsg, "%s", ch);
-                free(ch);
             }
 
             g_string_append(privmsg, privwin->fulljid);
@@ -964,11 +929,9 @@ _rosterwin_private_chats(ProfLayoutSplit* layout, GList* orphaned_privchats)
             if ((g_strcmp0(unreadpos, "after") == 0) && privwin->unread > 0) {
                 g_string_append_printf(privmsg, " (%d)", privwin->unread);
             }
-            g_free(unreadpos);
 
-            Jid* jidp = jid_create(privwin->fulljid);
+            auto_jid Jid* jidp = jid_create(privwin->fulljid);
             Occupant* occupant = muc_roster_item(jidp->barejid, jidp->resourcepart);
-            jid_destroy(jidp);
 
             const char* presence = "offline";
             if (occupant) {
@@ -1035,15 +998,14 @@ _rosterwin_unsubscribed_header(ProfLayoutSplit* layout, GList* wins)
     win_sub_newline_lazy(layout->subwin);
 
     GString* header = g_string_new(" ");
-    char* ch = prefs_get_roster_header_char();
+    auto_char char* ch = prefs_get_roster_header_char();
     if (ch) {
         g_string_append_printf(header, "%s", ch);
-        free(ch);
     }
 
     g_string_append(header, "Unsubscribed");
 
-    char* countpref = prefs_get_string(PREF_ROSTER_COUNT);
+    auto_gchar gchar* countpref = prefs_get_string(PREF_ROSTER_COUNT);
     if (g_strcmp0(countpref, "items") == 0) {
         int itemcount = g_list_length(wins);
         if (itemcount == 0 && prefs_get_boolean(PREF_ROSTER_COUNT_ZERO)) {
@@ -1065,7 +1027,6 @@ _rosterwin_unsubscribed_header(ProfLayoutSplit* layout, GList* wins)
             g_string_append_printf(header, " (%d)", unreadcount);
         }
     }
-    g_free(countpref);
 
     gboolean wrap = prefs_get_boolean(PREF_ROSTER_WRAP);
 
@@ -1082,15 +1043,14 @@ _rosterwin_contacts_header(ProfLayoutSplit* layout, const char* const title, GSL
     win_sub_newline_lazy(layout->subwin);
 
     GString* header = g_string_new(" ");
-    char* ch = prefs_get_roster_header_char();
+    auto_char char* ch = prefs_get_roster_header_char();
     if (ch) {
         g_string_append_printf(header, "%s", ch);
-        free(ch);
     }
 
     g_string_append(header, title);
 
-    char* countpref = prefs_get_string(PREF_ROSTER_COUNT);
+    auto_gchar gchar* countpref = prefs_get_string(PREF_ROSTER_COUNT);
     if (g_strcmp0(countpref, "items") == 0) {
         int itemcount = g_slist_length(contacts);
         if (itemcount == 0 && prefs_get_boolean(PREF_ROSTER_COUNT_ZERO)) {
@@ -1116,7 +1076,6 @@ _rosterwin_contacts_header(ProfLayoutSplit* layout, const char* const title, GSL
             g_string_append_printf(header, " (%d)", unreadcount);
         }
     }
-    g_free(countpref);
 
     gboolean wrap = prefs_get_boolean(PREF_ROSTER_WRAP);
 
@@ -1134,14 +1093,13 @@ _rosterwin_rooms_header(ProfLayoutSplit* layout, GList* rooms, char* title)
 {
     win_sub_newline_lazy(layout->subwin);
     GString* header = g_string_new(" ");
-    char* ch = prefs_get_roster_header_char();
+    auto_char char* ch = prefs_get_roster_header_char();
     if (ch) {
         g_string_append_printf(header, "%s", ch);
-        free(ch);
     }
     g_string_append(header, title);
 
-    char* countpref = prefs_get_string(PREF_ROSTER_COUNT);
+    auto_gchar gchar* countpref = prefs_get_string(PREF_ROSTER_COUNT);
     if (g_strcmp0(countpref, "items") == 0) {
         int count = g_list_length(rooms);
         if (count == 0 && prefs_get_boolean(PREF_ROSTER_COUNT_ZERO)) {
@@ -1157,7 +1115,7 @@ _rosterwin_rooms_header(ProfLayoutSplit* layout, GList* rooms, char* title)
             unread += mucwin->unread;
 
             // include private chats
-            char* prefpriv = prefs_get_string(PREF_ROSTER_PRIVATE);
+            auto_gchar gchar* prefpriv = prefs_get_string(PREF_ROSTER_PRIVATE);
             if (g_strcmp0(prefpriv, "room") == 0) {
                 GList* privwins = wins_get_private_chats(mucwin->roomjid);
                 GList* curr_priv = privwins;
@@ -1168,7 +1126,6 @@ _rosterwin_rooms_header(ProfLayoutSplit* layout, GList* rooms, char* title)
                 }
                 g_list_free(privwins);
             }
-            g_free(prefpriv);
 
             curr = g_list_next(curr);
         }
@@ -1179,7 +1136,6 @@ _rosterwin_rooms_header(ProfLayoutSplit* layout, GList* rooms, char* title)
             g_string_append_printf(header, " (%d)", unread);
         }
     }
-    g_free(countpref);
 
     gboolean wrap = prefs_get_boolean(PREF_ROSTER_WRAP);
 
@@ -1198,14 +1154,13 @@ _rosterwin_private_header(ProfLayoutSplit* layout, GList* privs)
     win_sub_newline_lazy(layout->subwin);
 
     GString* title_str = g_string_new(" ");
-    char* ch = prefs_get_roster_header_char();
+    auto_char char* ch = prefs_get_roster_header_char();
     if (ch) {
         g_string_append_printf(title_str, "%s", ch);
-        free(ch);
     }
     g_string_append(title_str, "Private chats");
 
-    char* countpref = prefs_get_string(PREF_ROSTER_COUNT);
+    auto_gchar gchar* countpref = prefs_get_string(PREF_ROSTER_COUNT);
     if (g_strcmp0(countpref, "items") == 0) {
         int itemcount = g_list_length(privs);
         if (itemcount == 0 && prefs_get_boolean(PREF_ROSTER_COUNT_ZERO)) {
@@ -1227,7 +1182,6 @@ _rosterwin_private_header(ProfLayoutSplit* layout, GList* privs)
             g_string_append_printf(title_str, " (%d)", unreadcount);
         }
     }
-    g_free(countpref);
 
     gboolean wrap = prefs_get_boolean(PREF_ROSTER_WRAP);
 
