@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2022 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -79,14 +79,15 @@ static StatusBar* statusbar;
 static WINDOW* statusbar_win;
 
 static int _status_bar_draw_time(int pos);
-static void _status_bar_draw_maintext(int pos);
-static int _status_bar_draw_bracket(gboolean current, int pos, char* ch);
+static int _status_bar_draw_maintext(int pos);
+static int _status_bar_draw_bracket(gboolean current, int pos, const char* ch);
 static int _status_bar_draw_extended_tabs(int pos);
 static int _status_bar_draw_tab(StatusBarTab* tab, int pos, int num);
 static void _destroy_tab(StatusBarTab* tab);
 static int _tabs_width(void);
 static char* _display_name(StatusBarTab* tab);
 static gboolean _extended_new(void);
+static gboolean _tabmode_is_actlist(void);
 
 void
 status_bar_init(void)
@@ -181,10 +182,7 @@ status_bar_inactive(const int win)
 void
 _create_tab(const int win, win_type_t wintype, char* identifier, gboolean highlight)
 {
-    int true_win = win;
-    if (true_win == 0) {
-        true_win = 10;
-    }
+    int true_win = win == 0 ? 10 : win;
 
     StatusBarTab* tab = malloc(sizeof(StatusBarTab));
     tab->identifier = strdup(identifier);
@@ -197,22 +195,21 @@ _create_tab(const int win, win_type_t wintype, char* identifier, gboolean highli
         if (roster_exists()) {
             contact = roster_get_contact(tab->identifier);
         }
-        if (contact && p_contact_name(contact)) {
-            tab->display_name = strdup(p_contact_name(contact));
-        } else {
-            char* pref = prefs_get_string(PREF_STATUSBAR_CHAT);
-            if (g_strcmp0("user", pref) == 0) {
-                Jid* jidp = jid_create(tab->identifier);
+        const char* pcontact_name = contact ? p_contact_name(contact) : NULL;
+        auto_gchar gchar* pref = prefs_get_string(PREF_STATUSBAR_CHAT);
+        if (g_strcmp0("user", pref) == 0) {
+            if (pcontact_name) {
+                tab->display_name = strdup(pcontact_name);
+            } else {
+                auto_jid Jid* jidp = jid_create(tab->identifier);
                 if (jidp) {
                     tab->display_name = jidp->localpart != NULL ? strdup(jidp->localpart) : strdup(jidp->barejid);
-                    jid_destroy(jidp);
                 } else {
                     tab->display_name = strdup(tab->identifier);
                 }
-            } else {
-                tab->display_name = strdup(tab->identifier);
             }
-            g_free(pref);
+        } else {
+            tab->display_name = strdup(tab->identifier);
         }
     }
 
@@ -289,22 +286,60 @@ status_bar_draw(void)
 
     pos = _status_bar_draw_time(pos);
 
-    _status_bar_draw_maintext(pos);
+    pos = _status_bar_draw_maintext(pos);
 
-    pos = getmaxx(stdscr) - _tabs_width();
-    if (pos < 0) {
-        pos = 0;
-    }
-    gint max_tabs = prefs_get_statusbartabs();
-    for (int i = 1; i <= max_tabs; i++) {
-        StatusBarTab* tab = g_hash_table_lookup(statusbar->tabs, GINT_TO_POINTER(i));
-        if (tab) {
-            pos = _status_bar_draw_tab(tab, pos, i);
+    if (!_tabmode_is_actlist()) {
+        pos = getmaxx(stdscr) - _tabs_width();
+        if (pos < 0) {
+            pos = 0;
+        }
+        gint max_tabs = prefs_get_statusbartabs();
+        for (int i = 1; i <= max_tabs; i++) {
+            StatusBarTab* tab = g_hash_table_lookup(statusbar->tabs, GINT_TO_POINTER(i));
+            if (tab) {
+                pos = _status_bar_draw_tab(tab, pos, i);
+            }
+        }
+
+        _status_bar_draw_extended_tabs(pos);
+    } else {
+        pos++;
+        guint print_act = 0;
+        guint tabnum = g_hash_table_size(statusbar->tabs);
+        for (guint i = 1; i <= tabnum; ++i) {
+            StatusBarTab* tab = g_hash_table_lookup(statusbar->tabs, GINT_TO_POINTER(i));
+            if (tab && tab->highlight) {
+                print_act++;
+            }
+        }
+        if (print_act) {
+            pos = _status_bar_draw_bracket(FALSE, pos, "[");
+            mvwprintw(statusbar_win, 0, pos, "Act: ");
+            pos += 5;
+            int status_attrs = theme_attrs(THEME_STATUS_NEW);
+
+            wattron(statusbar_win, status_attrs);
+            for (guint i = 1; i <= tabnum && print_act; ++i) {
+                StatusBarTab* tab = g_hash_table_lookup(statusbar->tabs, GINT_TO_POINTER(i));
+                if (tab && tab->highlight) {
+                    if (print_act == 1) {
+                        mvwprintw(statusbar_win, 0, pos, "%d", i);
+                        pos++;
+                    } else {
+                        mvwprintw(statusbar_win, 0, pos, "%d,", i);
+                        pos += 2;
+                    }
+                    for (guint limit = 10; i >= limit; limit *= 10) {
+                        pos++;
+                    }
+                    print_act--;
+                }
+            }
+            wattroff(statusbar_win, status_attrs);
+            pos = _status_bar_draw_bracket(FALSE, pos, "]");
         }
     }
 
-    _status_bar_draw_extended_tabs(pos);
-
     wnoutrefresh(statusbar_win);
     inp_put_back();
 }
@@ -397,10 +432,9 @@ _status_bar_draw_tab(StatusBarTab* tab, int pos, int num)
         pos++;
     }
     if (show_name) {
-        char* display_name = _display_name(tab);
+        auto_char char* display_name = _display_name(tab);
         mvwprintw(statusbar_win, 0, pos, "%s", display_name);
         pos += utf8_display_len(display_name);
-        free(display_name);
     }
     wattroff(statusbar_win, status_attrs);
 
@@ -410,7 +444,7 @@ _status_bar_draw_tab(StatusBarTab* tab, int pos, int num)
 }
 
 static int
-_status_bar_draw_bracket(gboolean current, int pos, char* ch)
+_status_bar_draw_bracket(gboolean current, int pos, const char* ch)
 {
     int bracket_attrs = theme_attrs(THEME_STATUS_BRACKET);
     wattron(statusbar_win, bracket_attrs);
@@ -428,9 +462,8 @@ _status_bar_draw_bracket(gboolean current, int pos, char* ch)
 static int
 _status_bar_draw_time(int pos)
 {
-    char* time_pref = prefs_get_string(PREF_TIME_STATUSBAR);
+    auto_gchar gchar* time_pref = prefs_get_string(PREF_TIME_STATUSBAR);
     if (g_strcmp0(time_pref, "off") == 0) {
-        g_free(time_pref);
         return pos;
     }
 
@@ -461,44 +494,80 @@ _status_bar_draw_time(int pos)
     wattroff(statusbar_win, bracket_attrs);
     pos += 2;
 
-    g_free(time_pref);
-
     return pos;
 }
 
-static void
+static gboolean
+_tabmode_is_actlist(void)
+{
+    auto_gchar gchar* tabmode = prefs_get_string(PREF_STATUSBAR_TABMODE);
+    return g_strcmp0(tabmode, "actlist") == 0;
+}
+
+static int
 _status_bar_draw_maintext(int pos)
 {
+    const char* maintext = NULL;
+    auto_jid Jid* jidp = NULL;
+    auto_gchar gchar* self = prefs_get_string(PREF_STATUSBAR_SELF);
+
     if (statusbar->prompt) {
         mvwprintw(statusbar_win, 0, pos, "%s", statusbar->prompt);
-        return;
+        return utf8_display_len(statusbar->prompt);
     }
 
-    gboolean stop = FALSE;
+    if (g_strcmp0(self, "off") == 0) {
+        return pos;
+    }
 
     if (statusbar->fulljid) {
-        char* pref = prefs_get_string(PREF_STATUSBAR_SELF);
+        jidp = jid_create(statusbar->fulljid);
+        if (g_strcmp0(self, "user") == 0) {
+            maintext = jidp->localpart;
+        } else if (g_strcmp0(self, "barejid") == 0) {
+            maintext = jidp->barejid;
+        } else {
+            maintext = statusbar->fulljid;
+        }
+    }
+
+    if (maintext == NULL) {
+        return pos;
+    }
+
+    if (statusbar->fulljid) {
+        auto_gchar gchar* pref = prefs_get_string(PREF_STATUSBAR_SELF);
 
         if (g_strcmp0(pref, "off") == 0) {
-            stop = true;
-        } else if (g_strcmp0(pref, "user") == 0) {
-            Jid* jidp = jid_create(statusbar->fulljid);
+            return pos;
+        }
+        if (g_strcmp0(pref, "user") == 0) {
+            auto_jid Jid* jidp = jid_create(statusbar->fulljid);
             mvwprintw(statusbar_win, 0, pos, "%s", jidp->localpart);
-            jid_destroy(jidp);
-            stop = true;
-        } else if (g_strcmp0(pref, "barejid") == 0) {
-            Jid* jidp = jid_create(statusbar->fulljid);
+            return pos;
+        }
+        if (g_strcmp0(pref, "barejid") == 0) {
+            auto_jid Jid* jidp = jid_create(statusbar->fulljid);
             mvwprintw(statusbar_win, 0, pos, "%s", jidp->barejid);
-            jid_destroy(jidp);
-            stop = true;
+            return pos;
         }
 
-        g_free(pref);
-        if (stop) {
-            return;
-        }
         mvwprintw(statusbar_win, 0, pos, "%s", statusbar->fulljid);
     }
+
+    gboolean actlist_tabmode = _tabmode_is_actlist();
+    auto_gchar gchar* maintext_ = NULL;
+    if (actlist_tabmode) {
+        pos = _status_bar_draw_bracket(FALSE, pos, "[");
+        maintext_ = g_strdup_printf("%d:%s", statusbar->current_tab, maintext);
+        maintext = maintext_;
+    }
+    mvwprintw(statusbar_win, 0, pos, "%s", maintext);
+    pos += utf8_display_len(maintext);
+    if (actlist_tabmode) {
+        pos = _status_bar_draw_bracket(FALSE, pos, "]");
+    }
+    return pos;
 }
 
 static void
@@ -533,10 +602,9 @@ _tabs_width(void)
                 if (!show_read && !is_current && !tab->highlight)
                     continue;
 
-                char* display_name = _display_name(tab);
+                auto_char char* display_name = _display_name(tab);
                 width += utf8_display_len(display_name);
                 width += 4;
-                free(display_name);
             }
         }
         return width;
@@ -552,10 +620,9 @@ _tabs_width(void)
                 if (!show_read && !is_current && !tab->highlight)
                     continue;
 
-                char* display_name = _display_name(tab);
+                auto_char char* display_name = _display_name(tab);
                 width += utf8_display_len(display_name);
                 width += 2;
-                free(display_name);
             }
         }
         return width;
@@ -583,49 +650,43 @@ _display_name(StatusBarTab* tab)
             fullname = strdup(tab->display_name);
         }
     } else if (tab->window_type == WIN_MUC) {
-        char* pref = prefs_get_string(PREF_STATUSBAR_ROOM);
+        auto_gchar gchar* pref = prefs_get_string(PREF_STATUSBAR_ROOM);
         if (g_strcmp0("room", pref) == 0) {
-            Jid* jidp = jid_create(tab->identifier);
+            auto_jid Jid* jidp = jid_create(tab->identifier);
             char* room = strdup(jidp->localpart);
-            jid_destroy(jidp);
             fullname = room;
         } else {
             fullname = strdup(tab->identifier);
         }
-        g_free(pref);
     } else if (tab->window_type == WIN_CONFIG) {
-        char* pref = prefs_get_string(PREF_STATUSBAR_ROOM);
+        auto_gchar gchar* pref = prefs_get_string(PREF_STATUSBAR_ROOM);
         GString* display_str = g_string_new("");
 
         if (g_strcmp0("room", pref) == 0) {
-            Jid* jidp = jid_create(tab->identifier);
+            auto_jid Jid* jidp = jid_create(tab->identifier);
             g_string_append(display_str, jidp->localpart);
-            jid_destroy(jidp);
         } else {
             g_string_append(display_str, tab->identifier);
         }
 
-        g_free(pref);
         g_string_append(display_str, " conf");
         char* result = strdup(display_str->str);
         g_string_free(display_str, TRUE);
         fullname = result;
     } else if (tab->window_type == WIN_PRIVATE) {
-        char* pref = prefs_get_string(PREF_STATUSBAR_ROOM);
+        auto_gchar gchar* pref = prefs_get_string(PREF_STATUSBAR_ROOM);
         if (g_strcmp0("room", pref) == 0) {
             GString* display_str = g_string_new("");
-            Jid* jidp = jid_create(tab->identifier);
+            auto_jid Jid* jidp = jid_create(tab->identifier);
             g_string_append(display_str, jidp->localpart);
             g_string_append(display_str, "/");
             g_string_append(display_str, jidp->resourcepart);
-            jid_destroy(jidp);
             char* result = strdup(display_str->str);
             g_string_free(display_str, TRUE);
             fullname = result;
         } else {
             fullname = strdup(tab->identifier);
         }
-        g_free(pref);
     } else {
         fullname = strdup("window");
     }
@@ -640,10 +701,9 @@ _display_name(StatusBarTab* tab)
         return fullname;
     }
 
-    gchar* trimmed = g_utf8_substring(fullname, 0, tablen);
-    free(fullname);
+    auto_gchar gchar* trimmed = g_utf8_substring(fullname, 0, tablen);
     char* trimmedname = strdup(trimmed);
-    g_free(trimmed);
+    free(fullname);
 
     return trimmedname;
 }
