@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2022 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -195,13 +195,11 @@ _title_bar_draw(void)
         waddch(win, ' ');
     }
 
-    char* title = win_get_title(current);
+    auto_char char* title = win_get_title(current);
 
     mvwprintw(win, 0, 0, " %s", title);
     pos = strlen(title) + 1;
 
-    free(title);
-
     // presence is written from the right side
     // calculate it first so we have a maxposition
     maxrightpos = _calc_self_presence();
