@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2022 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -35,6 +35,7 @@
  */
 
 #include "config.h"
+#include "ui/window_list.h"
 
 #include <stdlib.h>
 #include <string.h>
@@ -61,6 +62,7 @@
 #include "xmpp/xmpp.h"
 #include "xmpp/roster_list.h"
 #include "xmpp/connection.h"
+#include "database.h"
 
 #define CONS_WIN_TITLE "Profanity. Type /help for help information."
 #define XML_WIN_TITLE  "XML Console"
@@ -281,32 +283,58 @@ win_create_plugin(const char* const plugin_name, const char* const tag)
     return &new_win->window;
 }
 
+ProfWin*
+win_create_vcard(vCard* vcard)
+{
+    ProfVcardWin* new_win = malloc(sizeof(ProfVcardWin));
+    new_win->window.type = WIN_VCARD;
+    new_win->window.layout = _win_create_simple_layout();
+
+    new_win->vcard = vcard;
+    new_win->memcheck = PROFVCARDWIN_MEMCHECK;
+
+    return &new_win->window;
+}
+
 char*
 win_get_title(ProfWin* window)
 {
     if (window == NULL) {
         return strdup(CONS_WIN_TITLE);
     }
-    if (window->type == WIN_CONSOLE) {
+
+    switch (window->type) {
+    case WIN_CONSOLE:
+    {
         return strdup(CONS_WIN_TITLE);
     }
-    if (window->type == WIN_CHAT) {
+    case WIN_CHAT:
+    {
         ProfChatWin* chatwin = (ProfChatWin*)window;
         assert(chatwin->memcheck == PROFCHATWIN_MEMCHECK);
+
+        gboolean show_titlebar_jid = prefs_get_boolean(PREF_TITLEBAR_MUC_TITLE_JID);
+        gboolean show_titlebar_name = prefs_get_boolean(PREF_TITLEBAR_MUC_TITLE_NAME);
         jabber_conn_status_t conn_status = connection_get_status();
-        if (conn_status == JABBER_CONNECTED) {
-            PContact contact = roster_get_contact(chatwin->barejid);
-            if (contact) {
-                const char* name = p_contact_name_or_jid(contact);
-                return strdup(name);
-            } else {
-                return strdup(chatwin->barejid);
-            }
-        } else {
+
+        if (conn_status != JABBER_CONNECTED || !show_titlebar_name) {
             return strdup(chatwin->barejid);
         }
+        PContact contact = roster_get_contact(chatwin->barejid);
+        if (!contact) {
+            return strdup(chatwin->barejid);
+        }
+        const char* name = p_contact_name(contact);
+        if (name == NULL) {
+            return strdup(chatwin->barejid);
+        }
+        if (show_titlebar_jid) {
+            return g_strdup_printf("%s <%s>", name, chatwin->barejid);
+        }
+        return g_strdup_printf("%s", name);
     }
-    if (window->type == WIN_MUC) {
+    case WIN_MUC:
+    {
         ProfMucWin* mucwin = (ProfMucWin*)window;
         assert(mucwin->memcheck == PROFMUCWIN_MEMCHECK);
 
@@ -322,37 +350,55 @@ win_get_title(ProfWin* window)
             g_string_append(title, mucwin->roomjid);
         }
 
-        char* title_str = title->str;
-        g_string_free(title, FALSE);
-        return title_str;
+        return g_string_free(title, FALSE);
     }
-    if (window->type == WIN_CONFIG) {
-        ProfConfWin* confwin = (ProfConfWin*)window;
-        assert(confwin->memcheck == PROFCONFWIN_MEMCHECK);
-        GString* title = g_string_new(confwin->roomjid);
-        g_string_append(title, " config");
-        if (confwin->form->modified) {
-            g_string_append(title, " *");
+    case WIN_CONFIG:
+    {
+        if (window->type == WIN_CONFIG) {
+            ProfConfWin* confwin = (ProfConfWin*)window;
+            assert(confwin->memcheck == PROFCONFWIN_MEMCHECK);
+            GString* title = g_string_new(confwin->roomjid);
+            g_string_append(title, " config");
+            if (confwin->form->modified) {
+                g_string_append(title, " *");
+            }
+            return g_string_free(title, FALSE);
         }
-        char* title_str = title->str;
-        g_string_free(title, FALSE);
-        return title_str;
     }
-    if (window->type == WIN_PRIVATE) {
+    case WIN_PRIVATE:
+    {
         ProfPrivateWin* privatewin = (ProfPrivateWin*)window;
         assert(privatewin->memcheck == PROFPRIVATEWIN_MEMCHECK);
         return strdup(privatewin->fulljid);
     }
-    if (window->type == WIN_XML) {
+    case WIN_XML:
+    {
         return strdup(XML_WIN_TITLE);
     }
-    if (window->type == WIN_PLUGIN) {
+    case WIN_PLUGIN:
+    {
         ProfPluginWin* pluginwin = (ProfPluginWin*)window;
         assert(pluginwin->memcheck == PROFPLUGINWIN_MEMCHECK);
         return strdup(pluginwin->tag);
     }
+    case WIN_VCARD:
+    {
+        ProfVcardWin* vcardwin = (ProfVcardWin*)window;
+        assert(vcardwin->memcheck == PROFVCARDWIN_MEMCHECK);
 
-    return NULL;
+        GString* title = g_string_new("vCard ");
+        auto_char char* jid = connection_get_barejid();
+
+        g_string_append(title, jid);
+
+        if (vcardwin->vcard->modified) {
+            g_string_append(title, " *");
+        }
+
+        return g_string_free(title, FALSE);
+    }
+    }
+    assert(FALSE);
 }
 
 char*
@@ -394,9 +440,12 @@ win_get_tab_identifier(ProfWin* window)
     {
         return strdup("xmlconsole");
     }
-    default:
-        return strdup("UNKNOWN");
+    case WIN_VCARD:
+    {
+        return strdup("vcard");
     }
+    }
+    assert(FALSE);
 }
 
 char*
@@ -470,13 +519,15 @@ win_to_string(ProfWin* window)
         ProfPluginWin* pluginwin = (ProfPluginWin*)window;
         GString* gstring = g_string_new("");
         g_string_append_printf(gstring, "Plugin: %s", pluginwin->tag);
-        char* res = gstring->str;
-        g_string_free(gstring, FALSE);
-        return res;
+        return g_string_free(gstring, FALSE);
     }
-    default:
-        return NULL;
+    case WIN_VCARD:
+    {
+        ProfVcardWin* vcardwin = (ProfVcardWin*)window;
+        return vcardwin_get_string(vcardwin);
     }
+    }
+    assert(FALSE);
 }
 
 void
@@ -598,12 +649,21 @@ win_page_up(ProfWin* window)
     int page_space = rows - 4;
     int* page_start = &(window->layout->y_pos);
 
-    // dont need to scroll
-    if (*page_start == 0)
-        return;
-
     *page_start -= page_space;
 
+    if (*page_start == -page_space && window->type == WIN_CHAT) {
+        ProfChatWin* chatwin = (ProfChatWin*)window;
+        ProfBuffEntry* first_entry = buffer_size(window->layout->buffer) != 0 ? buffer_get_entry(window->layout->buffer, 0) : NULL;
+
+        // Don't do anything if still fetching mam messages
+        if (first_entry && !(first_entry->theme_item == THEME_ROOMINFO && g_strcmp0(first_entry->message, LOADING_MESSAGE) == 0)) {
+            if (!chatwin_db_history(chatwin, NULL, NULL, TRUE) && prefs_get_boolean(PREF_MAM)) {
+                win_print_loading_history(window);
+                iq_mam_request_older(chatwin);
+            }
+        }
+    }
+
     // went past beginning, show first page
     if (*page_start < 0)
         *page_start = 0;
@@ -627,6 +687,20 @@ win_page_down(ProfWin* window)
 
     *page_start += page_space;
 
+    // Scrolled down after reaching the bottom of the page
+    if ((*page_start == y || (*page_start == page_space && *page_start >= y)) && window->type == WIN_CHAT) {
+        int bf_size = buffer_size(window->layout->buffer);
+        if (bf_size > 0) {
+            auto_gchar gchar* start = g_date_time_format_iso8601(buffer_get_entry(window->layout->buffer, bf_size - 1)->time);
+            GDateTime* now = g_date_time_new_now_local();
+            gchar* end = g_date_time_format_iso8601(now);
+            // end is free'd inside
+            chatwin_db_history((ProfChatWin*)window, start, end, FALSE);
+
+            g_date_time_unref(now);
+        }
+    }
+
     // only got half a screen, show full screen
     if ((y - (*page_start)) < page_space)
         *page_start = y - page_space;
@@ -902,9 +976,8 @@ win_show_occupant_info(ProfWin* window, const char* const room, Occupant* occupa
     win_println(window, THEME_DEFAULT, "!", "  Affiliation: %s", occupant_affiliation);
     win_println(window, THEME_DEFAULT, "!", "  Role: %s", occupant_role);
 
-    Jid* jidp = jid_create_from_bare_and_resource(room, occupant->nick);
+    auto_jid Jid* jidp = jid_create_from_bare_and_resource(room, occupant->nick);
     EntityCapabilities* caps = caps_lookup(jidp->fulljid);
-    jid_destroy(jidp);
 
     if (caps) {
         // show identity
@@ -1025,9 +1098,8 @@ win_show_info(ProfWin* window, PContact contact)
         }
         win_newline(window);
 
-        Jid* jidp = jid_create_from_bare_and_resource(barejid, resource->name);
+        auto_jid Jid* jidp = jid_create_from_bare_and_resource(barejid, resource->name);
         EntityCapabilities* caps = caps_lookup(jidp->fulljid);
-        jid_destroy(jidp);
 
         if (caps) {
             // show identity
@@ -1082,6 +1154,160 @@ win_show_info(ProfWin* window, PContact contact)
     g_list_free(ordered_resources);
 }
 
+void
+win_show_vcard(ProfWin* window, vCard* vcard)
+{
+    GList* pointer;
+    int index = 0;
+
+    if (vcard->fullname) {
+        win_println(window, THEME_DEFAULT, "!", "Full name: %s", vcard->fullname);
+    }
+    if (vcard->name.family || vcard->name.given || vcard->name.middle || vcard->name.prefix || vcard->name.suffix) {
+        win_println(window, THEME_DEFAULT, "!", "Name: ");
+        if (vcard->name.family) {
+            win_println(window, THEME_DEFAULT, "!", "    Family: %s", vcard->name.family);
+        }
+        if (vcard->name.given) {
+            win_println(window, THEME_DEFAULT, "!", "    Given: %s", vcard->name.given);
+        }
+        if (vcard->name.middle) {
+            win_println(window, THEME_DEFAULT, "!", "    Middle: %s", vcard->name.middle);
+        }
+        if (vcard->name.prefix) {
+            win_println(window, THEME_DEFAULT, "!", "    Prefix: %s", vcard->name.prefix);
+        }
+        if (vcard->name.suffix) {
+            win_println(window, THEME_DEFAULT, "!", "    Suffix: %s", vcard->name.suffix);
+        }
+    }
+    index = 0;
+    for (pointer = g_queue_peek_head_link(vcard->elements); pointer != NULL; pointer = pointer->next, index++) {
+        assert(pointer->data != NULL);
+        vcard_element_t* element = (vcard_element_t*)pointer->data;
+
+        switch (element->type) {
+        case VCARD_NICKNAME:
+        {
+            win_println(window, THEME_DEFAULT, "!", "[%d] Nickname: %s", index, element->nickname);
+            break;
+        }
+        case VCARD_PHOTO:
+        {
+            if (element->photo.external) {
+                win_println(window, THEME_DEFAULT, "!", "[%d] Photo, External value: %s", index, element->photo.extval);
+            } else {
+                win_println(window, THEME_DEFAULT, "!", "[%d] Photo (%s), size: %zu", index, element->photo.type, element->photo.length);
+            }
+            break;
+        }
+        case VCARD_BIRTHDAY:
+        {
+            auto_gchar gchar* date_format = prefs_get_string(PREF_TIME_VCARD);
+            auto_gchar gchar* date = g_date_time_format(element->birthday, date_format);
+
+            assert(date != NULL);
+            win_println(window, THEME_DEFAULT, "!", "[%d] Birthday: %s", index, date);
+            break;
+        }
+        case VCARD_ADDRESS:
+        {
+            // Print the header with flags
+            win_println(window, THEME_DEFAULT, "!", "[%d] Address%s%s%s%s%s%s%s", index,
+                        (element->address.options & VCARD_HOME) ? " [home]" : "",
+                        (element->address.options & VCARD_WORK) == VCARD_WORK ? " [work]" : "",
+                        (element->address.options & VCARD_POSTAL) == VCARD_POSTAL ? " [postal]" : "",
+                        (element->address.options & VCARD_PARCEL) == VCARD_PARCEL ? " [parcel]" : "",
+                        (element->address.options & VCARD_INTL) == VCARD_INTL ? " [international]" : "",
+                        (element->address.options & VCARD_DOM) == VCARD_DOM ? " [domestic]" : "",
+                        (element->address.options & VCARD_PREF) == VCARD_PREF ? " [preferred]" : "");
+
+            if (element->address.pobox) {
+                win_println(window, THEME_DEFAULT, "!", "    P.O. Box: %s", element->address.pobox);
+            }
+            if (element->address.extaddr) {
+                win_println(window, THEME_DEFAULT, "!", "    Extended address: %s", element->address.extaddr);
+            }
+            if (element->address.street) {
+                win_println(window, THEME_DEFAULT, "!", "    Street: %s", element->address.street);
+            }
+            if (element->address.locality) {
+                win_println(window, THEME_DEFAULT, "!", "    Locality: %s", element->address.locality);
+            }
+            if (element->address.region) {
+                win_println(window, THEME_DEFAULT, "!", "    Region: %s", element->address.region);
+            }
+            if (element->address.pcode) {
+                win_println(window, THEME_DEFAULT, "!", "    Postal code: %s", element->address.pcode);
+            }
+            if (element->address.country) {
+                win_println(window, THEME_DEFAULT, "!", "    Country: %s", element->address.country);
+            }
+            break;
+        }
+        case VCARD_TELEPHONE:
+        {
+            // Print the header with flags
+            win_println(window, THEME_DEFAULT, "!", "[%d] Telephone%s%s%s%s%s%s%s%s%s%s%s%s%s", index,
+                        (element->telephone.options & VCARD_HOME) ? " [home]" : "",
+                        (element->telephone.options & VCARD_WORK) == VCARD_WORK ? " [work]" : "",
+                        (element->telephone.options & VCARD_TEL_VOICE) == VCARD_TEL_VOICE ? " [voice]" : "",
+                        (element->telephone.options & VCARD_TEL_FAX) == VCARD_TEL_FAX ? " [fax]" : "",
+                        (element->telephone.options & VCARD_TEL_PAGER) == VCARD_TEL_PAGER ? " [pager]" : "",
+                        (element->telephone.options & VCARD_TEL_MSG) == VCARD_TEL_MSG ? " [msg]" : "",
+                        (element->telephone.options & VCARD_TEL_CELL) == VCARD_TEL_CELL ? " [cell]" : "",
+                        (element->telephone.options & VCARD_TEL_VIDEO) == VCARD_TEL_VIDEO ? " [video]" : "",
+                        (element->telephone.options & VCARD_TEL_BBS) == VCARD_TEL_BBS ? " [bbs]" : "",
+                        (element->telephone.options & VCARD_TEL_MODEM) == VCARD_TEL_MODEM ? " [modem]" : "",
+                        (element->telephone.options & VCARD_TEL_ISDN) == VCARD_TEL_ISDN ? " [isdn]" : "",
+                        (element->telephone.options & VCARD_TEL_PCS) == VCARD_TEL_PCS ? " [pcs]" : "",
+                        (element->telephone.options & VCARD_PREF) == VCARD_PREF ? " [preferred]" : "");
+            if (element->telephone.number) {
+                win_println(window, THEME_DEFAULT, "!", "    Number: %s", element->telephone.number);
+            }
+            break;
+        }
+        case VCARD_EMAIL:
+        {
+            // Print the header with flags
+            win_println(window, THEME_DEFAULT, "!", "[%d] E-mail%s%s%s%s%s", index,
+                        (element->email.options & VCARD_HOME) ? " [home]" : "",
+                        (element->email.options & VCARD_WORK) == VCARD_WORK ? " [work]" : "",
+                        (element->email.options & VCARD_EMAIL_X400) == VCARD_EMAIL_X400 ? " [x400]" : "",
+                        (element->email.options & VCARD_EMAIL_INTERNET) == VCARD_EMAIL_INTERNET ? " [internet]" : "",
+                        (element->email.options & VCARD_PREF) == VCARD_PREF ? " [preferred]" : "");
+            if (element->email.userid) {
+                win_println(window, THEME_DEFAULT, "!", "    ID: %s", element->email.userid);
+            }
+            break;
+        }
+        case VCARD_JID:
+        {
+            win_println(window, THEME_DEFAULT, "!", "[%d] Jabber ID: %s", index, element->jid);
+            break;
+        }
+        case VCARD_TITLE:
+        {
+            win_println(window, THEME_DEFAULT, "!", "[%d] Title: %s", index, element->title);
+            break;
+        }
+        case VCARD_ROLE:
+        {
+            win_println(window, THEME_DEFAULT, "!", "[%d] Role: %s", index, element->role);
+            break;
+        }
+        case VCARD_NOTE:
+        {
+            win_println(window, THEME_DEFAULT, "!", "[%d] Note: %s", index, element->note);
+            break;
+        }
+        case VCARD_URL:
+            win_println(window, THEME_DEFAULT, "!", "[%d] URL: %s", index, element->url);
+            break;
+        }
+    }
+}
+
 void
 win_show_status_string(ProfWin* window, const char* const from,
                        const char* const show, const char* const status,
@@ -1106,15 +1332,12 @@ win_show_status_string(ProfWin* window, const char* const from,
         win_append(window, presence_colour, " is %s", default_show);
 
     if (last_activity) {
-        gchar* date_fmt = NULL;
-        char* time_pref = prefs_get_string(PREF_TIME_LASTACTIVITY);
+        auto_gchar gchar* date_fmt = NULL;
+        auto_gchar gchar* time_pref = prefs_get_string(PREF_TIME_LASTACTIVITY);
         date_fmt = g_date_time_format(last_activity, time_pref);
-        g_free(time_pref);
         assert(date_fmt != NULL);
 
         win_append(window, presence_colour, ", last activity: %s", date_fmt);
-
-        g_free(date_fmt);
     }
 
     if (status)
@@ -1177,7 +1400,7 @@ win_print_incoming(ProfWin* window, const char* const display_name_from, ProfMes
     switch (window->type) {
     case WIN_CHAT:
     {
-        char* enc_char;
+        auto_char char* enc_char;
         ProfChatWin* chatwin = (ProfChatWin*)window;
 
         if (chatwin->incoming_char) {
@@ -1197,13 +1420,11 @@ win_print_incoming(ProfWin* window, const char* const display_name_from, ProfMes
         if (prefs_get_boolean(PREF_CORRECTION_ALLOW) && message->replace_id) {
             _win_correct(window, message->plain, message->id, message->replace_id, message->from_jid->barejid);
         } else {
-            // Prevent duplicate messages when current client is sending a message
-            if (g_strcmp0(message->from_jid->fulljid, connection_get_fulljid()) != 0) {
+            // Prevent duplicate messages when current client is sending a message or if it's mam
+            if (g_strcmp0(message->from_jid->fulljid, connection_get_fulljid()) != 0 && !message->is_mam) {
                 _win_printf(window, enc_char, 0, message->timestamp, flags, THEME_TEXT_THEM, display_name_from, message->from_jid->barejid, message->id, "%s", message->plain);
             }
         }
-
-        free(enc_char);
         break;
     }
     case WIN_PRIVATE:
@@ -1257,7 +1478,7 @@ win_print_outgoing(ProfWin* window, const char* show_char, const char* const id,
     if (replace_id) {
         _win_correct(window, message, id, replace_id, myjid);
     } else {
-        char* outgoing_str = prefs_get_string(PREF_OUTGOING_STAMP);
+        auto_gchar gchar* outgoing_str = prefs_get_string(PREF_OUTGOING_STAMP);
         _win_printf(window, show_char, 0, timestamp, 0, THEME_TEXT_ME, outgoing_str, myjid, id, "%s", message);
     }
 
@@ -1270,10 +1491,10 @@ win_print_history(ProfWin* window, const ProfMessage* const message)
 {
     g_date_time_ref(message->timestamp);
 
-    char* display_name;
+    auto_gchar gchar* display_name;
     int flags = 0;
     const char* jid = connection_get_fulljid();
-    Jid* jidp = jid_create(jid);
+    auto_jid Jid* jidp = jid_create(jid);
 
     if (g_strcmp0(jidp->barejid, message->from_jid->barejid) == 0) {
         display_name = strdup("me");
@@ -1282,12 +1503,36 @@ win_print_history(ProfWin* window, const ProfMessage* const message)
         flags = NO_ME;
     }
 
-    jid_destroy(jidp);
-
     buffer_append(window->layout->buffer, "-", 0, message->timestamp, flags, THEME_TEXT_HISTORY, display_name, NULL, message->plain, NULL, NULL);
+    wins_add_urls_ac(window, message, FALSE);
+    wins_add_quotes_ac(window, message->plain, FALSE);
     _win_print_internal(window, "-", 0, message->timestamp, flags, THEME_TEXT_HISTORY, display_name, message->plain, NULL);
 
-    free(display_name);
+    inp_nonblocking(TRUE);
+    g_date_time_unref(message->timestamp);
+}
+
+void
+win_print_old_history(ProfWin* window, const ProfMessage* const message)
+{
+    g_date_time_ref(message->timestamp);
+
+    auto_char char* display_name;
+    int flags = 0;
+    const char* jid = connection_get_fulljid();
+    auto_jid Jid* jidp = jid_create(jid);
+
+    if (g_strcmp0(jidp->barejid, message->from_jid->barejid) == 0) {
+        display_name = strdup("me");
+    } else {
+        display_name = roster_get_msg_display_name(message->from_jid->barejid, message->from_jid->resourcepart);
+        flags = NO_ME;
+    }
+
+    buffer_prepend(window->layout->buffer, "-", 0, message->timestamp, flags, THEME_TEXT_HISTORY, display_name, NULL, message->plain, NULL, NULL);
+    wins_add_urls_ac(window, message, TRUE);
+    wins_add_quotes_ac(window, message->plain, TRUE);
+    _win_print_internal(window, "-", 0, message->timestamp, flags, THEME_TEXT_HISTORY, display_name, message->plain, NULL);
 
     inp_nonblocking(TRUE);
     g_date_time_unref(message->timestamp);
@@ -1434,9 +1679,9 @@ win_appendln_highlight(ProfWin* window, theme_item_t theme_item, const char* con
 }
 
 void
-win_print_http_transfer(ProfWin* window, const char* const message, char* url)
+win_print_http_transfer(ProfWin* window, const char* const message, char* id)
 {
-    win_print_outgoing_with_receipt(window, "!", NULL, message, url, NULL);
+    win_print_outgoing_with_receipt(window, "!", NULL, message, id, NULL);
 }
 
 void
@@ -1464,6 +1709,8 @@ win_print_outgoing_with_receipt(ProfWin* window, const char* show_char, const ch
 void
 win_mark_received(ProfWin* window, const char* const id)
 {
+    if (window->type == WIN_CONSOLE)
+        return;
     gboolean received = buffer_mark_received(window->layout->buffer, id);
     if (received) {
         win_redraw(window);
@@ -1473,6 +1720,8 @@ win_mark_received(ProfWin* window, const char* const id)
 void
 win_update_entry_message(ProfWin* window, const char* const id, const char* const message)
 {
+    if (window->type == WIN_CONSOLE)
+        return;
     ProfBuffEntry* entry = buffer_get_entry_by_id(window->layout->buffer, id);
     if (entry) {
         free(entry->message);
@@ -1509,7 +1758,6 @@ _win_printf(ProfWin* window, const char* show_char, int pad_indent, GDateTime* t
     g_string_vprintf(fmt_msg, message, arg);
 
     buffer_append(window->layout->buffer, show_char, pad_indent, timestamp, flags, theme_item, display_from, from_jid, fmt_msg->str, NULL, message_id);
-
     _win_print_internal(window, show_char, pad_indent, timestamp, flags, theme_item, display_from, fmt_msg->str, NULL);
 
     inp_nonblocking(TRUE);
@@ -1534,7 +1782,7 @@ _win_print_internal(ProfWin* window, const char* show_char, int pad_indent, GDat
     int colour = theme_attrs(THEME_ME);
     size_t indent = 0;
 
-    char* time_pref = NULL;
+    auto_gchar gchar* time_pref = NULL;
     switch (window->type) {
     case WIN_CHAT:
         time_pref = prefs_get_string(PREF_TIME_CHAT);
@@ -1556,13 +1804,12 @@ _win_print_internal(ProfWin* window, const char* show_char, int pad_indent, GDat
         break;
     }
 
-    gchar* date_fmt = NULL;
+    auto_gchar gchar* date_fmt = NULL;
     if (g_strcmp0(time_pref, "off") == 0 || time == NULL) {
         date_fmt = g_strdup("");
     } else {
         date_fmt = g_date_time_format(time, time_pref);
     }
-    g_free(time_pref);
     assert(date_fmt != NULL);
 
     if (strlen(date_fmt) != 0) {
@@ -1587,13 +1834,12 @@ _win_print_internal(ProfWin* window, const char* show_char, int pad_indent, GDat
             colour = theme_attrs(THEME_THEM);
         }
 
-        char* color_pref = prefs_get_string(PREF_COLOR_NICK);
+        auto_gchar gchar* color_pref = prefs_get_string(PREF_COLOR_NICK);
         if (color_pref != NULL && (strcmp(color_pref, "false") != 0)) {
             if ((flags & NO_ME) || (!(flags & NO_ME) && prefs_get_boolean(PREF_COLOR_NICK_OWN))) {
                 colour = theme_hash_attrs(from);
             }
         }
-        g_free(color_pref);
 
         if (flags & NO_COLOUR_FROM) {
             colour = 0;
@@ -1650,8 +1896,6 @@ _win_print_internal(ProfWin* window, const char* show_char, int pad_indent, GDat
             wattroff(window->layout->win, theme_attrs(theme_item));
         }
     }
-
-    g_free(date_fmt);
 }
 
 static void
@@ -1667,7 +1911,7 @@ _win_print_wrapped(WINDOW* win, const char* const message, size_t indent, int pa
 {
     int starty = getcury(win);
     int wordi = 0;
-    char* word = malloc(strlen(message) + 1);
+    auto_char char* word = malloc(strlen(message) + 1);
 
     gchar* curr_ch = g_utf8_offset_to_pointer(message, 0);
 
@@ -1774,8 +2018,6 @@ _win_print_wrapped(WINDOW* win, const char* const message, size_t indent, int pa
             curr_ch = g_utf8_next_char(curr_ch);
         }
     }
-
-    free(word);
 }
 
 void
@@ -1813,6 +2055,26 @@ win_redraw(ProfWin* window)
     }
 }
 
+void
+win_print_loading_history(ProfWin* window)
+{
+    GDateTime* timestamp;
+    gboolean is_buffer_empty = buffer_size(window->layout->buffer) == 0;
+
+    if (!is_buffer_empty) {
+        timestamp = buffer_get_entry(window->layout->buffer, 0)->time;
+    } else {
+        timestamp = g_date_time_new_now_local();
+    }
+
+    buffer_prepend(window->layout->buffer, "-", 0, timestamp, NO_DATE, THEME_ROOMINFO, NULL, NULL, LOADING_MESSAGE, NULL, NULL);
+
+    if (is_buffer_empty)
+        g_date_time_unref(timestamp);
+
+    win_redraw(window);
+}
+
 gboolean
 win_has_active_subwin(ProfWin* window)
 {
@@ -2007,8 +2269,7 @@ win_handle_command_exec_result_note(ProfWin* window, const char* const type, con
 void
 win_insert_last_read_position_marker(ProfWin* window, char* id)
 {
-    int size;
-    size = buffer_size(window->layout->buffer);
+    int size = buffer_size(window->layout->buffer);
 
     // TODO: this is somewhat costly. We should improve this later.
     // check if we already have a separator present
@@ -2039,23 +2300,17 @@ win_quote_autocomplete(ProfWin* window, const char* const input, gboolean previo
         return NULL;
     }
 
-    char* result = autocomplete_complete(window->quotes_ac, input + 1, FALSE, previous);
+    auto_gchar gchar* result = autocomplete_complete(window->quotes_ac, input + 1, FALSE, previous);
     if (result == NULL) {
         return NULL;
     }
 
-    gchar** parts = g_strsplit(result, "\n", -1);
-    gchar* quoted_result = g_strjoinv("\n> ", parts);
+    auto_gcharv gchar** parts = g_strsplit(result, "\n", -1);
+    auto_gchar gchar* quoted_result = g_strjoinv("\n> ", parts);
 
     GString* replace_with = g_string_new("> ");
     g_string_append(replace_with, quoted_result);
     g_string_append(replace_with, "\n");
 
-    g_free(result);
-    g_free(quoted_result);
-    g_strfreev(parts);
-
-    result = replace_with->str;
-    g_string_free(replace_with, FALSE);
-    return result;
+    return g_string_free(replace_with, FALSE);
 }
