@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2022 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -276,13 +276,13 @@ wins_get_private_chats(const char* const roomjid)
 void
 wins_private_nick_change(const char* const roomjid, const char* const oldnick, const char* const newnick)
 {
-    Jid* oldjid = jid_create_from_bare_and_resource(roomjid, oldnick);
+    auto_jid Jid* oldjid = jid_create_from_bare_and_resource(roomjid, oldnick);
 
     ProfPrivateWin* privwin = wins_get_private(oldjid->fulljid);
     if (privwin) {
         free(privwin->fulljid);
 
-        Jid* newjid = jid_create_from_bare_and_resource(roomjid, newnick);
+        auto_jid Jid* newjid = jid_create_from_bare_and_resource(roomjid, newnick);
         privwin->fulljid = strdup(newjid->fulljid);
         win_println((ProfWin*)privwin, THEME_THEM, "!", "** %s is now known as %s.", oldjid->resourcepart, newjid->resourcepart);
 
@@ -290,11 +290,7 @@ wins_private_nick_change(const char* const roomjid, const char* const oldnick, c
         autocomplete_remove(wins_close_ac, oldjid->fulljid);
         autocomplete_add(wins_ac, newjid->fulljid);
         autocomplete_add(wins_close_ac, newjid->fulljid);
-
-        jid_destroy(newjid);
     }
-
-    jid_destroy(oldjid);
 }
 
 void
@@ -711,6 +707,18 @@ wins_new_plugin(const char* const plugin_name, const char* const tag)
     return newwin;
 }
 
+ProfWin*
+wins_new_vcard(vCard* vcard)
+{
+    GList* keys = g_hash_table_get_keys(windows);
+    int result = _wins_get_next_available_num(keys);
+    g_list_free(keys);
+    ProfWin* newwin = win_create_vcard(vcard);
+    g_hash_table_insert(windows, GINT_TO_POINTER(result), newwin);
+
+    return newwin;
+}
+
 gboolean
 wins_do_notify_remind(void)
 {
@@ -805,6 +813,27 @@ wins_get_xmlconsole(void)
     return NULL;
 }
 
+ProfVcardWin*
+wins_get_vcard(void)
+{
+    GList* values = g_hash_table_get_values(windows);
+    GList* curr = values;
+
+    while (curr) {
+        ProfWin* window = curr->data;
+        if (window->type == WIN_VCARD) {
+            ProfVcardWin* vcardwin = (ProfVcardWin*)window;
+            assert(vcardwin->memcheck == PROFVCARDWIN_MEMCHECK);
+            g_list_free(values);
+            return vcardwin;
+        }
+        curr = g_list_next(curr);
+    }
+
+    g_list_free(values);
+    return NULL;
+}
+
 GSList*
 wins_get_chat_recipients(void)
 {
@@ -916,13 +945,12 @@ wins_swap(int source_win, int target_win)
             g_hash_table_steal(windows, GINT_TO_POINTER(source_win));
             g_hash_table_insert(windows, GINT_TO_POINTER(target_win), source);
             status_bar_inactive(source_win);
-            char* identifier = win_get_tab_identifier(source);
+            auto_char char* identifier = win_get_tab_identifier(source);
             if (win_unread(source) > 0) {
                 status_bar_new(target_win, source->type, identifier);
             } else {
                 status_bar_active(target_win, source->type, identifier);
             }
-            free(identifier);
             if (wins_get_current_num() == source_win) {
                 wins_set_current_by_num(target_win);
                 ui_focus_win(console);
@@ -934,8 +962,8 @@ wins_swap(int source_win, int target_win)
             g_hash_table_steal(windows, GINT_TO_POINTER(target_win));
             g_hash_table_insert(windows, GINT_TO_POINTER(source_win), target);
             g_hash_table_insert(windows, GINT_TO_POINTER(target_win), source);
-            char* source_identifier = win_get_tab_identifier(source);
-            char* target_identifier = win_get_tab_identifier(target);
+            auto_char char* source_identifier = win_get_tab_identifier(source);
+            auto_char char* target_identifier = win_get_tab_identifier(target);
             if (win_unread(source) > 0) {
                 status_bar_new(target_win, source->type, source_identifier);
             } else {
@@ -946,8 +974,6 @@ wins_swap(int source_win, int target_win)
             } else {
                 status_bar_active(source_win, target->type, target_identifier);
             }
-            free(source_identifier);
-            free(target_identifier);
             if ((wins_get_current_num() == source_win) || (wins_get_current_num() == target_win)) {
                 ui_focus_win(console);
             }
@@ -1055,7 +1081,7 @@ wins_tidy(void)
         GList* curr = keys;
         while (curr) {
             ProfWin* window = g_hash_table_lookup(windows, curr->data);
-            char* identifier = win_get_tab_identifier(window);
+            auto_char char* identifier = win_get_tab_identifier(window);
             g_hash_table_steal(windows, curr->data);
             if (num == 10) {
                 g_hash_table_insert(new_windows, GINT_TO_POINTER(0), window);
@@ -1072,7 +1098,6 @@ wins_tidy(void)
                     status_bar_active(num, window->type, identifier);
                 }
             }
-            free(identifier);
             num++;
             curr = g_list_next(curr);
         }
@@ -1109,14 +1134,13 @@ wins_create_summary(gboolean unread)
             GString* line = g_string_new("");
 
             int ui_index = GPOINTER_TO_INT(curr->data);
-            char* winstring = win_to_string(window);
+            auto_char char* winstring = win_to_string(window);
             if (!winstring) {
                 g_string_free(line, TRUE);
                 continue;
             }
 
             g_string_append_printf(line, "%d: %s", ui_index, winstring);
-            free(winstring);
 
             result = g_slist_append(result, strdup(line->str));
             g_string_free(line, TRUE);
@@ -1155,14 +1179,13 @@ wins_create_summary_attention()
             GString* line = g_string_new("");
 
             int ui_index = GPOINTER_TO_INT(curr->data);
-            char* winstring = win_to_string(window);
+            auto_char char* winstring = win_to_string(window);
             if (!winstring) {
                 g_string_free(line, TRUE);
                 continue;
             }
 
             g_string_append_printf(line, "%d: %s", ui_index, winstring);
-            free(winstring);
 
             result = g_slist_append(result, strdup(line->str));
             g_string_free(line, TRUE);
@@ -1282,7 +1305,7 @@ wins_get_next_attention(void)
 }
 
 void
-wins_add_urls_ac(const ProfWin* const win, const ProfMessage* const message)
+wins_add_urls_ac(const ProfWin* const win, const ProfMessage* const message, const gboolean flip)
 {
     GRegex* regex;
     GMatchInfo* match_info;
@@ -1291,13 +1314,16 @@ wins_add_urls_ac(const ProfWin* const win, const ProfMessage* const message)
     g_regex_match(regex, message->plain, 0, &match_info);
 
     while (g_match_info_matches(match_info)) {
-        gchar* word = g_match_info_fetch(match_info, 0);
+        auto_gchar gchar* word = g_match_info_fetch(match_info, 0);
 
-        autocomplete_add_reverse(win->urls_ac, word);
+        if (flip) {
+            autocomplete_add_unsorted(win->urls_ac, word, FALSE);
+        } else {
+            autocomplete_add_unsorted(win->urls_ac, word, TRUE);
+        }
         // for people who run profanity a long time, we don't want to waste a lot of memory
         autocomplete_remove_older_than_max_reverse(win->urls_ac, 20);
 
-        g_free(word);
         g_match_info_next(match_info, NULL);
     }
 
@@ -1306,9 +1332,14 @@ wins_add_urls_ac(const ProfWin* const win, const ProfMessage* const message)
 }
 
 void
-wins_add_quotes_ac(const ProfWin* const win, const char* const message)
+wins_add_quotes_ac(const ProfWin* const win, const char* const message, const gboolean flip)
 {
-    autocomplete_add_reverse(win->quotes_ac, message);
+    if (flip) {
+        autocomplete_add_unsorted(win->quotes_ac, message, FALSE);
+    } else {
+        autocomplete_add_unsorted(win->quotes_ac, message, TRUE);
+    }
+
     // for people who run profanity a long time, we don't want to waste a lot of memory
     autocomplete_remove_older_than_max_reverse(win->quotes_ac, 20);
 }
