@@ -110,9 +110,8 @@ chat_session_get_jid(const char* const barejid)
     ChatSession* session = chat_session_get(barejid);
     char* jid = NULL;
     if (session) {
-        Jid* jidp = jid_create_from_bare_and_resource(session->barejid, session->resource);
+        auto_jid Jid* jidp = jid_create_from_bare_and_resource(session->barejid, session->resource);
         jid = strdup(jidp->fulljid);
-        jid_destroy(jidp);
     } else {
         jid = strdup(barejid);
     }
@@ -120,11 +119,11 @@ chat_session_get_jid(const char* const barejid)
     return jid;
 }
 
-char*
+const char*
 chat_session_get_state(const char* const barejid)
 {
     ChatSession* session = chat_session_get(barejid);
-    char* state = NULL;
+    const char* state = NULL;
     if (session) {
         if (prefs_get_boolean(PREF_STATES) && session->send_states) {
             state = STANZA_NAME_ACTIVE;
