@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2018 - 2022 Michael Vetter <jubalh@idoru.org>
+ * Copyright (C) 2018 - 2023 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -140,6 +140,14 @@ connection_init(void)
     conn.requested_features = g_hash_table_new_full(g_str_hash, g_str_equal, free, NULL);
 
     conn.xmpp_ctx = xmpp_ctx_new(&prof_mem, &prof_log);
+    auto_gchar gchar* v = prefs_get_string(PREF_STROPHE_VERBOSITY);
+    auto_gchar gchar* err_msg = NULL;
+    int verbosity;
+    if (string_to_verbosity(v, &verbosity, &err_msg)) {
+        xmpp_ctx_set_verbosity(conn.xmpp_ctx, verbosity);
+    } else {
+        cons_show(err_msg);
+    }
     conn.xmpp_conn = xmpp_conn_new(conn.xmpp_ctx);
 
     _random_bytes_init();
@@ -173,7 +181,7 @@ connection_shutdown(void)
 static gboolean
 _conn_apply_settings(const char* const jid, const char* const passwd, const char* const tls_policy, const char* const auth_policy)
 {
-    Jid* jidp = jid_create(jid);
+    auto_jid Jid* jidp = jid_create(jid);
     if (jidp == NULL) {
         log_error("Malformed JID not able to connect: %s", jid);
         conn.conn_status = JABBER_DISCONNECTED;
@@ -181,7 +189,6 @@ _conn_apply_settings(const char* const jid, const char* const passwd, const char
     }
 
     _compute_identifier(jidp->barejid);
-    jid_destroy(jidp);
 
     xmpp_ctx_set_verbosity(conn.xmpp_ctx, 0);
     xmpp_conn_set_jid(conn.xmpp_conn, jid);
@@ -190,6 +197,10 @@ _conn_apply_settings(const char* const jid, const char* const passwd, const char
 
     long flags = xmpp_conn_get_flags(conn.xmpp_conn);
 
+    /* clear all TLS & auth related flags */
+    flags &= ~(XMPP_CONN_FLAG_DISABLE_TLS | XMPP_CONN_FLAG_MANDATORY_TLS
+               | XMPP_CONN_FLAG_LEGACY_SSL | XMPP_CONN_FLAG_TRUST_TLS
+               | XMPP_CONN_FLAG_LEGACY_AUTH);
     if (!tls_policy || (g_strcmp0(tls_policy, "force") == 0)) {
         flags |= XMPP_CONN_FLAG_MANDATORY_TLS;
     } else if (g_strcmp0(tls_policy, "trust") == 0) {
@@ -205,8 +216,6 @@ _conn_apply_settings(const char* const jid, const char* const passwd, const char
         flags |= XMPP_CONN_FLAG_LEGACY_AUTH;
     }
 
-    xmpp_conn_set_flags(conn.xmpp_conn, flags);
-
     /* Print debug logs that can help when users share the logs */
     if (flags != 0) {
         log_debug("Connecting with flags (0x%lx):", flags);
@@ -222,10 +231,15 @@ _conn_apply_settings(const char* const jid, const char* const passwd, const char
 #undef LOG_FLAG_IF_SET
     }
 
-    char* cert_path = prefs_get_tls_certpath();
+    if (xmpp_conn_set_flags(conn.xmpp_conn, flags)) {
+        log_error("libstrophe doesn't accept this combination of flags: 0x%x", flags);
+        conn.conn_status = JABBER_DISCONNECTED;
+        return FALSE;
+    }
+
+    auto_char char* cert_path = prefs_get_tls_certpath();
     if (cert_path) {
         xmpp_conn_set_capath(conn.xmpp_conn, cert_path);
-        free(cert_path);
     }
 
     xmpp_conn_set_certfail_handler(conn.xmpp_conn, _connection_certfail_cb);
@@ -398,7 +412,7 @@ _register_handle_features(xmpp_conn_t* xmpp_conn, xmpp_stanza_t* stanza, void* u
     /* secure connection if possible */
     child = xmpp_stanza_get_child_by_name(stanza, "starttls");
     if (child && (strcmp(xmpp_stanza_get_ns(child), XMPP_NS_TLS) == 0)) {
-        log_debug("Server supports TLS. Attempting to establish...");
+        log_debug("Server supports TLS. Attempting to establish…");
         child = xmpp_stanza_new(ctx);
         xmpp_stanza_set_name(child, "starttls");
         xmpp_stanza_set_ns(child, XMPP_NS_TLS);
@@ -718,9 +732,8 @@ connection_get_barejid(void)
     if (!jid)
         return NULL;
 
-    Jid* jidp = jid_create(jid);
+    auto_jid Jid* jidp = jid_create(jid);
     char* result = strdup(jidp->barejid);
-    jid_destroy(jidp);
 
     return result;
 }
@@ -760,6 +773,12 @@ connection_get_available_resources(void)
     return g_hash_table_get_values(conn.available_resources);
 }
 
+int
+connection_count_available_resources(void)
+{
+    return g_hash_table_size(conn.available_resources);
+}
+
 void
 connection_add_available_resource(Resource* resource)
 {
@@ -789,18 +808,15 @@ connection_free_uuid(char* uuid)
 char*
 connection_create_stanza_id(void)
 {
-    char* rndid = get_random_string(CON_RAND_ID_LEN);
+    auto_char char* rndid = get_random_string(CON_RAND_ID_LEN);
     assert(rndid != NULL);
 
-    gchar* hmac = g_compute_hmac_for_string(G_CHECKSUM_SHA1,
-                                            (guchar*)prof_identifier, strlen(prof_identifier),
-                                            rndid, strlen(rndid));
+    auto_gchar gchar* hmac = g_compute_hmac_for_string(G_CHECKSUM_SHA1,
+                                                       (guchar*)prof_identifier, strlen(prof_identifier),
+                                                       rndid, strlen(rndid));
 
     char* ret = g_strdup_printf("%s%s", rndid, hmac);
 
-    free(rndid);
-    g_free(hmac);
-
     return ret;
 }
 
@@ -831,8 +847,6 @@ connection_set_priority(const int priority)
     conn.priority = priority;
 }
 
-#if defined(LIBXMPP_VERSION_MAJOR) && defined(LIBXMPP_VERSION_MINOR) \
-    && ((LIBXMPP_VERSION_MAJOR > 0) || (LIBXMPP_VERSION_MINOR >= 12))
 static xmpp_stanza_t*
 _get_soh_error(xmpp_stanza_t* error_stanza)
 {
@@ -841,19 +855,6 @@ _get_soh_error(xmpp_stanza_t* error_stanza)
                                          XMPP_STANZA_NAME_IN_NS("see-other-host", STANZA_NS_XMPP_STREAMS),
                                          NULL);
 }
-#else
-static xmpp_stanza_t*
-_get_soh_error(xmpp_stanza_t* error_stanza)
-{
-    const char* name = xmpp_stanza_get_name(error_stanza);
-    const char* ns = xmpp_stanza_get_ns(error_stanza);
-    if (!name || !ns || strcmp(name, "error") || strcmp(ns, STANZA_NS_STREAMS)) {
-        log_debug("_get_soh_error: could not find error stanza");
-        return NULL;
-    }
-    return xmpp_stanza_get_child_by_name_and_ns(error_stanza, "see-other-host", STANZA_NS_XMPP_STREAMS);
-}
-#endif
 
 #if GLIB_CHECK_VERSION(2, 66, 0)
 static gboolean
@@ -862,12 +863,8 @@ _split_url(const char* alturi, gchar** host, gint* port)
     /* Construct a valid URI with `schema://` as `g_uri_split_network()`
      * requires this to be there.
      */
-    const char* xmpp = "xmpp://";
-    char* xmpp_uri = _xmalloc(strlen(xmpp) + strlen(alturi) + 1, NULL);
-    memcpy(xmpp_uri, xmpp, strlen(xmpp));
-    memcpy(xmpp_uri + strlen(xmpp), alturi, strlen(alturi) + 1);
+    auto_gchar gchar* xmpp_uri = g_strdup_printf("xmpp://%s", alturi);
     gboolean ret = g_uri_split_network(xmpp_uri, 0, NULL, host, port, NULL);
-    free(xmpp_uri);
     /* fix-up `port` as g_uri_split_network() sets port to `-1` if it's missing
      * in the passed-in URI, but libstrophe expects a "missing port"
      * to be passed as `0` (which then results in connecting to the standard port).
@@ -961,6 +958,7 @@ _connection_handler(xmpp_conn_t* const xmpp_conn, const xmpp_conn_event_t status
         conn.domain = strdup(my_jid->domainpart);
         jid_destroy(my_jid);
 
+        connection_clear_data();
         conn.features_by_jid = g_hash_table_new_full(g_str_hash, g_str_equal, free, (GDestroyNotify)g_hash_table_destroy);
         g_hash_table_insert(conn.features_by_jid, strdup(conn.domain), g_hash_table_new_full(g_str_hash, g_str_equal, free, NULL));
 
@@ -987,6 +985,7 @@ _connection_handler(xmpp_conn_t* const xmpp_conn, const xmpp_conn_event_t status
         conn.domain = strdup(my_raw_jid->domainpart);
         jid_destroy(my_raw_jid);
 
+        connection_clear_data();
         conn.features_by_jid = g_hash_table_new_full(g_str_hash, g_str_equal, free, (GDestroyNotify)g_hash_table_destroy);
         g_hash_table_insert(conn.features_by_jid, strdup(conn.domain), g_hash_table_new_full(g_str_hash, g_str_equal, free, NULL));
 
@@ -1000,13 +999,15 @@ _connection_handler(xmpp_conn_t* const xmpp_conn, const xmpp_conn_event_t status
 
         // lost connection for unknown reason
         if (conn.conn_status == JABBER_CONNECTED) {
-            int send_queue_len = xmpp_conn_send_queue_len(conn.xmpp_conn);
-            log_debug("Connection handler: Lost connection for unknown reason");
-            conn.sm_state = xmpp_conn_get_sm_state(conn.xmpp_conn);
-            if (send_queue_len > 0) {
-                conn.queued_messages = calloc(send_queue_len + 1, sizeof(*conn.queued_messages));
-                for (int n = 0; n < send_queue_len && conn.queued_messages[n]; ++n) {
-                    conn.queued_messages[n] = xmpp_conn_send_queue_drop_element(conn.xmpp_conn, XMPP_QUEUE_OLDEST);
+            if (prefs_get_boolean(PREF_STROPHE_SM_ENABLED)) {
+                int send_queue_len = xmpp_conn_send_queue_len(conn.xmpp_conn);
+                log_debug("Connection handler: Lost connection for unknown reason");
+                conn.sm_state = xmpp_conn_get_sm_state(conn.xmpp_conn);
+                if (send_queue_len > 0 && prefs_get_boolean(PREF_STROPHE_SM_RESEND)) {
+                    conn.queued_messages = calloc(send_queue_len + 1, sizeof(*conn.queued_messages));
+                    for (int n = 0; n < send_queue_len && conn.queued_messages[n]; ++n) {
+                        conn.queued_messages[n] = xmpp_conn_send_queue_drop_element(conn.xmpp_conn, XMPP_QUEUE_OLDEST);
+                    }
                 }
             }
             session_lost_connection();
@@ -1101,11 +1102,9 @@ _xmpp_file_logger(void* const userdata, const xmpp_log_level_t xmpp_level, const
 static void
 _random_bytes_init(void)
 {
-    char* rndbytes_loc;
+    auto_gchar gchar* rndbytes_loc = files_get_data_path(FILE_PROFANITY_IDENTIFIER);
     GKeyFile* rndbytes;
 
-    rndbytes_loc = files_get_data_path(FILE_PROFANITY_IDENTIFIER);
-
     if (g_file_test(rndbytes_loc, G_FILE_TEST_EXISTS)) {
         g_chmod(rndbytes_loc, S_IRUSR | S_IWUSR);
     }
@@ -1120,18 +1119,13 @@ _random_bytes_init(void)
         g_key_file_set_string(rndbytes, "identifier", "random_bytes", profanity_instance_id);
 
         gsize g_data_size;
-        gchar* g_accounts_data = g_key_file_to_data(rndbytes, &g_data_size, NULL);
+        auto_gchar gchar* g_accounts_data = g_key_file_to_data(rndbytes, &g_data_size, NULL);
 
-        gchar* base = g_path_get_dirname(rndbytes_loc);
-        gchar* true_loc = get_file_or_linked(rndbytes_loc, base);
+        auto_gchar gchar* base = g_path_get_dirname(rndbytes_loc);
+        auto_gchar gchar* true_loc = get_file_or_linked(rndbytes_loc, base);
         g_file_set_contents(true_loc, g_accounts_data, g_data_size, NULL);
-
-        g_free(base);
-        free(true_loc);
-        g_free(g_accounts_data);
     }
 
-    free(rndbytes_loc);
     g_key_file_free(rndbytes);
 }
 
