@@ -76,6 +76,7 @@ jid_create(const gchar* const str)
     result->resourcepart = NULL;
     result->barejid = NULL;
     result->fulljid = NULL;
+    result->refcnt = 1;
 
     gchar* atp = g_utf8_strchr(trimmed, -1, '@');
     gchar* slashp = g_utf8_strchr(trimmed, -1, '/');
@@ -89,10 +90,9 @@ jid_create(const gchar* const str)
     if (slashp) {
         result->resourcepart = g_strdup(slashp + 1);
         result->domainpart = g_utf8_substring(domain_start, 0, g_utf8_pointer_to_offset(domain_start, slashp));
-        char* barejidraw = g_utf8_substring(trimmed, 0, g_utf8_pointer_to_offset(trimmed, slashp));
+        auto_gchar gchar* barejidraw = g_utf8_substring(trimmed, 0, g_utf8_pointer_to_offset(trimmed, slashp));
         result->barejid = g_utf8_strdown(barejidraw, -1);
         result->fulljid = g_strdup(trimmed);
-        g_free(barejidraw);
     } else {
         result->domainpart = g_strdup(domain_start);
         result->barejid = g_utf8_strdown(trimmed, -1);
@@ -111,12 +111,22 @@ jid_create(const gchar* const str)
 Jid*
 jid_create_from_bare_and_resource(const char* const barejid, const char* const resource)
 {
-    Jid* result;
-    char* jid = create_fulljid(barejid, resource);
-    result = jid_create(jid);
-    free(jid);
+    auto_char char* jid = create_fulljid(barejid, resource);
+    return jid_create(jid);
+}
 
-    return result;
+void
+jid_auto_destroy(Jid** jid)
+{
+    if (jid == NULL)
+        return;
+    jid_destroy(*jid);
+}
+
+void
+jid_ref(Jid* jid)
+{
+    jid->refcnt++;
 }
 
 void
@@ -125,6 +135,10 @@ jid_destroy(Jid* jid)
     if (jid == NULL) {
         return;
     }
+    if (jid->refcnt > 1) {
+        jid->refcnt--;
+        return;
+    }
 
     g_free(jid->str);
     g_free(jid->localpart);
@@ -149,9 +163,8 @@ jid_is_valid_room_form(Jid* jid)
 char*
 create_fulljid(const char* const barejid, const char* const resource)
 {
-    gchar* barejidlower = g_utf8_strdown(barejid, -1);
+    auto_gchar gchar* barejidlower = g_utf8_strdown(barejid, -1);
     GString* full_jid = g_string_new(barejidlower);
-    g_free(barejidlower);
     g_string_append(full_jid, "/");
     g_string_append(full_jid, resource);
 
@@ -170,15 +183,13 @@ create_fulljid(const char* const barejid, const char* const resource)
 char*
 get_nick_from_full_jid(const char* const full_room_jid)
 {
-    char** tokens = g_strsplit(full_room_jid, "/", 0);
+    auto_gcharv gchar** tokens = g_strsplit(full_room_jid, "/", 0);
     char* nick_part = NULL;
 
     if (tokens) {
         if (tokens[0] && tokens[1]) {
             nick_part = strdup(tokens[1]);
         }
-
-        g_strfreev(tokens);
     }
 
     return nick_part;
@@ -197,13 +208,12 @@ jid_fulljid_or_barejid(Jid* jid)
     }
 }
 
-char*
+gchar*
 jid_random_resource(void)
 {
-    char* rand = get_random_string(4);
+    auto_char char* rand = get_random_string(4);
 
     gchar* result = g_strdup_printf("profanity.%s", rand);
-    free(rand);
 
     return result;
 }
