@@ -3,7 +3,7 @@
  * vim: expandtab:ts=4:sts=4:sw=4
  *
  * Copyright (C) 2012 - 2019 James Booth <boothj5@gmail.com>
- * Copyright (C) 2019 - 2022 Michael Vetter <jubalh@iodoru.org>
+ * Copyright (C) 2019 - 2023 Michael Vetter <jubalh@iodoru.org>
  *
  * This file is part of Profanity.
  *
@@ -243,7 +243,7 @@ _message_handler(xmpp_conn_t* const conn, xmpp_stanza_t* const stanza, void* con
         if (carbons) {
 
             // carbon must come from ourselves
-            char* mybarejid = connection_get_barejid();
+            auto_char char* mybarejid = connection_get_barejid();
             const char* const stanza_from = xmpp_stanza_get_from(stanza);
 
             if (stanza_from) {
@@ -256,8 +256,6 @@ _message_handler(xmpp_conn_t* const conn, xmpp_stanza_t* const stanza, void* con
                     msg_stanza = _handle_carbons(carbons);
                 }
             }
-
-            free(mybarejid);
         }
 
         if (msg_stanza) {
@@ -424,13 +422,12 @@ message_send_chat(const char* const barejid, const char* const msg, const char*
 {
     xmpp_ctx_t* const ctx = connection_get_ctx();
 
-    char* state = chat_session_get_state(barejid);
-    char* jid = chat_session_get_jid(barejid);
+    const char* state = chat_session_get_state(barejid);
+    auto_char char* jid = chat_session_get_jid(barejid);
     char* id = connection_create_stanza_id();
 
     xmpp_stanza_t* message = xmpp_message_new(ctx, STANZA_TYPE_CHAT, jid, id);
     xmpp_message_set_body(message, msg);
-    free(jid);
 
     if (state) {
         stanza_attach_state(ctx, message, state);
@@ -459,8 +456,8 @@ message_send_chat_pgp(const char* const barejid, const char* const msg, gboolean
 {
     xmpp_ctx_t* const ctx = connection_get_ctx();
 
-    char* state = chat_session_get_state(barejid);
-    char* jid = chat_session_get_jid(barejid);
+    const char* state = chat_session_get_state(barejid);
+    auto_char char* jid = chat_session_get_jid(barejid);
     char* id = connection_create_stanza_id();
 
     xmpp_stanza_t* message = NULL;
@@ -468,8 +465,8 @@ message_send_chat_pgp(const char* const barejid, const char* const msg, gboolean
     char* account_name = session_get_account_name();
     ProfAccount* account = accounts_get_account(account_name);
     if (account->pgp_keyid) {
-        Jid* jidp = jid_create(jid);
-        char* encrypted = p_gpg_encrypt(jidp->barejid, msg, account->pgp_keyid);
+        auto_jid Jid* jidp = jid_create(jid);
+        auto_char char* encrypted = p_gpg_encrypt(jidp->barejid, msg, account->pgp_keyid);
         if (encrypted) {
             message = xmpp_message_new(ctx, STANZA_TYPE_CHAT, jid, id);
             xmpp_message_set_body(message, "This message is encrypted (XEP-0027).");
@@ -482,12 +479,10 @@ message_send_chat_pgp(const char* const barejid, const char* const msg, gboolean
             xmpp_stanza_release(enc_st);
             xmpp_stanza_add_child(message, x);
             xmpp_stanza_release(x);
-            free(encrypted);
         } else {
             message = xmpp_message_new(ctx, STANZA_TYPE_CHAT, jid, id);
             xmpp_message_set_body(message, msg);
         }
-        jid_destroy(jidp);
     } else {
         message = xmpp_message_new(ctx, STANZA_TYPE_CHAT, jid, id);
         xmpp_message_set_body(message, msg);
@@ -498,7 +493,6 @@ message_send_chat_pgp(const char* const barejid, const char* const msg, gboolean
     message = xmpp_message_new(ctx, STANZA_TYPE_CHAT, jid, id);
     xmpp_message_set_body(message, msg);
 #endif
-    free(jid);
 
     if (state) {
         stanza_attach_state(ctx, message, state);
@@ -525,12 +519,11 @@ message_send_chat_ox(const char* const barejid, const char* const msg, gboolean
 #ifdef HAVE_LIBGPGME
     xmpp_ctx_t* const ctx = connection_get_ctx();
 
-    char* state = chat_session_get_state(barejid);
-    char* jid = chat_session_get_jid(barejid);
+    const char* state = chat_session_get_state(barejid);
+    auto_char char* jid = chat_session_get_jid(barejid);
     char* id = connection_create_stanza_id();
 
     xmpp_stanza_t* message = NULL;
-    Jid* jidp = jid_create(jid);
 
     char* account_name = session_get_account_name();
     ProfAccount* account = accounts_get_account(account_name);
@@ -561,8 +554,6 @@ message_send_chat_ox(const char* const barejid, const char* const msg, gboolean
     xmpp_stanza_to_text(message, &c, &s);
 
     account_free(account);
-    jid_destroy(jidp);
-    free(jid);
 
     if (state) {
         stanza_attach_state(ctx, message, state);
@@ -589,23 +580,16 @@ message_send_chat_otr(const char* const barejid, const char* const msg, gboolean
 {
     xmpp_ctx_t* const ctx = connection_get_ctx();
 
-    char* state = chat_session_get_state(barejid);
-    char* jid = chat_session_get_jid(barejid);
+    const char* state = chat_session_get_state(barejid);
     char* id = connection_create_stanza_id();
 
     xmpp_stanza_t* message = xmpp_message_new(ctx, STANZA_TYPE_CHAT, barejid, id);
     xmpp_message_set_body(message, msg);
 
-    free(jid);
-
     if (state) {
         stanza_attach_state(ctx, message, state);
     }
 
-    stanza_attach_carbons_private(ctx, message);
-    stanza_attach_hints_no_copy(ctx, message);
-    stanza_attach_hints_no_store(ctx, message);
-
     if (request_receipt) {
         stanza_attach_receipt_request(ctx, message);
     }
@@ -627,7 +611,7 @@ message_send_chat_omemo(const char* const jid, uint32_t sid, GList* keys,
                         const unsigned char* const ciphertext, size_t ciphertext_len,
                         gboolean request_receipt, gboolean muc, const char* const replace_id)
 {
-    char* state = chat_session_get_state(jid);
+    const char* state = chat_session_get_state(jid);
     xmpp_ctx_t* const ctx = connection_get_ctx();
     char* id;
     xmpp_stanza_t* message;
@@ -646,10 +630,9 @@ message_send_chat_omemo(const char* const jid, uint32_t sid, GList* keys,
 
     xmpp_stanza_t* header = xmpp_stanza_new(ctx);
     xmpp_stanza_set_name(header, "header");
-    char* sid_text = g_strdup_printf("%d", sid);
+    auto_gchar gchar* sid_text = g_strdup_printf("%d", sid);
     log_debug("[OMEMO] Sending from device sid %s", sid_text);
     xmpp_stanza_set_attribute(header, "sid", sid_text);
-    g_free(sid_text);
 
     GList* key_iter;
     for (key_iter = keys; key_iter != NULL; key_iter = key_iter->next) {
@@ -657,18 +640,16 @@ message_send_chat_omemo(const char* const jid, uint32_t sid, GList* keys,
 
         xmpp_stanza_t* key_stanza = xmpp_stanza_new(ctx);
         xmpp_stanza_set_name(key_stanza, "key");
-        char* rid = g_strdup_printf("%d", key->device_id);
-        log_debug("[OMEMO] Sending to device rid %s", rid == NULL ? "NULL" : rid);
+        auto_gchar gchar* rid = g_strdup_printf("%d", key->device_id);
+        log_debug("[OMEMO] Sending to device rid %s", STR_MAYBE_NULL(rid));
         xmpp_stanza_set_attribute(key_stanza, "rid", rid);
-        g_free(rid);
         if (key->prekey) {
             xmpp_stanza_set_attribute(key_stanza, "prekey", "true");
         }
 
-        gchar* key_raw = g_base64_encode(key->data, key->length);
+        auto_gchar gchar* key_raw = g_base64_encode(key->data, key->length);
         xmpp_stanza_t* key_text = xmpp_stanza_new(ctx);
         xmpp_stanza_set_text(key_text, key_raw);
-        g_free(key_raw);
 
         xmpp_stanza_add_child(key_stanza, key_text);
         xmpp_stanza_add_child(header, key_stanza);
@@ -679,10 +660,9 @@ message_send_chat_omemo(const char* const jid, uint32_t sid, GList* keys,
     xmpp_stanza_t* iv_stanza = xmpp_stanza_new(ctx);
     xmpp_stanza_set_name(iv_stanza, "iv");
 
-    gchar* iv_raw = g_base64_encode(iv, iv_len);
+    auto_gchar gchar* iv_raw = g_base64_encode(iv, iv_len);
     xmpp_stanza_t* iv_text = xmpp_stanza_new(ctx);
     xmpp_stanza_set_text(iv_text, iv_raw);
-    g_free(iv_raw);
 
     xmpp_stanza_add_child(iv_stanza, iv_text);
     xmpp_stanza_add_child(header, iv_stanza);
@@ -695,10 +675,9 @@ message_send_chat_omemo(const char* const jid, uint32_t sid, GList* keys,
     xmpp_stanza_t* payload = xmpp_stanza_new(ctx);
     xmpp_stanza_set_name(payload, "payload");
 
-    gchar* ciphertext_raw = g_base64_encode(ciphertext, ciphertext_len);
+    auto_gchar gchar* ciphertext_raw = g_base64_encode(ciphertext, ciphertext_len);
     xmpp_stanza_t* payload_text = xmpp_stanza_new(ctx);
     xmpp_stanza_set_text(payload_text, ciphertext_raw);
-    g_free(ciphertext_raw);
 
     xmpp_stanza_add_child(payload, payload_text);
     xmpp_stanza_add_child(encrypted, payload);
@@ -862,7 +841,7 @@ _handle_error(xmpp_stanza_t* const stanza)
     }
 
     // stanza_get_error never returns NULL
-    char* err_msg = stanza_get_error_message(stanza);
+    auto_char char* err_msg = stanza_get_error_message(stanza);
 
     GString* log_msg = g_string_new("message stanza error received");
     if (id) {
@@ -888,16 +867,13 @@ _handle_error(xmpp_stanza_t* const stanza)
         ui_handle_error(err_msg);
     } else {
         if (type && (strcmp(type, "cancel") == 0)) {
-            Jid* jidp = jid_create(jid);
+            auto_jid Jid* jidp = jid_create(jid);
             if (jidp) {
                 chat_session_remove(jidp->barejid);
-                jid_destroy(jidp);
             }
         }
         ui_handle_recipient_error(jid, err_msg);
     }
-
-    free(err_msg);
 }
 
 static void
@@ -928,7 +904,7 @@ _handle_muc_user(xmpp_stanza_t* const stanza)
         return;
     }
 
-    Jid* jidp = jid_create(invitor_jid);
+    auto_jid Jid* jidp = jid_create(invitor_jid);
     if (!jidp) {
         return;
     }
@@ -947,7 +923,6 @@ _handle_muc_user(xmpp_stanza_t* const stanza)
     }
 
     sv_ev_room_invite(INVITE_MEDIATED, invitor, room, reason, password);
-    jid_destroy(jidp);
     if (reason) {
         xmpp_free(ctx, reason);
     }
@@ -974,7 +949,7 @@ _handle_conference(xmpp_stanza_t* const stanza)
             return;
         }
 
-        Jid* jidp = jid_create(from);
+        auto_jid Jid* jidp = jid_create(from);
         if (!jidp) {
             return;
         }
@@ -984,7 +959,6 @@ _handle_conference(xmpp_stanza_t* const stanza)
         const char* password = xmpp_stanza_get_attribute(xns_conference, STANZA_ATTR_PASSWORD);
 
         sv_ev_room_invite(INVITE_DIRECT, jidp->barejid, room, reason, password);
-        jid_destroy(jidp);
     }
 }
 
@@ -1040,7 +1014,7 @@ _handle_groupchat(xmpp_stanza_t* const stanza)
     if (!room_jid) {
         return;
     }
-    Jid* from_jid = jid_create(room_jid);
+    auto_jid Jid* from_jid = jid_create(room_jid);
     if (!from_jid) {
         return;
     }
@@ -1053,7 +1027,6 @@ _handle_groupchat(xmpp_stanza_t* const stanza)
         sv_ev_room_subject(from_jid->barejid, from_jid->resourcepart, subject_text);
         xmpp_free(ctx, subject_text);
 
-        jid_destroy(from_jid);
         return;
     }
 
@@ -1072,10 +1045,9 @@ _handle_groupchat(xmpp_stanza_t* const stanza)
 
                     if (code) {
                         // If configuration change notification send disco info to get updated info of the muc
-                        char* disqo_info_id = connection_create_stanza_id();
+                        auto_char char* disqo_info_id = connection_create_stanza_id();
                         xmpp_stanza_t* iq = stanza_create_disco_info_iq(ctx, disqo_info_id, room_jid, NULL);
                         iq_id_handler_add(disqo_info_id, _room_config_handler, NULL, NULL);
-                        free(disqo_info_id);
                         iq_send_stanza(iq);
                         xmpp_stanza_release(iq);
 
@@ -1084,31 +1056,28 @@ _handle_groupchat(xmpp_stanza_t* const stanza)
                 }
             }
 
-            jid_destroy(from_jid);
             return;
         }
 
         sv_ev_room_broadcast(room_jid, broadcast);
         xmpp_free(ctx, broadcast);
 
-        jid_destroy(from_jid);
         return;
     }
 
     if (!jid_is_valid_room_form(from_jid)) {
         log_error("Invalid room JID: %s", from_jid->str);
-        jid_destroy(from_jid);
         return;
     }
 
     // room not active in profanity
     if (!muc_active(from_jid->barejid)) {
         log_error("Message received for inactive chat room: %s", from_jid->str);
-        jid_destroy(from_jid);
         return;
     }
 
     ProfMessage* message = message_init();
+    jid_ref(from_jid);
     message->from_jid = from_jid;
     message->type = PROF_MSG_TYPE_MUC;
 
@@ -1198,9 +1167,8 @@ _message_send_receipt(const char* const fulljid, const char* const message_id)
 {
     xmpp_ctx_t* const ctx = connection_get_ctx();
 
-    char* id = connection_create_stanza_id();
+    auto_char char* id = connection_create_stanza_id();
     xmpp_stanza_t* message = xmpp_message_new(ctx, NULL, fulljid, id);
-    free(id);
 
     xmpp_stanza_t* receipt = xmpp_stanza_new(ctx);
     xmpp_stanza_set_name(receipt, "received");
@@ -1234,13 +1202,12 @@ _handle_receipt_received(xmpp_stanza_t* const stanza)
             return;
         }
 
-        Jid* jidp = jid_create(fulljid);
+        auto_jid Jid* jidp = jid_create(fulljid);
         if (!jidp) {
             return;
         }
 
         sv_ev_message_receipt(jidp->barejid, id);
-        jid_destroy(jidp);
     }
 }
 
@@ -1268,10 +1235,9 @@ _receipt_request_handler(xmpp_stanza_t* const stanza)
 
     const gchar* from = xmpp_stanza_get_from(stanza);
     if (from) {
-        Jid* jid = jid_create(from);
+        auto_jid Jid* jid = jid_create(from);
         if (jid) {
             _message_send_receipt(jid->fulljid, id);
-            jid_destroy(jid);
         }
     }
 }
@@ -1362,12 +1328,10 @@ _handle_carbons(xmpp_stanza_t* const stanza)
     }
 
     // Eliminate duplicate messages in chat with oneself when another client is sending a message
-    char* bare_from = xmpp_jid_bare(connection_get_ctx(), xmpp_stanza_get_from(message_stanza));
+    auto_char char* bare_from = xmpp_jid_bare(connection_get_ctx(), xmpp_stanza_get_from(message_stanza));
     if (g_strcmp0(bare_from, xmpp_stanza_get_to(message_stanza)) == 0) {
-        free(bare_from);
         return NULL;
     }
-    free(bare_from);
 
     return message_stanza;
 }
@@ -1388,7 +1352,7 @@ _handle_chat(xmpp_stanza_t* const stanza, gboolean is_mam, gboolean is_carbon, c
     if (!from) {
         return;
     }
-    Jid* jid = jid_create(from);
+    auto_jid Jid* jid = jid_create(from);
     if (!jid) {
         return;
     }
@@ -1396,13 +1360,13 @@ _handle_chat(xmpp_stanza_t* const stanza, gboolean is_mam, gboolean is_carbon, c
     // private message from chat room use full jid (room/nick)
     if (muc_active(jid->barejid)) {
         _handle_muc_private_message(stanza);
-        jid_destroy(jid);
         return;
     }
 
     // standard chat message, use jid without resource
     ProfMessage* message = message_init();
     message->is_mam = is_mam;
+    jid_ref(jid);
     message->from_jid = jid;
     const gchar* to = xmpp_stanza_get_to(stanza);
     if (to) {
@@ -1488,7 +1452,7 @@ _handle_chat(xmpp_stanza_t* const stanza, gboolean is_mam, gboolean is_carbon, c
 
     if (message->plain || message->body || message->encrypted) {
         if (is_carbon) {
-            char* mybarejid = connection_get_barejid();
+            auto_char char* mybarejid = connection_get_barejid();
 
             // if we are the recipient, treat as standard incoming message
             if (g_strcmp0(mybarejid, message->to_jid->barejid) == 0) {
@@ -1497,8 +1461,6 @@ _handle_chat(xmpp_stanza_t* const stanza, gboolean is_mam, gboolean is_carbon, c
             } else {
                 sv_ev_outgoing_carbon(message);
             }
-
-            free(mybarejid);
         } else {
             sv_ev_incoming_message(message);
             _receipt_request_handler(stanza);
@@ -1608,10 +1570,9 @@ _send_message_stanza(xmpp_stanza_t* const stanza)
     xmpp_stanza_to_text(stanza, &text, &text_size);
 
     xmpp_conn_t* conn = connection_get_conn();
-    char* plugin_text = plugins_on_message_stanza_send(text);
+    auto_char char* plugin_text = plugins_on_message_stanza_send(text);
     if (plugin_text) {
         xmpp_send_raw_string(conn, "%s", plugin_text);
-        free(plugin_text);
     } else {
         xmpp_send_raw_string(conn, "%s", text);
     }
@@ -1642,19 +1603,16 @@ message_is_sent_by_us(const ProfMessage* const message, bool checkOID)
 
             // our client sents at CON_RAND_ID_LEN + identifier
             if (tmp_len > CON_RAND_ID_LEN) {
-                char* uuid = g_strndup(tmp_id, CON_RAND_ID_LEN);
+                auto_gchar gchar* uuid = g_strndup(tmp_id, CON_RAND_ID_LEN);
                 const char* prof_identifier = connection_get_profanity_identifier();
 
-                gchar* hmac = g_compute_hmac_for_string(G_CHECKSUM_SHA1,
-                                                        (guchar*)prof_identifier, strlen(prof_identifier),
-                                                        uuid, strlen(uuid));
+                auto_gchar gchar* hmac = g_compute_hmac_for_string(G_CHECKSUM_SHA1,
+                                                                   (guchar*)prof_identifier, strlen(prof_identifier),
+                                                                   uuid, strlen(uuid));
 
                 if (g_strcmp0(&tmp_id[CON_RAND_ID_LEN], hmac) == 0) {
                     ret = TRUE;
                 }
-
-                g_free(uuid);
-                g_free(hmac);
             }
         }
     }
@@ -1761,9 +1719,8 @@ _should_ignore_based_on_silence(xmpp_stanza_t* const stanza)
 {
     if (prefs_get_boolean(PREF_SILENCE_NON_ROSTER)) {
         const char* const from = xmpp_stanza_get_from(stanza);
-        Jid* from_jid = jid_create(from);
+        auto_jid Jid* from_jid = jid_create(from);
         PContact contact = roster_get_contact(from_jid->barejid);
-        jid_destroy(from_jid);
         if (!contact) {
             log_debug("[Silence] Ignoring message from: %s", from);
             return TRUE;
