@@ -495,11 +495,10 @@ muc_roster_add(const char* const room, const char* const nick, const char* const
         g_hash_table_replace(chat_room->roster, strdup(nick), occupant);
 
         if (jid) {
-            Jid* jidp = jid_create(jid);
+            auto_jid Jid* jidp = jid_create(jid);
             if (jidp->barejid) {
                 autocomplete_add(chat_room->jid_ac, jidp->barejid);
             }
-            jid_destroy(jidp);
         }
     }
 
@@ -737,7 +736,7 @@ muc_autocomplete(ProfWin* window, const char* const input, gboolean previous)
         }
     }
 
-    char* result = autocomplete_complete(chat_room->nick_ac, search_str, FALSE, previous);
+    auto_gchar gchar* result = autocomplete_complete(chat_room->nick_ac, search_str, FALSE, previous);
     if (result == NULL) {
         return NULL;
     }
@@ -748,10 +747,7 @@ muc_autocomplete(ProfWin* window, const char* const input, gboolean previous)
     if (strlen(chat_room->autocomplete_prefix) == 0) {
         g_string_append(replace_with, ": ");
     }
-    g_free(result);
-    result = replace_with->str;
-    g_string_free(replace_with, FALSE);
-    return result;
+    return g_string_free(replace_with, FALSE);
 }
 
 void
@@ -774,13 +770,12 @@ muc_jid_autocomplete_add_all(const char* const room, GSList* jids)
             GSList* curr_jid = jids;
             while (curr_jid) {
                 const char* jid = curr_jid->data;
-                Jid* jidp = jid_create(jid);
+                auto_jid Jid* jidp = jid_create(jid);
                 if (jidp) {
                     if (jidp->barejid) {
                         autocomplete_add(chat_room->jid_ac, jidp->barejid);
                     }
                 }
-                jid_destroy(jidp);
                 curr_jid = g_slist_next(curr_jid);
             }
         }
@@ -887,11 +882,10 @@ muc_members_add(const char* const room, const char* const jid)
         if (g_hash_table_insert(chat_room->members, strdup(jid), NULL)) {
 #ifdef HAVE_OMEMO
             if (chat_room->anonymity_type == MUC_ANONYMITY_TYPE_NONANONYMOUS) {
-                char* our_barejid = connection_get_barejid();
+                auto_char char* our_barejid = connection_get_barejid();
                 if (strcmp(jid, our_barejid) != 0) {
                     omemo_start_session(jid);
                 }
-                free(our_barejid);
             }
 #endif
         }
