@@ -51,6 +51,8 @@
 #include "event/server_events.h"
 #include "plugins/plugins.h"
 #include "ui/ui.h"
+#include "ui/window.h"
+#include "ui/window_list.h"
 #include "xmpp/connection.h"
 #include "xmpp/capabilities.h"
 #include "xmpp/session.h"
@@ -94,7 +96,7 @@ presence_subscription(const char* const jid, const jabber_subscr_t action)
 {
     assert(jid != NULL);
 
-    Jid* jidp = jid_create(jid);
+    auto_jid Jid* jidp = jid_create(jid);
     autocomplete_remove(sub_requests_ac, jidp->barejid);
 
     const char* type = NULL;
@@ -122,13 +124,11 @@ presence_subscription(const char* const jid, const jabber_subscr_t action)
     xmpp_ctx_t* const ctx = connection_get_ctx();
     xmpp_stanza_t* presence = xmpp_presence_new(ctx);
 
-    char* id = connection_create_stanza_id();
+    auto_char char* id = connection_create_stanza_id();
     xmpp_stanza_set_id(presence, id);
-    free(id);
 
     xmpp_stanza_set_type(presence, type);
     xmpp_stanza_set_to(presence, jidp->barejid);
-    jid_destroy(jidp);
 
     _send_presence_stanza(presence);
 
@@ -205,9 +205,8 @@ presence_send(const resource_presence_t presence_type, const int idle, char* sig
     xmpp_ctx_t* const ctx = connection_get_ctx();
     xmpp_stanza_t* presence = xmpp_presence_new(ctx);
 
-    char* id = connection_create_stanza_id();
+    auto_char char* id = connection_create_stanza_id();
     xmpp_stanza_set_id(presence, id);
-    free(id);
 
     const char* show = stanza_get_presence_string_from_type(presence_type);
     stanza_attach_show(ctx, presence, show);
@@ -263,10 +262,9 @@ _send_room_presence(xmpp_stanza_t* presence)
         const char* nick = muc_nick(room);
 
         if (nick) {
-            char* full_room_jid = create_fulljid(room, nick);
+            auto_char char* full_room_jid = create_fulljid(room, nick);
             xmpp_stanza_set_to(presence, full_room_jid);
             log_debug("Sending presence to room: %s", full_room_jid);
-            free(full_room_jid);
 
             _send_presence_stanza(presence);
         }
@@ -279,7 +277,7 @@ _send_room_presence(xmpp_stanza_t* presence)
 void
 presence_join_room(const char* const room, const char* const nick, const char* const passwd)
 {
-    Jid* jid = jid_create_from_bare_and_resource(room, nick);
+    auto_jid Jid* jid = jid_create_from_bare_and_resource(room, nick);
     log_debug("Sending room join presence to: %s", jid->fulljid);
 
     resource_presence_t presence_type = accounts_get_last_presence(session_get_account_name());
@@ -297,7 +295,6 @@ presence_join_room(const char* const room, const char* const nick, const char* c
     _send_presence_stanza(presence);
 
     xmpp_stanza_release(presence);
-    jid_destroy(jid);
 }
 
 void
@@ -311,7 +308,7 @@ presence_change_room_nick(const char* const room, const char* const nick)
     const char* show = stanza_get_presence_string_from_type(presence_type);
     char* status = connection_get_presence_msg();
     int pri = accounts_get_priority_for_presence_type(session_get_account_name(), presence_type);
-    char* full_room_jid = create_fulljid(room, nick);
+    auto_char char* full_room_jid = create_fulljid(room, nick);
 
     xmpp_ctx_t* ctx = connection_get_ctx();
     xmpp_stanza_t* presence = stanza_create_room_newnick_presence(ctx, full_room_jid);
@@ -323,7 +320,6 @@ presence_change_room_nick(const char* const room, const char* const nick)
     _send_presence_stanza(presence);
 
     xmpp_stanza_release(presence);
-    free(full_room_jid);
 }
 
 void
@@ -417,13 +413,12 @@ _presence_error_handler(xmpp_stanza_t* const stanza)
             error_cond = "unknown";
         }
 
-        Jid* fulljid = jid_create(from);
+        auto_jid Jid* fulljid = jid_create(from);
         log_info("Error joining room: %s, reason: %s", fulljid->barejid, error_cond);
         if (muc_active(fulljid->barejid)) {
             muc_leave(fulljid->barejid);
         }
         cons_show_error("Error joining room %s, reason: %s", fulljid->barejid, error_cond);
-        jid_destroy(fulljid);
 
         return;
     }
@@ -449,7 +444,7 @@ _presence_error_handler(xmpp_stanza_t* const stanza)
     }
 
     // stanza_get_error never returns NULL
-    char* err_msg = stanza_get_error_message(stanza);
+    auto_char char* err_msg = stanza_get_error_message(stanza);
     g_string_append(log_msg, " error=");
     g_string_append(log_msg, err_msg);
 
@@ -462,8 +457,6 @@ _presence_error_handler(xmpp_stanza_t* const stanza)
     } else {
         ui_handle_error(err_msg);
     }
-
-    free(err_msg);
 }
 
 static void
@@ -476,11 +469,9 @@ _unsubscribed_handler(xmpp_stanza_t* const stanza)
     }
     log_debug("Unsubscribed presence handler fired for %s", from);
 
-    Jid* from_jid = jid_create(from);
+    auto_jid Jid* from_jid = jid_create(from);
     sv_ev_subscription(from_jid->barejid, PRESENCE_UNSUBSCRIBED);
     autocomplete_remove(sub_requests_ac, from_jid->barejid);
-
-    jid_destroy(from_jid);
 }
 
 static void
@@ -493,11 +484,9 @@ _subscribed_handler(xmpp_stanza_t* const stanza)
     }
     log_debug("Subscribed presence handler fired for %s", from);
 
-    Jid* from_jid = jid_create(from);
+    auto_jid Jid* from_jid = jid_create(from);
     sv_ev_subscription(from_jid->barejid, PRESENCE_SUBSCRIBED);
     autocomplete_remove(sub_requests_ac, from_jid->barejid);
-
-    jid_destroy(from_jid);
 }
 
 static void
@@ -509,15 +498,13 @@ _subscribe_handler(xmpp_stanza_t* const stanza)
     }
     log_debug("Subscribe presence handler fired for %s", from);
 
-    Jid* from_jid = jid_create(from);
+    auto_jid Jid* from_jid = jid_create(from);
     if (from_jid == NULL) {
         return;
     }
 
     sv_ev_subscription(from_jid->barejid, PRESENCE_SUBSCRIBE);
     autocomplete_add(sub_requests_ac, from_jid->barejid);
-
-    jid_destroy(from_jid);
 }
 
 static void
@@ -533,16 +520,14 @@ _unavailable_handler(xmpp_stanza_t* const stanza)
     }
     log_debug("Unavailable presence handler fired for %s", from);
 
-    Jid* my_jid = jid_create(jid);
-    Jid* from_jid = jid_create(from);
+    auto_jid Jid* my_jid = jid_create(jid);
+    auto_jid Jid* from_jid = jid_create(from);
     if (my_jid == NULL || from_jid == NULL) {
-        jid_destroy(my_jid);
-        jid_destroy(from_jid);
         return;
     }
 
     if (strcmp(my_jid->barejid, from_jid->barejid) != 0) {
-        char* status_str = stanza_get_status(stanza, NULL);
+        auto_char char* status_str = stanza_get_status(stanza, NULL);
         if (from_jid->resourcepart) {
             sv_ev_contact_offline(from_jid->barejid, from_jid->resourcepart, status_str);
 
@@ -550,15 +535,11 @@ _unavailable_handler(xmpp_stanza_t* const stanza)
         } else {
             sv_ev_contact_offline(from_jid->barejid, "__prof_default", status_str);
         }
-        free(status_str);
     } else {
         if (from_jid->resourcepart) {
             connection_remove_available_resource(from_jid->resourcepart);
         }
     }
-
-    jid_destroy(my_jid);
-    jid_destroy(from_jid);
 }
 
 static void
@@ -573,25 +554,22 @@ _handle_caps(const char* const jid, XMPPCaps* caps)
                 caps_map_jid_to_ver(jid, caps->ver);
             } else {
                 log_debug("Capabilities cache miss: %s, for %s, sending service discovery request", caps->ver, jid);
-                char* id = connection_create_stanza_id();
+                auto_char char* id = connection_create_stanza_id();
                 iq_send_caps_request(jid, id, caps->node, caps->ver);
-                free(id);
             }
         }
 
         // unsupported hash, xep-0115, associate with JID, no cache
     } else if (caps->hash) {
         log_info("Hash %s not supported: %s, sending service discovery request", caps->hash, jid);
-        char* id = connection_create_stanza_id();
+        auto_char char* id = connection_create_stanza_id();
         iq_send_caps_request_for_jid(jid, id, caps->node, caps->ver);
-        free(id);
 
         // no hash, legacy caps, cache against node#ver
     } else if (caps->node && caps->ver) {
         log_info("No hash specified: %s, legacy request made for %s#%s", jid, caps->node, caps->ver);
-        char* id = connection_create_stanza_id();
+        auto_char char* id = connection_create_stanza_id();
         iq_send_caps_request_legacy(jid, id, caps->node, caps->ver);
-        free(id);
     } else {
         log_info("No hash specified: %s, could not create ver string, not sending service discovery request.", jid);
     }
@@ -642,7 +620,7 @@ _available_handler(xmpp_stanza_t* const stanza)
 
     xmpp_conn_t* conn = connection_get_conn();
     const char* my_jid_str = xmpp_conn_get_jid(conn);
-    Jid* my_jid = jid_create(my_jid_str);
+    auto_jid Jid* my_jid = jid_create(my_jid_str);
 
     XMPPCaps* caps = stanza_parse_caps(stanza);
     if ((g_strcmp0(my_jid->fulljid, xmpp_presence->jid->fulljid) != 0) && caps) {
@@ -656,6 +634,68 @@ _available_handler(xmpp_stanza_t* const stanza)
 
     if (g_strcmp0(xmpp_presence->jid->barejid, my_jid->barejid) == 0) {
         connection_add_available_resource(resource);
+        const char* account_name = session_get_account_name();
+        int max_sessions = accounts_get_max_sessions(account_name);
+        if (max_sessions > 0) {
+            const gchar* cur_resource = accounts_get_resource(account_name);
+            int res_count = connection_count_available_resources();
+            if (res_count > max_sessions && g_strcmp0(cur_resource, resource->name)) {
+                ProfWin* console = wins_get_console();
+                ProfWin* current_window = wins_get_current();
+                auto_gchar gchar* message = g_strdup_printf("Max sessions alarm! (%d/%d devices in use)", res_count, max_sessions);
+                win_println(console, THEME_RED, "|", "%s", message);
+                if (console != current_window) {
+                    win_println(current_window, THEME_RED, "|", "%s - check the console for more details!", message);
+                }
+                notify(message, 10000, "Security alert");
+
+                const char* resource_presence = string_from_resource_presence(resource->presence);
+                win_print(console, THEME_DEFAULT, "|", "New device info: \n    %s (%d), %s", resource->name, resource->priority, resource_presence);
+
+                if (resource->status) {
+                    win_append(console, THEME_DEFAULT, ", \"%s\"", resource->status);
+                }
+                win_appendln(console, THEME_DEFAULT, "");
+                auto_jid Jid* jidp = jid_create_from_bare_and_resource(my_jid->barejid, resource->name);
+                EntityCapabilities* caps = caps_lookup(jidp->fulljid);
+
+                if (caps) {
+                    if (caps->identity) {
+                        DiscoIdentity* identity = caps->identity;
+                        win_print(console, THEME_DEFAULT, "|", "    %s %s %s", identity->name, identity->type, identity->category);
+                        win_newline(console);
+                    }
+
+                    if (caps->software_version) {
+                        SoftwareVersion* software_version = caps->software_version;
+                        if (software_version->software) {
+                            win_print(console, THEME_DEFAULT, "|", "    Software: %s", software_version->software);
+                        }
+                        if (software_version->software_version) {
+                            win_append(console, THEME_DEFAULT, ", %s", software_version->software_version);
+                        }
+                        if (software_version->software || software_version->software_version) {
+                            win_newline(console);
+                        }
+                        if (software_version->os) {
+                            win_print(console, THEME_DEFAULT, "|", "    OS: %s", software_version->os);
+                        }
+                        if (software_version->os_version) {
+                            win_append(console, THEME_DEFAULT, ", %s", software_version->os_version);
+                        }
+                        if (software_version->os || software_version->os_version) {
+                            win_newline(console);
+                        }
+                    }
+
+                    caps_destroy(caps);
+                }
+
+                win_println(console, THEME_RED_BOLD, "|", "If it wasn't you, change your password. Use: /changepassword");
+                win_println(console, THEME_GREEN, "|", "If it was you, update the `session_alarm` limit that determines when to trigger this alarm, use: /account set %s session_alarm %d", account_name, res_count);
+                cons_alert(NULL);
+            }
+        }
     } else {
         char* pgpsig = NULL;
         xmpp_stanza_t* x = xmpp_stanza_get_child_by_ns(stanza, STANZA_NS_SIGNED);
@@ -667,7 +707,6 @@ _available_handler(xmpp_stanza_t* const stanza)
         xmpp_free(ctx, pgpsig);
     }
 
-    jid_destroy(my_jid);
     stanza_free_presence(xmpp_presence);
 }
 
@@ -698,7 +737,7 @@ static void
 _muc_user_self_handler(xmpp_stanza_t* stanza)
 {
     const char* from = xmpp_stanza_get_from(stanza);
-    Jid* from_jid = jid_create(from);
+    auto_jid Jid* from_jid = jid_create(from);
 
     log_debug("Room self presence received from %s", from_jid->fulljid);
 
@@ -719,25 +758,21 @@ _muc_user_self_handler(xmpp_stanza_t* stanza)
             // room destroyed
             if (stanza_room_destroyed(stanza)) {
                 const char* new_jid = stanza_get_muc_destroy_alternative_room(stanza);
-                char* password = stanza_get_muc_destroy_alternative_password(stanza);
-                char* reason = stanza_get_muc_destroy_reason(stanza);
+                auto_char char* password = stanza_get_muc_destroy_alternative_password(stanza);
+                auto_char char* reason = stanza_get_muc_destroy_reason(stanza);
                 sv_ev_room_destroyed(room, new_jid, password, reason);
-                free(password);
-                free(reason);
 
                 // kicked from room
             } else if (g_slist_find_custom(status_codes, "307", (GCompareFunc)g_strcmp0)) {
                 const char* actor = stanza_get_actor(stanza);
-                char* reason = stanza_get_reason(stanza);
+                auto_char char* reason = stanza_get_reason(stanza);
                 sv_ev_room_kicked(room, actor, reason);
-                free(reason);
 
                 // banned from room
             } else if (g_slist_find_custom(status_codes, "301", (GCompareFunc)g_strcmp0)) {
                 const char* actor = stanza_get_actor(stanza);
-                char* reason = stanza_get_reason(stanza);
+                auto_char char* reason = stanza_get_reason(stanza);
                 sv_ev_room_banned(room, actor, reason);
-                free(reason);
 
                 // normal exit
             } else {
@@ -754,9 +789,9 @@ _muc_user_self_handler(xmpp_stanza_t* stanza)
         }
 
         muc_nick_change_complete(room, nick);
-        char* reason = stanza_get_reason(stanza);
-        char* show_str = stanza_get_show(stanza, "online");
-        char* status_str = stanza_get_status(stanza, NULL);
+        auto_char char* reason = stanza_get_reason(stanza);
+        auto_char char* show_str = stanza_get_show(stanza, "online");
+        auto_char char* status_str = stanza_get_status(stanza, NULL);
         const char* actor = stanza_get_actor(stanza);
         const char* jid = NULL;
         const char* role = NULL;
@@ -772,25 +807,20 @@ _muc_user_self_handler(xmpp_stanza_t* stanza)
             }
         }
         sv_ev_muc_self_online(room, nick, config_required, role, affiliation, actor, reason, jid, show_str, status_str);
-        free(show_str);
-        free(status_str);
-        free(reason);
     }
-
-    jid_destroy(from_jid);
 }
 
 static void
 _muc_user_occupant_handler(xmpp_stanza_t* stanza)
 {
     const char* from = xmpp_stanza_get_from(stanza);
-    Jid* from_jid = jid_create(from);
+    auto_jid Jid* from_jid = jid_create(from);
 
     log_debug("Room presence received from %s", from_jid->fulljid);
 
     char* room = from_jid->barejid;
     char* nick = from_jid->resourcepart;
-    char* status_str = stanza_get_status(stanza, NULL);
+    auto_char char* status_str = stanza_get_status(stanza, NULL);
 
     if (!nick) {
         log_warning("presence: jid without resource");
@@ -812,16 +842,14 @@ _muc_user_occupant_handler(xmpp_stanza_t* stanza)
             // kicked from room
             if (g_slist_find_custom(status_codes, "307", (GCompareFunc)g_strcmp0)) {
                 const char* actor = stanza_get_actor(stanza);
-                char* reason = stanza_get_reason(stanza);
+                auto_char char* reason = stanza_get_reason(stanza);
                 sv_ev_room_occupent_kicked(room, nick, actor, reason);
-                free(reason);
 
                 // banned from room
             } else if (g_slist_find_custom(status_codes, "301", (GCompareFunc)g_strcmp0)) {
                 const char* actor = stanza_get_actor(stanza);
-                char* reason = stanza_get_reason(stanza);
+                auto_char char* reason = stanza_get_reason(stanza);
                 sv_ev_room_occupent_banned(room, nick, actor, reason);
-                free(reason);
 
                 // normal exit
             } else {
@@ -841,8 +869,8 @@ _muc_user_occupant_handler(xmpp_stanza_t* stanza)
         stanza_free_caps(caps);
 
         const char* actor = stanza_get_actor(stanza);
-        char* show_str = stanza_get_show(stanza, "online");
-        char* reason = stanza_get_reason(stanza);
+        auto_char char* show_str = stanza_get_show(stanza, "online");
+        auto_char char* reason = stanza_get_reason(stanza);
         const char* jid = NULL;
         const char* role = NULL;
         const char* affiliation = NULL;
@@ -856,12 +884,7 @@ _muc_user_occupant_handler(xmpp_stanza_t* stanza)
             }
         }
         sv_ev_muc_occupant_online(room, nick, jid, role, affiliation, actor, reason, show_str, status_str);
-        free(show_str);
-        free(reason);
     }
-
-    jid_destroy(from_jid);
-    free(status_str);
 }
 
 static void
@@ -881,13 +904,11 @@ _muc_user_handler(xmpp_stanza_t* const stanza)
         return;
     }
 
-    Jid* from_jid = jid_create(from);
+    auto_jid Jid* from_jid = jid_create(from);
     if (from_jid == NULL || from_jid->resourcepart == NULL) {
         log_warning("MUC User stanza received with invalid from attribute: %s", from);
-        jid_destroy(from_jid);
         return;
     }
-    jid_destroy(from_jid);
 
     if (stanza_is_muc_self_presence(stanza, connection_get_fulljid())) {
         _muc_user_self_handler(stanza);
@@ -904,10 +925,9 @@ _send_presence_stanza(xmpp_stanza_t* const stanza)
     xmpp_stanza_to_text(stanza, &text, &text_size);
 
     xmpp_conn_t* conn = connection_get_conn();
-    char* plugin_text = plugins_on_presence_stanza_send(text);
+    auto_char char* plugin_text = plugins_on_presence_stanza_send(text);
     if (plugin_text) {
         xmpp_send_raw_string(conn, "%s", plugin_text);
-        free(plugin_text);
     } else {
         xmpp_send_raw_string(conn, "%s", text);
     }
