@@ -151,9 +151,8 @@ roster_update_presence(const char* const barejid, Resource* resource, GDateTime*
         p_contact_set_last_activity(contact, last_activity);
     }
     p_contact_set_presence(contact, resource);
-    Jid* jid = jid_create_from_bare_and_resource(barejid, resource->name);
+    auto_jid Jid* jid = jid_create_from_bare_and_resource(barejid, resource->name);
     autocomplete_add(roster->fulljid_ac, jid->fulljid);
-    jid_destroy(jid);
 
     return TRUE;
 }
@@ -163,9 +162,8 @@ roster_get_contact(const char* const barejid)
 {
     assert(roster != NULL);
 
-    gchar* barejidlower = g_utf8_strdown(barejid, -1);
+    auto_gchar gchar* barejidlower = g_utf8_strdown(barejid, -1);
     PContact contact = g_hash_table_lookup(roster->contacts, barejidlower);
-    g_free(barejidlower);
 
     return contact;
 }
@@ -188,18 +186,15 @@ roster_get_display_name(const char* const barejid)
         g_string_append(result, barejid);
     }
 
-    char* result_str = result->str;
-    g_string_free(result, FALSE);
-
-    return result_str;
+    return g_string_free(result, FALSE);
 }
 
-char*
+gchar*
 roster_get_msg_display_name(const char* const barejid, const char* const resource)
 {
     assert(roster != NULL);
 
-    char* incoming_str = prefs_get_string(PREF_INCOMING_STAMP);
+    gchar* incoming_str = prefs_get_string(PREF_INCOMING_STAMP);
 
     if (incoming_str) {
         return incoming_str;
@@ -223,10 +218,7 @@ roster_get_msg_display_name(const char* const barejid, const char* const resourc
         g_string_append(result, resource);
     }
 
-    char* result_str = result->str;
-    g_string_free(result, FALSE);
-
-    return result_str;
+    return g_string_free(result, FALSE);
 }
 
 gboolean
@@ -244,9 +236,8 @@ roster_contact_offline(const char* const barejid, const char* const resource, co
     } else {
         gboolean result = p_contact_remove_resource(contact, resource);
         if (result == TRUE) {
-            Jid* jid = jid_create_from_bare_and_resource(barejid, resource);
+            auto_jid Jid* jid = jid_create_from_bare_and_resource(barejid, resource);
             autocomplete_remove(roster->fulljid_ac, jid->fulljid);
-            jid_destroy(jid);
         }
 
         return result;
@@ -270,7 +261,7 @@ roster_change_name(PContact contact, const char* const new_name)
     assert(roster != NULL);
     assert(contact != NULL);
 
-    char* current_name = NULL;
+    auto_char char* current_name = NULL;
     const char* barejid = p_contact_barejid(contact);
 
     if (p_contact_name(contact)) {
@@ -279,7 +270,6 @@ roster_change_name(PContact contact, const char* const new_name)
 
     p_contact_set_name(contact, new_name);
     _replace_name(current_name, new_name, barejid);
-    free(current_name);
 }
 
 void
@@ -421,6 +411,15 @@ roster_add(const char* const barejid, const char* const name, GSList* groups, co
     return TRUE;
 }
 
+/**
+ * Retrieve the bare JID from the roster based on the name.
+ *
+ * @param name The name associated with the bare JID.
+ * @return The corresponding bare JID if found, or NULL if not found.
+ *
+ * @note The returned value should not be freed explicitly. The memory
+ *       management is handled by the underlying hash table.
+ */
 char*
 roster_barejid_from_name(const char* const name)
 {
