@@ -96,8 +96,6 @@ static activity_state_t activity_state;
 static resource_presence_t saved_presence;
 static char* saved_status;
 
-static void _session_reconnect(void);
-
 static void _session_free_internals(void);
 static void _session_free_saved_details(void);
 
@@ -123,11 +121,10 @@ session_connect_with_account(const ProfAccount* const account)
     saved_account.name = strdup(account->name);
     saved_account.passwd = strdup(account->password);
 
-    char* jid = NULL;
+    auto_char char* jid = NULL;
     if (account->resource) {
-        Jid* jidp = jid_create_from_bare_and_resource(account->jid, account->resource);
+        auto_jid Jid* jidp = jid_create_from_bare_and_resource(account->jid, account->resource);
         jid = strdup(jidp->fulljid);
-        jid_destroy(jidp);
     } else {
         jid = strdup(account->jid);
     }
@@ -139,7 +136,6 @@ session_connect_with_account(const ProfAccount* const account)
         account->port,
         account->tls_policy,
         account->auth_policy);
-    free(jid);
 
     return result;
 }
@@ -178,17 +174,15 @@ session_connect_with_details(const char* const jid, const char* const passwd, co
     }
 
     // use 'profanity' when no resourcepart in provided jid
-    Jid* jidp = jid_create(jid);
+    auto_jid Jid* jidp = jid_create(jid);
     if (jidp->resourcepart == NULL) {
+        auto_gchar gchar* resource = jid_random_resource();
         jid_destroy(jidp);
-        char* resource = jid_random_resource();
         jidp = jid_create_from_bare_and_resource(jid, resource);
-        free(resource);
         saved_details.jid = strdup(jidp->fulljid);
     } else {
         saved_details.jid = strdup(jid);
     }
-    jid_destroy(jidp);
 
     // connect with fulljid
     log_info("Connecting without account, JID: %s", saved_details.jid);
@@ -268,12 +262,12 @@ session_process_events(void)
         if ((reconnect_sec != 0) && reconnect_timer) {
             int elapsed_sec = g_timer_elapsed(reconnect_timer, NULL);
             if (elapsed_sec > reconnect_sec) {
-                _session_reconnect();
+                session_reconnect_now();
             }
         }
         break;
     case JABBER_RECONNECT:
-        _session_reconnect();
+        session_reconnect_now();
         break;
     default:
         break;
@@ -307,7 +301,7 @@ _receive_mood(xmpp_stanza_t* const stanza, void* const userdata)
                         const char* m = xmpp_stanza_get_name(c);
                         xmpp_stanza_t* t = xmpp_stanza_get_child_by_name(mood, STANZA_NAME_TEXT);
                         if (t) {
-                            const char* text = xmpp_stanza_get_text(t);
+                            auto_char char* text = xmpp_stanza_get_text(t);
                             cons_show("Mood from %s %s (%s)", from, m, text);
                         } else {
                             cons_show("Mood from %s %s", from, m);
@@ -418,7 +412,7 @@ session_check_autoaway(void)
         return;
     }
 
-    char* mode = prefs_get_string(PREF_AUTOAWAY_MODE);
+    auto_gchar gchar* mode = prefs_get_string(PREF_AUTOAWAY_MODE);
     gboolean check = prefs_get_boolean(PREF_AUTOAWAY_CHECK);
     gint away_time = prefs_get_autoaway_time();
     gint xa_time = prefs_get_autoxa_time();
@@ -427,7 +421,7 @@ session_check_autoaway(void)
 
     char* account = session_get_account_name();
     resource_presence_t curr_presence = accounts_get_last_presence(account);
-    char* curr_status = accounts_get_last_status(account);
+    auto_char char* curr_status = accounts_get_last_status(account);
 
     unsigned long idle_ms = ui_get_idle_time();
 
@@ -450,7 +444,7 @@ session_check_autoaway(void)
                     }
 
                     // send away presence with last activity
-                    char* message = prefs_get_string(PREF_AUTOAWAY_MESSAGE);
+                    auto_gchar gchar* message = prefs_get_string(PREF_AUTOAWAY_MESSAGE);
                     connection_set_presence_msg(message);
                     if (prefs_get_boolean(PREF_LASTACTIVITY)) {
                         cl_ev_presence_send(RESOURCE_AWAY, idle_ms / 1000);
@@ -464,7 +458,6 @@ session_check_autoaway(void)
                     } else {
                         cons_show("Idle for %d minutes, status set to away (priority %d).", away_time, pri);
                     }
-                    g_free(message);
 
                     title_bar_set_presence(CONTACT_AWAY);
                 }
@@ -493,7 +486,7 @@ session_check_autoaway(void)
             activity_state = ACTIVITY_ST_XA;
 
             // send extended away presence with last activity
-            char* message = prefs_get_string(PREF_AUTOXA_MESSAGE);
+            auto_gchar gchar* message = prefs_get_string(PREF_AUTOXA_MESSAGE);
             connection_set_presence_msg(message);
             if (prefs_get_boolean(PREF_LASTACTIVITY)) {
                 cl_ev_presence_send(RESOURCE_XA, idle_ms / 1000);
@@ -507,7 +500,6 @@ session_check_autoaway(void)
             } else {
                 cons_show("Idle for %d minutes, status set to xa (priority %d).", xa_time, pri);
             }
-            g_free(message);
 
             title_bar_set_presence(CONTACT_XA);
         } else if (check && (idle_ms < away_time_ms)) {
@@ -536,9 +528,6 @@ session_check_autoaway(void)
         }
         break;
     }
-
-    free(curr_status);
-    g_free(mode);
 }
 
 static struct
@@ -557,8 +546,8 @@ session_reconnect(gchar* altdomain, unsigned short altport)
     reconnect.altport = altport;
 }
 
-static void
-_session_reconnect(void)
+void
+session_reconnect_now(void)
 {
     // reconnect with account.
     ProfAccount* account = accounts_get_account(saved_account.name);
@@ -567,7 +556,7 @@ _session_reconnect(void)
         return;
     }
 
-    char* jid = NULL;
+    auto_char char* jid = NULL;
     if (account->resource) {
         jid = create_fulljid(account->jid, account->resource);
     } else {
@@ -585,7 +574,6 @@ _session_reconnect(void)
 
     log_debug("Attempting reconnect with account %s", account->name);
     connection_connect(jid, saved_account.passwd, server, port, account->tls_policy, account->auth_policy);
-    free(jid);
     account_free(account);
     if (reconnect_timer)
         g_timer_start(reconnect_timer);
