@@ -64,14 +64,14 @@ cmd_join_shows_error_message_when_invalid_room_jid(void** state)
 void
 cmd_join_uses_account_mucservice_when_no_service_specified(void** state)
 {
-    char* account_name = "an_account";
+    gchar* account_name = g_strdup("an_account");
     char* room = "room";
     char* nick = "bob";
-    char* account_service = "conference.server.org";
+    gchar* account_service = g_strdup("conference.server.org");
     char* expected_room = "room@conference.server.org";
     gchar* args[] = { room, "nick", nick, NULL };
-    ProfAccount* account = account_new(account_name, "user@server.org", NULL, NULL,
-                                       TRUE, NULL, 0, "laptop", NULL, NULL, 0, 0, 0, 0, 0, account_service, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+    ProfAccount* account = account_new(account_name, g_strdup("user@server.org"), NULL, NULL,
+                                       TRUE, NULL, 0, g_strdup("laptop"), NULL, NULL, 0, 0, 0, 0, 0, account_service, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0);
 
     muc_init();
 
@@ -94,12 +94,12 @@ cmd_join_uses_account_mucservice_when_no_service_specified(void** state)
 void
 cmd_join_uses_supplied_nick(void** state)
 {
-    char* account_name = "an_account";
+    gchar* account_name = g_strdup("an_account");
     char* room = "room@conf.server.org";
     char* nick = "bob";
     gchar* args[] = { room, "nick", nick, NULL };
-    ProfAccount* account = account_new(account_name, "user@server.org", NULL, NULL,
-                                       TRUE, NULL, 0, "laptop", NULL, NULL, 0, 0, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+    ProfAccount* account = account_new(account_name, g_strdup("user@server.org"), NULL, NULL,
+                                       TRUE, NULL, 0, g_strdup("laptop"), NULL, NULL, 0, 0, 0, 0, 0, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0);
 
     muc_init();
 
@@ -122,12 +122,12 @@ cmd_join_uses_supplied_nick(void** state)
 void
 cmd_join_uses_account_nick_when_not_supplied(void** state)
 {
-    char* account_name = "an_account";
+    gchar* account_name = g_strdup("an_account");
     char* room = "room2@conf.server.org";
-    char* account_nick = "a_nick";
+    gchar* account_nick = g_strdup("a_nick");
     gchar* args[] = { room, NULL };
-    ProfAccount* account = account_new(account_name, "user@server.org", NULL, NULL,
-                                       TRUE, NULL, 0, "laptop", NULL, NULL, 0, 0, 0, 0, 0, NULL, account_nick, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+    ProfAccount* account = account_new(account_name, g_strdup("user@server.org"), NULL, NULL,
+                                       TRUE, NULL, 0, g_strdup("laptop"), NULL, NULL, 0, 0, 0, 0, 0, NULL, account_nick, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0);
 
     muc_init();
 
@@ -150,15 +150,15 @@ cmd_join_uses_account_nick_when_not_supplied(void** state)
 void
 cmd_join_uses_password_when_supplied(void** state)
 {
-    char* account_name = "an_account";
+    gchar* account_name = g_strdup("an_account");
     char* room = "room";
     char* password = "a_password";
-    char* account_nick = "a_nick";
-    char* account_service = "a_service";
+    gchar* account_nick = g_strdup("a_nick");
+    gchar* account_service = g_strdup("a_service");
     char* expected_room = "room@a_service";
     gchar* args[] = { room, "password", password, NULL };
-    ProfAccount* account = account_new(account_name, "user@server.org", NULL, NULL,
-                                       TRUE, NULL, 0, "laptop", NULL, NULL, 0, 0, 0, 0, 0, account_service, account_nick, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+    ProfAccount* account = account_new(account_name, g_strdup("user@server.org"), NULL, NULL,
+                                       TRUE, NULL, 0, g_strdup("laptop"), NULL, NULL, 0, 0, 0, 0, 0, account_service, account_nick, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, 0);
 
     muc_init();
 
