@@ -490,7 +490,8 @@ mucwin_unset_message_char(ProfMucWin* mucwin)
 
 void win_update_entry_message(ProfWin* window, const char* const id, const char* const message){};
 void win_mark_received(ProfWin* window, const char* const id){};
-void win_print_http_transfer(ProfWin* window, const char* const message, char* url){};
+void win_print_http_transfer(ProfWin* window, const char* const message, char* id){};
+void win_print_loading_history(ProfWin* window){};
 
 void
 ui_show_roster(void)
@@ -1124,10 +1125,6 @@ cons_tray_setting(void)
 {
 }
 void
-cons_os_setting(void)
-{
-}
-void
 cons_correction_setting(void)
 {
 }
@@ -1154,6 +1151,16 @@ cons_mood_setting(void)
 {
 }
 
+void
+cons_strophe_setting(void)
+{
+}
+
+void
+cons_privacy_setting(void)
+{
+}
+
 void
 cons_show_bookmarks_ignore(gchar** list, gsize len)
 {
@@ -1256,6 +1263,11 @@ win_create_plugin(const char* const plugin_name, const char* const tag)
 {
     return NULL;
 }
+ProfWin*
+win_create_vcard(vCard* vcard)
+{
+    return NULL;
+}
 
 char*
 win_get_tab_identifier(ProfWin* window)
@@ -1356,6 +1368,10 @@ void
 win_clear(ProfWin* window)
 {
 }
+void
+win_redraw(ProfWin* window)
+{
+}
 char*
 win_to_string(ProfWin* window)
 {
