@@ -38,6 +38,10 @@ session_connect_with_account(const ProfAccount* const account)
     return mock_type(jabber_conn_status_t);
 }
 
+void
+session_reconnect_now(void)
+{
+}
 void
 session_disconnect(void)
 {
@@ -50,6 +54,10 @@ void
 session_process_events(void)
 {
 }
+void
+connection_disconnect(void)
+{
+}
 const char*
 connection_get_fulljid(void)
 {
@@ -426,6 +434,11 @@ iq_muc_register_nick(const char* const roomjid)
 {
 }
 
+void
+iq_mam_request(ProfChatWin* win, GDateTime* enddate)
+{
+}
+
 void
 publish_user_mood(const char* const mood, const char* const text)
 {
